/* 
 * event.c --
 *
 *	This file provides functions for parsing arguments
 *	and converting them to XEvent structs.
 */

/*
 * Copyright (C) 1998-2001, Neil D. McKay
 */

#include "tk.h"
#include <X11/Xproto.h>

#include "xopInt.h"

/*
 * eventNames is a list of X event names, stored as text strings.
 * This array maps an event name to a unique integer, as specified
 * by its index into this array. We'll use this same index in
 * other places, so IF YOU CHANGE THIS TABLE, MAKE SURE ALL OTHER
 * TABLES ARE ORDERED APPROPRIATELY.
 */

static char *eventNames[] = {
    "ButtonPress",	/*  0 */
    "ButtonRelease",	/*  1 */
    "CirculateNotify",	/*  2 */
    "CirculateRequest",	/*  3 */
    "ClientMessage",	/*  4 */
    "ColormapNotify",	/*  5 */
    "ConfigureNotify",	/*  6 */
    "ConfigureRequest",	/*  7 */
    "CreateNotify",	/*  8 */
    "DestroyNotify",	/*  9 */
    "EnterNotify",	/* 10 */
    "LeaveNotify",	/* 11 */
    "Expose",		/* 12 */
    "FocusIn",		/* 13 */
    "FocusOut",		/* 14 */
    "GraphicsExpose",	/* 15 */
    "NoExpose",		/* 16 */
    "GravityNotify",	/* 17 */
    "KeyPress",		/* 18 */
    "KeyRelease",	/* 19 */
    "KeymapNotify",	/* 20 */
    "MapNotify",	/* 21 */
    "UnmapNotify",	/* 22 */
    "MapRequest",	/* 23 */
    "MappingNotify",	/* 24 */
    "MotionNotify",	/* 25 */
    "PropertyNotify",	/* 26 */
    "ReparentNotify",	/* 27 */
    "ResizeRequest",	/* 28 */
    "SelectionClear",	/* 29 */
    "SelectionNotify",	/* 30 */
    "SelectionRequest",	/* 31 */
    "VisibilityNotify",	/* 32 */
    NULL
};

#define N_EVENT_TYPES ((sizeof(eventNames)/sizeof(char *)) - 1)

static long eventMaskList[N_EVENT_TYPES] = {
    ButtonPressMask,
    ButtonReleaseMask,
    StructureNotifyMask|SubstructureNotifyMask,
    SubstructureRedirectMask,
    0,
    ColormapChangeMask,
    StructureNotifyMask|SubstructureNotifyMask,
    SubstructureRedirectMask,
    SubstructureNotifyMask,
    SubstructureNotifyMask,
    EnterWindowMask,
    LeaveWindowMask,
    ExposureMask,
    FocusChangeMask,
    FocusChangeMask,
    0,
    0,
    StructureNotifyMask|SubstructureNotifyMask,
    KeyPressMask,
    KeyReleaseMask,
    KeymapStateMask,
    StructureNotifyMask|SubstructureNotifyMask,
    StructureNotifyMask|SubstructureNotifyMask,
    SubstructureRedirectMask,
    0,
    ButtonMotionMask|Button1MotionMask|Button2MotionMask
		|Button3MotionMask|Button4MotionMask|Button5MotionMask
		|Button1MotionMask|PointerMotionMask|PointerMotionHintMask,
    PropertyChangeMask,
    StructureNotifyMask|SubstructureNotifyMask,
    ResizeRedirectMask,
    0,
    0,
    0,
    VisibilityChangeMask
};

/*
 * These are procedures that set default values for event contents.
 * There is one procedure for each event type.
 */

static void
DefButtonPress(display, window, eventPtr)
    Display *display;
    Window window;
    XEvent *eventPtr;
{
    eventPtr->xbutton.type = ButtonPress;
    eventPtr->xbutton.serial = 0;
    eventPtr->xbutton.send_event = 1;
    eventPtr->xbutton.display = display;
    eventPtr->xbutton.window = window;
    eventPtr->xbutton.root = XRootWindow(display, 0);
    eventPtr->xbutton.subwindow = window;
    eventPtr->xbutton.time = (Time) 0;
    eventPtr->xbutton.x = 0;
    eventPtr->xbutton.y = 0;
    eventPtr->xbutton.x_root = 0;
    eventPtr->xbutton.y_root = 0;
    eventPtr->xbutton.state = 0;
    eventPtr->xbutton.button = 0;
    eventPtr->xbutton.same_screen = 1;
}

static void
DefButtonRelease(display, window, eventPtr)
    Display *display;
    Window window;
    XEvent *eventPtr;
{
    eventPtr->xbutton.type = ButtonRelease;
    eventPtr->xbutton.serial = 0;
    eventPtr->xbutton.send_event = 1;
    eventPtr->xbutton.display = display;
    eventPtr->xbutton.window = window;
    eventPtr->xbutton.root = XRootWindow(display, 0);
    eventPtr->xbutton.subwindow = window;
    eventPtr->xbutton.time = (Time) 0;
    eventPtr->xbutton.x = 0;
    eventPtr->xbutton.y = 0;
    eventPtr->xbutton.x_root = 0;
    eventPtr->xbutton.y_root = 0;
    eventPtr->xbutton.state = 0;
    eventPtr->xbutton.button = 0;
    eventPtr->xbutton.same_screen = 1;
}

static void
DefCirculateNotify(display, window, eventPtr)
    Display *display;
    Window window;
    XEvent *eventPtr;
{
    eventPtr->xcirculate.type = CirculateNotify;
    eventPtr->xcirculate.serial = 0;
    eventPtr->xcirculate.send_event = 1;
    eventPtr->xcirculate.display = display;
    eventPtr->xcirculate.event = window;
    eventPtr->xcirculate.window = window;
    eventPtr->xcirculate.place = PlaceOnTop;
}

static void
DefCirculateRequest(display, window, eventPtr)
    Display *display;
    Window window;
    XEvent *eventPtr;
{
    eventPtr->xcirculaterequest.type = CirculateRequest;
    eventPtr->xcirculaterequest.serial = 0;
    eventPtr->xcirculaterequest.send_event = 1;
    eventPtr->xcirculaterequest.display = display;
    eventPtr->xcirculaterequest.parent = window;
    eventPtr->xcirculaterequest.window = window;
    eventPtr->xcirculaterequest.place = PlaceOnTop;
}


static void
DefClientMessage(display, window, eventPtr)
    Display *display;
    Window window;
    XEvent *eventPtr;
{
    int i;

    eventPtr->xclient.type = ClientMessage;
    eventPtr->xclient.serial = 0;
    eventPtr->xclient.send_event = 1;
    eventPtr->xclient.display = display;
    eventPtr->xclient.window = window;
    eventPtr->xclient.message_type = (Atom) 0;
    eventPtr->xclient.format = 8;
    for(i = 0; i < 20; i++) {
	eventPtr->xclient.data.b[i] = '\0';
    }
}


static void
DefColormapNotify(display, window, eventPtr)
    Display *display;
    Window window;
    XEvent *eventPtr;
{
    eventPtr->xcolormap.type = ColormapNotify;
    eventPtr->xcolormap.serial = 0;
    eventPtr->xcolormap.send_event = 1;
    eventPtr->xcolormap.display = display;
    eventPtr->xcolormap.window = window;
    eventPtr->xcolormap.colormap = None;
    eventPtr->xcolormap.new = False;
    eventPtr->xcolormap.state = ColormapInstalled;
}

static void
DefConfigureNotify(display, window, eventPtr)
    Display *display;
    Window window;
    XEvent *eventPtr;
{
    eventPtr->xconfigure.type = ConfigureNotify;
    eventPtr->xconfigure.serial = 0;
    eventPtr->xconfigure.send_event = 1;
    eventPtr->xconfigure.display = display;
    eventPtr->xconfigure.event = window;
    eventPtr->xconfigure.window = window;
    eventPtr->xconfigure.x = 0;
    eventPtr->xconfigure.y = 0;
    eventPtr->xconfigure.width = 0;
    eventPtr->xconfigure.height = 0;
    eventPtr->xconfigure.border_width = 0;
    eventPtr->xconfigure.above = None;
    eventPtr->xconfigure.override_redirect = False;
}


static void
DefConfigureRequest(display, window, eventPtr)
    Display *display;
    Window window;
    XEvent *eventPtr;
{
    eventPtr->xconfigurerequest.type = ConfigureRequest;
    eventPtr->xconfigurerequest.serial = 0;
    eventPtr->xconfigurerequest.send_event = 1;
    eventPtr->xconfigurerequest.display = display;
    eventPtr->xconfigurerequest.parent = window;
    eventPtr->xconfigurerequest.window = window;
    eventPtr->xconfigurerequest.x = 0;
    eventPtr->xconfigurerequest.y = 0;
    eventPtr->xconfigurerequest.width = 0;
    eventPtr->xconfigurerequest.height = 0;
    eventPtr->xconfigurerequest.border_width = 0;
    eventPtr->xconfigurerequest.above = None;
    eventPtr->xconfigurerequest.detail = None;
    eventPtr->xconfigurerequest.value_mask = 0;
}

static void
DefCreateNotify(display, window, eventPtr)
    Display *display;
    Window window;
    XEvent *eventPtr;
{
    eventPtr->xcreatewindow.type = CreateNotify;
    eventPtr->xcreatewindow.serial = 0;
    eventPtr->xcreatewindow.send_event = 1;
    eventPtr->xcreatewindow.display = display;
    eventPtr->xcreatewindow.parent = window;
    eventPtr->xcreatewindow.window = window;
    eventPtr->xcreatewindow.x = 0;
    eventPtr->xcreatewindow.y = 0;
    eventPtr->xcreatewindow.width = 0;
    eventPtr->xcreatewindow.height = 0;
    eventPtr->xcreatewindow.border_width = 0;
    eventPtr->xcreatewindow.override_redirect = False;
}

static void
DefDestroyNotify(display, window, eventPtr)
    Display *display;
    Window window;
    XEvent *eventPtr;
{
    eventPtr->xdestroywindow.type = DestroyNotify;
    eventPtr->xdestroywindow.serial = 0;
    eventPtr->xdestroywindow.send_event = 1;
    eventPtr->xdestroywindow.display = display;
    eventPtr->xdestroywindow.event = window;
    eventPtr->xdestroywindow.window = window;
}

static void
DefEnterNotify(display, window, eventPtr)
    Display *display;
    Window window;
    XEvent *eventPtr;
{
    eventPtr->xcrossing.type = EnterNotify;
    eventPtr->xcrossing.serial = 0;
    eventPtr->xcrossing.send_event = 1;
    eventPtr->xcrossing.display = display;
    eventPtr->xcrossing.window = window;
    eventPtr->xcrossing.root = XRootWindow(display, 0);
    eventPtr->xcrossing.subwindow = window;
    eventPtr->xcrossing.time = (Time) 0;
    eventPtr->xcrossing.x = 0;
    eventPtr->xcrossing.y = 0;
    eventPtr->xcrossing.x_root = 0;
    eventPtr->xcrossing.y_root = 0;
    eventPtr->xcrossing.mode = NotifyNormal;
    eventPtr->xcrossing.detail = NotifyAncestor;
    eventPtr->xcrossing.same_screen = True;
    eventPtr->xcrossing.focus = False;
    eventPtr->xcrossing.state = 0;
}

static void
DefLeaveNotify(display, window, eventPtr)
    Display *display;
    Window window;
    XEvent *eventPtr;
{
    eventPtr->xcrossing.type = LeaveNotify;
    eventPtr->xcrossing.serial = 0;
    eventPtr->xcrossing.send_event = 1;
    eventPtr->xcrossing.display = display;
    eventPtr->xcrossing.window = window;
    eventPtr->xcrossing.root = XRootWindow(display, 0);
    eventPtr->xcrossing.subwindow = window;
    eventPtr->xcrossing.time = (Time) 0;
    eventPtr->xcrossing.x = 0;
    eventPtr->xcrossing.y = 0;
    eventPtr->xcrossing.x_root = 0;
    eventPtr->xcrossing.y_root = 0;
    eventPtr->xcrossing.mode = NotifyNormal;
    eventPtr->xcrossing.detail = NotifyAncestor;
    eventPtr->xcrossing.same_screen = True;
    eventPtr->xcrossing.focus = False;
    eventPtr->xcrossing.state = 0;
}

static void
DefExpose(display, window, eventPtr)
    Display *display;
    Window window;
    XEvent *eventPtr;
{
    eventPtr->xexpose.type = Expose;
    eventPtr->xexpose.serial = 0;
    eventPtr->xexpose.send_event = 1;
    eventPtr->xexpose.display = display;
    eventPtr->xexpose.window = window;
    eventPtr->xexpose.x = 0;
    eventPtr->xexpose.y = 0;
    eventPtr->xexpose.width = 0;
    eventPtr->xexpose.height = 0;
    eventPtr->xexpose.count = 0;
}

static void
DefFocusIn(display, window, eventPtr)
    Display *display;
    Window window;
    XEvent *eventPtr;
{
    eventPtr->xfocus.type = FocusIn;
    eventPtr->xfocus.serial = 0;
    eventPtr->xfocus.send_event = 1;
    eventPtr->xfocus.display = display;
    eventPtr->xfocus.window = window;
    eventPtr->xfocus.mode = NotifyNormal;
    eventPtr->xfocus.detail = NotifyAncestor;
}

static void
DefFocusOut(display, window, eventPtr)
    Display *display;
    Window window;
    XEvent *eventPtr;
{
    eventPtr->xfocus.type = FocusOut;
    eventPtr->xfocus.serial = 0;
    eventPtr->xfocus.send_event = 1;
    eventPtr->xfocus.display = display;
    eventPtr->xfocus.window = window;
    eventPtr->xfocus.mode = NotifyNormal;
    eventPtr->xfocus.detail = NotifyAncestor;
}

static void
DefGraphicsExpose(display, window, eventPtr)
    Display *display;
    Window window;
    XEvent *eventPtr;
{
    eventPtr->xgraphicsexpose.type = GraphicsExpose;
    eventPtr->xgraphicsexpose.serial = 0;
    eventPtr->xgraphicsexpose.send_event = 1;
    eventPtr->xgraphicsexpose.display = display;
    eventPtr->xgraphicsexpose.drawable = window;
    eventPtr->xgraphicsexpose.x = 0;
    eventPtr->xgraphicsexpose.y = 0;
    eventPtr->xgraphicsexpose.width = 0;
    eventPtr->xgraphicsexpose.height = 0;
    eventPtr->xgraphicsexpose.count = 0;
    eventPtr->xgraphicsexpose.major_code = X_CopyArea;
    eventPtr->xgraphicsexpose.minor_code = 0;
}

static void
DefNoExpose(display, window, eventPtr)
    Display *display;
    Window window;
    XEvent *eventPtr;
{
    eventPtr->xnoexpose.type = NoExpose;
    eventPtr->xnoexpose.serial = 0;
    eventPtr->xnoexpose.send_event = 1;
    eventPtr->xnoexpose.display = display;
    eventPtr->xnoexpose.drawable = window;
    eventPtr->xnoexpose.major_code = X_CopyArea;
    eventPtr->xnoexpose.minor_code = 0;
}

static void
DefGravityNotify(display, window, eventPtr)
    Display *display;
    Window window;
    XEvent *eventPtr;
{
    eventPtr->xgravity.type = GravityNotify;
    eventPtr->xgravity.serial = 0;
    eventPtr->xgravity.send_event = 1;
    eventPtr->xgravity.display = display;
    eventPtr->xgravity.event = window;
    eventPtr->xgravity.window = window;
    eventPtr->xgravity.x = 0;
    eventPtr->xgravity.y = 0;
}

static void
DefKeyPress(display, window, eventPtr)
    Display *display;
    Window window;
    XEvent *eventPtr;
{
    eventPtr->xkey.type = KeyPress;
    eventPtr->xkey.serial = 0;
    eventPtr->xkey.send_event = 1;
    eventPtr->xkey.display = display;
    eventPtr->xkey.window = window;
    eventPtr->xkey.root = XRootWindow(display, 0);
    eventPtr->xkey.subwindow = window;
    eventPtr->xkey.time = (Time) 0;
    eventPtr->xkey.x = 0;
    eventPtr->xkey.y = 0;
    eventPtr->xkey.x_root = 0;
    eventPtr->xkey.y_root = 0;
    eventPtr->xkey.state = 0;
    eventPtr->xkey.keycode = 0;
    eventPtr->xkey.same_screen = True;
}

static void
DefKeyRelease(display, window, eventPtr)
    Display *display;
    Window window;
    XEvent *eventPtr;
{
    eventPtr->xkey.type = KeyRelease;
    eventPtr->xkey.serial = 0;
    eventPtr->xkey.send_event = 1;
    eventPtr->xkey.display = display;
    eventPtr->xkey.window = window;
    eventPtr->xkey.root = XRootWindow(display, 0);
    eventPtr->xkey.subwindow = window;
    eventPtr->xkey.time = (Time) 0;
    eventPtr->xkey.x = 0;
    eventPtr->xkey.y = 0;
    eventPtr->xkey.x_root = 0;
    eventPtr->xkey.y_root = 0;
    eventPtr->xkey.state = 0;
    eventPtr->xkey.keycode = 0;
    eventPtr->xkey.same_screen = True;
}

static void
DefKeymapNotify(display, window, eventPtr)
    Display *display;
    Window window;
    XEvent *eventPtr;
{
    int i;

    eventPtr->xkeymap.type = KeymapNotify;
    eventPtr->xkeymap.serial = 0;
    eventPtr->xkeymap.send_event = 1;
    eventPtr->xkeymap.display = display;
    eventPtr->xkeymap.window = window;
    for (i = 0; i < 32; i++) {
	eventPtr->xkeymap.key_vector[i] = '\0';
    }
}

static void
DefMapNotify(display, window, eventPtr)
    Display *display;
    Window window;
    XEvent *eventPtr;
{
    eventPtr->xmap.type = MapNotify;
    eventPtr->xmap.serial = 0;
    eventPtr->xmap.send_event = 1;
    eventPtr->xmap.display = display;
    eventPtr->xmap.event = window;
    eventPtr->xmap.window = window;
    eventPtr->xmap.override_redirect = False;
}

static void
DefUnmapNotify(display, window, eventPtr)
    Display *display;
    Window window;
    XEvent *eventPtr;
{
    eventPtr->xunmap.type = UnmapNotify;
    eventPtr->xunmap.serial = 0;
    eventPtr->xunmap.send_event = 1;
    eventPtr->xunmap.display = display;
    eventPtr->xunmap.event = window;
    eventPtr->xunmap.window = window;
    eventPtr->xunmap.from_configure = False;
}

static void
DefMapRequest(display, window, eventPtr)
    Display *display;
    Window window;
    XEvent *eventPtr;
{
    eventPtr->xmaprequest.type = MapRequest;
    eventPtr->xmaprequest.serial = 0;
    eventPtr->xmaprequest.send_event = 1;
    eventPtr->xmaprequest.display = display;
    eventPtr->xmaprequest.parent = window;
    eventPtr->xmaprequest.window = window;
}

static void
DefMappingNotify(display, window, eventPtr)
    Display *display;
    Window window;
    XEvent *eventPtr;
{
    eventPtr->xmapping.type = MappingNotify;
    eventPtr->xmapping.serial = 0;
    eventPtr->xmapping.send_event = 1;
    eventPtr->xmapping.display = display;
    eventPtr->xmapping.window = window;
    eventPtr->xmapping.request = MappingKeyboard;
    eventPtr->xmapping.first_keycode = 0;
    eventPtr->xmapping.count = 0;
}

static void
DefMotionNotify(display, window, eventPtr)
    Display *display;
    Window window;
    XEvent *eventPtr;
{
    eventPtr->xmotion.type = MotionNotify;
    eventPtr->xmotion.serial = 0;
    eventPtr->xmotion.send_event = 1;
    eventPtr->xmotion.display = display;
    eventPtr->xmotion.window = window;
    eventPtr->xmotion.root = XRootWindow(display, 0);
    eventPtr->xmotion.subwindow = window;
    eventPtr->xmotion.time = (Time) 0;
    eventPtr->xmotion.x = 0;
    eventPtr->xmotion.y = 0;
    eventPtr->xmotion.x_root = 0;
    eventPtr->xmotion.y_root = 0;
    eventPtr->xmotion.state = 0;
    eventPtr->xmotion.is_hint = 0;
    eventPtr->xmotion.same_screen = True;
}

static void
DefPropertyNotify(display, window, eventPtr)
    Display *display;
    Window window;
    XEvent *eventPtr;
{
    eventPtr->xproperty.type = PropertyNotify;
    eventPtr->xproperty.serial = 0;
    eventPtr->xproperty.send_event = 1;
    eventPtr->xproperty.display = display;
    eventPtr->xproperty.window = window;
    eventPtr->xproperty.atom = (Atom) 0;
    eventPtr->xproperty.time = (Time) 0;
    eventPtr->xproperty.state = PropertyNewValue;
}

static void
DefReparentNotify(display, window, eventPtr)
    Display *display;
    Window window;
    XEvent *eventPtr;
{
    eventPtr->xreparent.type = ReparentNotify;
    eventPtr->xreparent.serial = 0;
    eventPtr->xreparent.send_event = 1;
    eventPtr->xreparent.display = display;
    eventPtr->xreparent.event = window;
    eventPtr->xreparent.window = window;
    eventPtr->xreparent.parent = window;
    eventPtr->xreparent.x = 0;
    eventPtr->xreparent.y = 0;
    eventPtr->xreparent.override_redirect = False;
}

static void
DefResizeRequest(display, window, eventPtr)
    Display *display;
    Window window;
    XEvent *eventPtr;
{
    eventPtr->xresizerequest.type = ResizeRequest;
    eventPtr->xresizerequest.serial = 0;
    eventPtr->xresizerequest.send_event = 1;
    eventPtr->xresizerequest.display = display;
    eventPtr->xresizerequest.window = window;
    eventPtr->xresizerequest.width = 0;
    eventPtr->xresizerequest.height = 0;
}

static void
DefSelectionClear(display, window, eventPtr)
    Display *display;
    Window window;
    XEvent *eventPtr;
{
    eventPtr->xselectionclear.type = SelectionClear;
    eventPtr->xselectionclear.serial = 0;
    eventPtr->xselectionclear.send_event = 1;
    eventPtr->xselectionclear.display = display;
    eventPtr->xselectionclear.window = window;
    eventPtr->xselectionclear.selection = (Atom) 0;
    eventPtr->xselectionclear.time = (Time) 0;
}

static void
DefSelectionNotify(display, window, eventPtr)
    Display *display;
    Window window;
    XEvent *eventPtr;
{
    eventPtr->xselection.type = SelectionNotify;
    eventPtr->xselection.serial = 0;
    eventPtr->xselection.send_event = 1;
    eventPtr->xselection.display = display;
    eventPtr->xselection.requestor = window;
    eventPtr->xselection.selection = (Atom) 0;
    eventPtr->xselection.target = (Atom) 0;
    eventPtr->xselection.property = (Atom) None;
    eventPtr->xselection.time = (Time) 0;
}

static void
DefSelectionRequest(display, window, eventPtr)
    Display *display;
    Window window;
    XEvent *eventPtr;
{
    eventPtr->xselectionrequest.type = SelectionRequest;
    eventPtr->xselectionrequest.serial = 0;
    eventPtr->xselectionrequest.send_event = 1;
    eventPtr->xselectionrequest.display = display;
    eventPtr->xselectionrequest.owner = window;
    eventPtr->xselectionrequest.requestor = window;
    eventPtr->xselectionrequest.selection = (Atom) 0;
    eventPtr->xselectionrequest.target = (Atom) 0;
    eventPtr->xselectionrequest.property = (Atom) None;
    eventPtr->xselectionrequest.time = (Time) 0;
}

static void
DefVisibilityNotify(display, window, eventPtr)
    Display *display;
    Window window;
    XEvent *eventPtr;
{
    eventPtr->xvisibility.type = VisibilityNotify;
    eventPtr->xvisibility.serial = 0;
    eventPtr->xvisibility.send_event = 1;
    eventPtr->xvisibility.display = display;
    eventPtr->xvisibility.window = window;
    eventPtr->xvisibility.state = VisibilityFullyObscured;
}

/*
 * Type DefEventProcPtr is used for pointers to procedures
 * that set default values in an XEvent structure.
 */

typedef void (*DefEventProcPtr) _ANSI_ARGS_((Display *display,
					Window window, XEvent *eventPtr));

/*
 * defEventProcList is an array of procedures for setting default
 * values in XEvent structures. It is indexed in the same way as the
 * eventNames array. NOTE: THE ORDER OF THIS ARRAY MUST MATCH THE
 * ORDER OF THE eventNames ARRAY!
 */

static DefEventProcPtr defEventProcList[N_EVENT_TYPES] = {
    DefButtonPress,
    DefButtonRelease,
    DefCirculateNotify,
    DefCirculateRequest,
    DefClientMessage,
    DefColormapNotify,
    DefConfigureNotify,
    DefConfigureRequest,
    DefCreateNotify,
    DefDestroyNotify,
    DefEnterNotify,
    DefLeaveNotify,
    DefExpose,
    DefFocusIn,
    DefFocusOut,
    DefGraphicsExpose,
    DefNoExpose,
    DefGravityNotify,
    DefKeyPress,
    DefKeyRelease,
    DefKeymapNotify,
    DefMapNotify,
    DefUnmapNotify,
    DefMapRequest,
    DefMappingNotify,
    DefMotionNotify,
    DefPropertyNotify,
    DefReparentNotify,
    DefResizeRequest,
    DefSelectionClear,
    DefSelectionNotify,
    DefSelectionRequest,
    DefVisibilityNotify
};

/*
 * eventOptNames contains an ordered list of event option names.
 * These are the names of the options we'll use to specify an
 * XEvent struct.
 */

static char *eventOptNames[] = {
	"-above",
	"-atom",
	"-borderwidth",
	"-button",
	"-colormap",
	"-count",
	"-data",
	"-detail",
	"-drawable",
	"-event",
	"-firstkey",
	"-focus",
	"-format",
	"-fromconfigure",
	"-height",
	"-ishint",
	"-keycode",
	"-keyvector",
	"-majorcode",
	"-messagetype",
	"-minorcode",
	"-mode",
	"-new",
	"-overrideredirect",
	"-owner",
	"-parent",
	"-place",
	"-property",
	"-request",
	"-requestor",
	"-root",
	"-samescreen",
	"-selection",
	"-sendevent",
	"-serial",
	"-state",
	"-subwindow",
	"-target",
	"-time",
	"-width",
	"-window",
	"-x",
	"-xroot",
	"-y",
	"-yroot",
	NULL
};

#define	N_EVENT_OPTS (((sizeof(eventOptNames))/sizeof(char *)) - 1)

/*
 * Type EventParseFuncPtr is a pointer to a function for parsing
 * event options. These functions accept five arguments: a pointer
 * to Tcl_Interp, a pointer to an X display structure, a string,
 * a pointer to an XEvent structure, and an offset (in bytes) into
 * the XEvent structure. The last two arguments are used to compute
 * the location where the result will be put.
 */

typedef int (*EventParseFuncPtr) _ANSI_ARGS_((Tcl_Interp *interp,
		Display *display, char *str, XEvent *eventPtr, int offset));

/*
 * Next, we define a number of functions that we'll use for parsing
 * event options. Many of these are generic, and are just wrappers
 * for standard Tcl functions like Tcl_GetInt; others are custom
 * code for a particular X event.
 */

/*
 *----------------------------------------------------------------------
 *
 * ParseEventInt --
 *
 *	ParseEventInt parses an integer. This is just a wrapper
 *	function for Tcl_GetInt.
 *
 * Results:
 *	Returns a standard Tcl result.
 *
 * Side effects:
 *	None.
 *
 *----------------------------------------------------------------------
 */

static int
ParseEventInt(interp, display, str, eventPtr, offset)
    Tcl_Interp *interp;
    Display *display;
    char *str;
    XEvent *eventPtr;
    int offset;
{
    int *intPtr;

    intPtr = (int *) (((char *) eventPtr) + offset);
    return(Tcl_GetInt(interp, str, intPtr));
}

/*
 *----------------------------------------------------------------------
 *
 * ParseEventBool --
 *
 *	ParseEventBool parses a Boolean value. This is just a wrapper
 *	function for Tcl_GetBoolean.
 *
 * Results:
 *	Returns a standard Tcl result.
 *
 * Side effects:
 *	None.
 *
 *----------------------------------------------------------------------
 */

static int
ParseEventBool(interp, display, str, eventPtr, offset)
    Tcl_Interp *interp;
    Display *display;
    char *str;
    XEvent *eventPtr;
    int offset;
{
    int *boolPtr;

    boolPtr = (int *) (((char *) eventPtr) + offset);
    return(Tcl_GetBoolean(interp, str, boolPtr));
}

/*
 *----------------------------------------------------------------------
 *
 * ParseEventWindowId --
 *
 *	ParseEventWindowId parses a window ID and checks to
 *	see that the window exists.
 *
 * Results:
 *	Returns a standard Tcl result.
 *
 * Side effects:
 *	None.
 *
 *----------------------------------------------------------------------
 */

static int
ParseEventWindowId(interp, display, str, eventPtr, offset)
    Tcl_Interp *interp;
    Display *display;
    char *str;
    XEvent *eventPtr;
    int offset;
{
    Window *windowPtr;

    windowPtr = (Window *) (((char *) eventPtr) + offset);
    return(XopGetAndCheckWindowId(interp, display, str, windowPtr));
}

/*
 *----------------------------------------------------------------------
 *
 * ParseEventDrawableId --
 *
 *	ParseEventDrawableId parses a drawable ID and checks to
 *	see that the drawable exists.
 *
 * Results:
 *	Returns a standard Tcl result.
 *
 * Side effects:
 *	None.
 *
 *----------------------------------------------------------------------
 */

static int
ParseEventDrawableId(interp, display, str, eventPtr, offset)
    Tcl_Interp *interp;
    Display *display;
    char *str;
    XEvent *eventPtr;
    int offset;
{
    Drawable *drawablePtr;

    drawablePtr = (Drawable *) (((char *) eventPtr) + offset);
    return(XopGetAndCheckDrawableId(interp, display, str, drawablePtr));
}

/*
 *----------------------------------------------------------------------
 *
 * ParseEventWindowOrNone --
 *
 *	ParseEventWindowOrNone parses a window ID and checks to
 *	see that the window exists; it also accepts "none" as
 *	an argument.
 *
 * Results:
 *	Returns a standard Tcl result.
 *
 * Side effects:
 *	None.
 *
 *----------------------------------------------------------------------
 */

static int
ParseEventWindowOrNone(interp, display, str, eventPtr, offset)
    Tcl_Interp *interp;
    Display *display;
    char *str;
    XEvent *eventPtr;
    int offset;
{
    Window *windowPtr;

    windowPtr = (Window *) (((char *) eventPtr) + offset);

    if (strncasecmp(str, "none", strlen(str)) == 0) {
	*windowPtr = (Window) None;
	return TCL_OK;
    }

    return(XopGetAndCheckWindowId(interp, display, str, windowPtr));
}

/*
 *----------------------------------------------------------------------
 *
 * ParseEventModMask --
 *
 *	ParseEventModMask parses a modifier mask, i.e. a set
 *	of mouse and keyboard-modifier buttons.
 *
 * Results:
 *	Returns a standard Tcl result.
 *
 * Side effects:
 *	None.
 *
 *----------------------------------------------------------------------
 */

static struct {
    char *name;
    unsigned int mask;
} pointerModList[] = {
    {"Shift",   ShiftMask},
    {"Lock",    LockMask},
    {"Control", ControlMask},
    {"Mod1",    Mod1Mask},
    {"Mod2",    Mod2Mask},
    {"Mod3",    Mod3Mask},
    {"Mod4",    Mod4Mask},
    {"Mod5",    Mod5Mask},
    {"Button1", Button1Mask},
    {"Button2", Button2Mask},
    {"Button3", Button3Mask},
    {"Button4", Button4Mask},
    {"Button5", Button5Mask},
    {NULL,      0}
};

static int
ParseEventModMask(interp, display, str, eventPtr, offset)
    Tcl_Interp *interp;
    Display *display;
    char *str;
    XEvent *eventPtr;
    int offset;
{
    int i, j, argc;
    char **argv;
    int mask, matched;
    int *modMaskPtr;

    if (Tcl_SplitList(interp, str, &argc, &argv) != TCL_OK) {
	return TCL_ERROR;
    }

    mask = 0;
    for (i = 0; i < argc; i++) {
	matched = 0;
	for (j = 0; pointerModList[j].name != NULL; j++) {
	    if (strcmp(argv[i], pointerModList[j].name) == 0) {
		mask |= pointerModList[j].mask;
		matched = 1;
		break;
	    }
	}
	if (!matched) {
	    Tcl_ResetResult(interp);
	    Tcl_AppendResult(interp, "Unknown modifier \"",
			argv[i], "\"", (char *) 0);
	    Tcl_Free((char *) argv);
	    return TCL_ERROR;
	}
    }

    Tcl_Free((char *) argv);
    modMaskPtr = (int *) (((char *) eventPtr) + offset);
    *modMaskPtr = mask;
    return TCL_OK;
}

/*
 *----------------------------------------------------------------------
 *
 * ParseButtonName --
 *
 *	ParseButtonName parses a button name and produces an
 *	appropriate integer.
 *
 * Results:
 *	Returns a standard Tcl result.
 *
 * Side effects:
 *	None.
 *
 *----------------------------------------------------------------------
 */

static int intToButton[] = {
    0, Button1, Button2, Button3, Button4, Button5
};

typedef struct {
    char *name;
    int button;
} NameToButtonMapEntry;

NameToButtonMapEntry nameToButton[] = {
    {"button1", Button1},
    {"b1", Button1},
    {"button2", Button2},
    {"b2", Button2},
    {"button3", Button3},
    {"b3", Button3},
    {"button4", Button4},
    {"b4", Button4},
    {"button5", Button5},
    {"b5", Button5},
    {NULL, 0}
};

static int
ParseButtonName(interp, display, str, eventPtr, offset)
    Tcl_Interp *interp;
    Display *display;
    char *str;
    XEvent *eventPtr;
    int offset;
{
    int i, buttonInt;
    int *buttonPtr;

    buttonPtr = (int *) (((char *) eventPtr) + offset);

    /* Reject the null string. */

    if (str[0] == '\0') {
	Tcl_SetResult(interp,
		"Empty string is not a valid button name", TCL_STATIC);
	return TCL_ERROR;
    }

    /* See if we have an integer. */

    if (Tcl_GetInt(interp, str, &buttonInt) == TCL_OK) {
	if(buttonInt < 1 || buttonInt > 5) {
	    Tcl_SetResult(interp,
			"Button number must be between 1 and 5", TCL_STATIC);
	    return TCL_ERROR;
	}

	*buttonPtr = intToButton[buttonInt];
	return TCL_OK;
    }

    /*
     * Not an integer; try "left", "middle", and "right", in
     * any case, with any abbreviation.
     */

    if (strncasecmp(str, "left", strlen(str)) == 0) {
	*buttonPtr = Button1;
	return TCL_OK;
    }

    if (strncasecmp(str, "middle", strlen(str)) == 0) {
	*buttonPtr = Button2;
	return TCL_OK;
    }

    if (strncasecmp(str, "right", strlen(str)) == 0) {
	*buttonPtr = Button3;
	return TCL_OK;
    }

    /*
     * Try all the other reasonable possibilities.
     */

    for (i = 0; nameToButton[i].name != NULL; i++) {
	if(strcasecmp(str, nameToButton[i].name) == 0) {
	    *buttonPtr = nameToButton[i].button;
	    return TCL_OK;
	}
    }

    /* Nope... */

    Tcl_ResetResult(interp);
    Tcl_AppendResult(interp, "\"", str,
		"\" is not a valid button name", (char *) 0);
    return TCL_ERROR;
}

/*
 *----------------------------------------------------------------------
 *
 * ParseEventPlace --
 *
 *	ParseEventPlace parses a "place" option, producing either
 *	"PlaceOnTop" or "PlaceOnBottom".
 *
 * Results:
 *	Returns a standard Tcl result.
 *
 * Side effects:
 *	None.
 *
 *----------------------------------------------------------------------
 */

static int
ParseEventPlace(interp, display, str, eventPtr, offset)
    Tcl_Interp *interp;
    Display *display;
    char *str;
    XEvent *eventPtr;
    int offset;
{
    int *placePtr;

    placePtr = (int *) (((char *) eventPtr) + offset);

    /* Skip a leading "on". */

    if (strlen(str) >= 2 && strncasecmp(str, "on", 2) == 0) {
	str += 2;
    }

    /* Check for "top" and "bottom". */

    if (strncasecmp(str, "top", strlen(str)) == 0) {
	*placePtr = PlaceOnTop;
	return TCL_OK;
    }

    if (strncasecmp(str, "bottom", strlen(str)) == 0) {
	*placePtr = PlaceOnTop;
	return TCL_OK;
    }

    Tcl_ResetResult(interp);
    Tcl_AppendResult(interp, "Unknown place \"", str,
		"\"; should be \"top\" or \"bottom\"", (char *) 0);
    return TCL_ERROR;
}

/*
 *----------------------------------------------------------------------
 *
 * ParseEventAtom --
 *
 *	ParseEventAtom parses an atom number.
 *
 * Results:
 *	Returns a standard Tcl result.
 *
 * Side effects:
 *	None.
 *
 *----------------------------------------------------------------------
 */

static int
ParseEventAtom(interp, display, str, eventPtr, offset)
    Tcl_Interp *interp;
    Display *display;
    char *str;
    XEvent *eventPtr;
    int offset;
{
    Atom *atomPtr;

    atomPtr = (Atom *) (((char *) eventPtr) + offset);
    return(Tcl_GetInt(interp, str, (int *) atomPtr));
}

/*
 *----------------------------------------------------------------------
 *
 * ParseEventAtomOrNone --
 *
 *	ParseEventAtomOrNone parses an atom number. It also accepts
 *	the empty string or the string "none", which it translates to
 *	"None".
 *
 * Results:
 *	Returns a standard Tcl result.
 *
 * Side effects:
 *	None.
 *
 *----------------------------------------------------------------------
 */

static int
ParseEventAtomOrNone(interp, display, str, eventPtr, offset)
    Tcl_Interp *interp;
    Display *display;
    char *str;
    XEvent *eventPtr;
    int offset;
{
    Atom atom, *atomPtr;

    atomPtr = (Atom *) (((char *) eventPtr) + offset);
    if (strlen(str) == 0 || strcasecmp(str, "none") == 0) {
	*atomPtr = None;
	return TCL_OK;
    }

    return(Tcl_GetInt(interp, str, (int *) atomPtr));
}

/*
 *----------------------------------------------------------------------
 *
 * ParseELFMode --
 *
 *	ParseELFMode parses the "mode" entry for EnterNotify,
 *	LeaveNotify, FocusIn, and FocusOut events.
 *
 * Results:
 *	Returns a standard Tcl result.
 *
 * Side effects:
 *	None.
 *
 *----------------------------------------------------------------------
 */

static int
ParseELFMode(interp, display, str, eventPtr, offset)
    Tcl_Interp *interp;
    Display *display;
    char *str;
    XEvent *eventPtr;
    int offset;
{
    int *modePtr;

    modePtr = (int *) (((char *) eventPtr) + offset);

    /* Skip a leading "notify" prefix. */

    if (strncasecmp(str, "notify", 6) == 0) {
	str += 6;
    }

    /* Check out the string... */

    if (strncasecmp(str, "normal", strlen(str)) == 0) {
	*modePtr = NotifyNormal;
	return TCL_OK;
    }

    if (strncasecmp(str, "grab", strlen(str)) == 0) {
	*modePtr = NotifyGrab;
	return TCL_OK;
    }

    if (strncasecmp(str, "ungrab", strlen(str)) == 0) {
	*modePtr = NotifyUngrab;
	return TCL_OK;
    }

    Tcl_SetResult(interp,
	    "Mode must be normal, grab, or ungrab", TCL_STATIC);
    return TCL_ERROR;
}

/*
 *----------------------------------------------------------------------
 *
 * ParseELFDetail --
 *
 *	ParseELFDetail parses the "detail" entry for EnterNotify,
 *	LeaveNotify, FocusIn, and FocusOut events.
 *
 * Results:
 *	Returns a standard Tcl result.
 *
 * Side effects:
 *	None.
 *
 *----------------------------------------------------------------------
 */

static int
ParseELFDetail(interp, display, str, eventPtr, offset)
    Tcl_Interp *interp;
    Display *display;
    char *str;
    XEvent *eventPtr;
    int offset;
{
    int *detailPtr;

    detailPtr = (int *) (((char *) eventPtr) + offset);

    /* Skip a leading "notify" prefix. */

    if (strncasecmp(str, "notify", 6) == 0) {
	str += 6;
    }

    /*
     * First we have to handle "Nonlinear",
     * since it is a prefix of "NonlinearVirtual".
     */

    if (strncasecmp(str, "nonlinear", strlen(str)) == 0) {
	*detailPtr = NotifyNonlinear;
	return TCL_OK;
    }

    /* Now we can handle all the rest... */

    if (strncasecmp(str, "ancestor", strlen(str)) == 0) {
	*detailPtr = NotifyAncestor;
	return TCL_OK;
    }

    if (strncasecmp(str, "inferior", strlen(str)) == 0) {
	*detailPtr = NotifyInferior;
	return TCL_OK;
    }

    if (strncasecmp(str, "nonlinearvirtual", strlen(str)) == 0) {
	*detailPtr = NotifyNonlinearVirtual;
	return TCL_OK;
    }

    if (strncasecmp(str, "virtual", strlen(str)) == 0) {
	*detailPtr = NotifyVirtual;
	return TCL_OK;
    }

    Tcl_SetResult(interp,
	    "Detail must be ancestor, inferior, nonlinear, nonlinearvirtual, or virtual", TCL_STATIC);
    return TCL_ERROR;
}

/*
 * Now we define some non-universal, event-specific
 * parsing functions.
 */

/*
 *----------------------------------------------------------------------
 *
 * ParseClientMsgFmt --
 *
 *	ParseClientMsgFmt parses the "format" field for
 *	a ClientMessage event.
 *
 * Results:
 *	Returns a standard Tcl result.
 *
 * Side effects:
 *	None.
 *
 *----------------------------------------------------------------------
 */

static int
ParseClientMsgFmt(interp, display, str, eventPtr, offset)
    Tcl_Interp *interp;
    Display *display;
    char *str;
    XEvent *eventPtr;
    int offset;
{
    int i, *intPtr;

    intPtr = (int *) (((char *) eventPtr) + offset);

    if(Tcl_GetInt(interp, str, &i) != TCL_OK) {
	return TCL_ERROR;
    }

    if( i != 8 && i != 16 && i != 32) {
	Tcl_SetResult(interp,
		"ClientMessage format must be 8, 16, or 32", TCL_STATIC);
	return TCL_ERROR;
    }

    *intPtr = i;
    return TCL_OK;
}

/*
 *----------------------------------------------------------------------
 *
 * ParseClientMsgData --
 *
 *	ParseClientMsgData parses the "data" field for
 *	a ClientMessage event.
 *
 * Results:
 *	Returns a standard Tcl result.
 *
 * Side effects:
 *	None.
 *
 *----------------------------------------------------------------------
 */

static int
ParseClientMsgData(interp, display, str, eventPtr, offset)
    Tcl_Interp *interp;
    Display *display;
    char *str;
    XEvent *eventPtr;
    int offset;
{
    int i, j;
    int maxListLen, maxItemVal, format;
    char *dataPtr;
    int argc;
    char **argv;
    char junkStr[32];

    /* Check the data format. */

    format = eventPtr->xclient.format;
    switch(format) {
	case 8:
		maxListLen = 20;
		maxItemVal = 0xff;
		break;

	case 16:
		maxListLen = 10;
		maxItemVal = 0xffff;
		break;

	case 32:
		maxListLen = 5;
		maxItemVal = -1;
		break;

	default:
		Tcl_SetResult(interp,
			"format must be 8, 16, or 32", TCL_STATIC);
		return TCL_ERROR;
		break;
    }

    /* The data must be a list; split it up. */

    if (Tcl_SplitList(interp, str, &argc, &argv) != TCL_OK) {
	return TCL_ERROR;
    }

    /* Make sure we don't have too many elements. */

    if (argc > maxListLen) {
	Tcl_Free((char *) argv);
	Tcl_ResetResult(interp);
	Tcl_AppendResult(interp, "Data with format \"", (char *) 0);
	sprintf(junkStr, "%1d", format);
	Tcl_AppendResult(interp, junkStr, "\" must have at most ", (char *) 0);
	sprintf(junkStr, "%1d", maxListLen);
	Tcl_AppendResult(interp, junkStr, " elements", (char *) 0);
	return TCL_ERROR;
    }

    /* Convert the integers. */

    for(i = 0; i < argc; i++) {
	if(Tcl_GetInt(interp, argv[i], &j) != TCL_OK) {
	    Tcl_Free((char *) argv);
	    return TCL_ERROR;
	}

	if (maxItemVal > 0 && (j < 0 || j > maxItemVal)) {
	    Tcl_Free((char *) argv);
	    Tcl_ResetResult(interp);
	    Tcl_AppendResult(interp, "Data with format \"", (char *) 0);
	    sprintf(junkStr, "%1d", format);
	    Tcl_AppendResult(interp, junkStr,
			"\" must be >= 0 and <= ", (char *) 0);
	    sprintf(junkStr, "%1d", maxItemVal);
	    Tcl_AppendResult(interp, junkStr, (char *) 0);
	    return TCL_ERROR;
	}

	switch(format) {
	    case 8:
		eventPtr->xclient.data.b[i] = j;
		break;

	    case 16:
		eventPtr->xclient.data.s[i] = j;
		break;

	    case 32:
		eventPtr->xclient.data.l[i] = j;
		break;
	}
    }

    return TCL_OK;
}

/*
 *----------------------------------------------------------------------
 *
 * ParseColormap --
 *
 *	ParseColormap parses the "colormap" field for
 *	a ColormapNotify event.
 *
 * Results:
 *	Returns a standard Tcl result.
 *
 * Side effects:
 *	None.
 *
 *----------------------------------------------------------------------
 */

static int
ParseColormap(interp, display, str, eventPtr, offset)
    Tcl_Interp *interp;
    Display *display;
    char *str;
    XEvent *eventPtr;
    int offset;
{
    Colormap *colormapPtr;

    colormapPtr = (Colormap *) (((char *) eventPtr) + offset);
    return(XopGetAndCheckColormapId(interp, display, str, colormapPtr));
}

/*
 *----------------------------------------------------------------------
 *
 * ParseColormapState --
 *
 *	ParseColormapState parses the "state" field for
 *	a ColormapNotify event.
 *
 * Results:
 *	Returns a standard Tcl result.
 *
 * Side effects:
 *	None.
 *
 *----------------------------------------------------------------------
 */

static int
ParseColormapState(interp, display, str, eventPtr, offset)
    Tcl_Interp *interp;
    Display *display;
    char *str;
    XEvent *eventPtr;
    int offset;
{
    int *statePtr;

    if (strlen(str) == 0) {
	Tcl_SetResult(interp,
		"\"state\" entry must not be null string", TCL_STATIC);
	return TCL_ERROR;
    }

    statePtr = (int *) (((char *) eventPtr) + offset);

    if (strncasecmp(str, "installed", strlen(str)) == 0) {
	*statePtr = ColormapInstalled;
	return TCL_OK;
    }

    if (strncasecmp(str, "uninstalled", strlen(str)) == 0) {
	*statePtr = ColormapUninstalled;
	return TCL_OK;
    }

    Tcl_SetResult(interp,
	    "\"state\" entry must be \"installed\" or \"uninstalled\"",
								TCL_STATIC);
    return TCL_ERROR;
}

/*
 *----------------------------------------------------------------------
 *
 * ParseConfigReqAbove --
 *
 *	ParseConfigReqAbove parses the "above" field for
 *	a ConfigureRequest event.
 *
 * Results:
 *	Returns a standard Tcl result.
 *
 * Side effects:
 *	None.
 *
 *----------------------------------------------------------------------
 */

static int
ParseConfigReqAbove(interp, display, str, eventPtr, offset)
    Tcl_Interp *interp;
    Display *display;
    char *str;
    XEvent *eventPtr;
    int offset;
{
    Window *abovePtr;

    abovePtr = (Window *) (((char *) eventPtr) + offset);

    if (strcasecmp(str, "above") == 0) {
	*abovePtr = Above;
	return TCL_OK;
    }

    if(XopGetAndCheckWindowId(interp, display, str, abovePtr) != TCL_OK) {
	return TCL_ERROR;
    }

    eventPtr->xconfigurerequest.value_mask |= CWSibling;
    return(TCL_OK);
}

/*
 *----------------------------------------------------------------------
 *
 * ParseConfigReqX --
 *
 *	ParseConfigReqX parses the "x" field for
 *	a ConfigureRequest event.
 *
 * Results:
 *	Returns a standard Tcl result.
 *
 * Side effects:
 *	None.
 *
 *----------------------------------------------------------------------
 */

static int
ParseConfigReqX(interp, display, str, eventPtr, offset)
    Tcl_Interp *interp;
    Display *display;
    char *str;
    XEvent *eventPtr;
    int offset;
{
    int *xPtr;

    xPtr = (int *) (((char *) eventPtr) + offset);
    if(Tcl_GetInt(interp, str, xPtr) != TCL_OK) {
	return TCL_ERROR;
    }

    eventPtr->xconfigurerequest.value_mask |= CWX;
    return(TCL_OK);
}

/*
 *----------------------------------------------------------------------
 *
 * ParseConfigReqY --
 *
 *	ParseConfigReqY parses the "y" field for
 *	a ConfigureRequest event.
 *
 * Results:
 *	Returns a standard Tcl result.
 *
 * Side effects:
 *	None.
 *
 *----------------------------------------------------------------------
 */

static int
ParseConfigReqY(interp, display, str, eventPtr, offset)
    Tcl_Interp *interp;
    Display *display;
    char *str;
    XEvent *eventPtr;
    int offset;
{
    int *yPtr;

    yPtr = (int *) (((char *) eventPtr) + offset);
    if(Tcl_GetInt(interp, str, yPtr) != TCL_OK) {
	return TCL_ERROR;
    }

    eventPtr->xconfigurerequest.value_mask |= CWY;
    return(TCL_OK);
}

/*
 *----------------------------------------------------------------------
 *
 * ParseConfigReqWidth --
 *
 *	ParseConfigReqWidth parses the "width" field for
 *	a ConfigureRequest event.
 *
 * Results:
 *	Returns a standard Tcl result.
 *
 * Side effects:
 *	None.
 *
 *----------------------------------------------------------------------
 */

static int
ParseConfigReqWidth(interp, display, str, eventPtr, offset)
    Tcl_Interp *interp;
    Display *display;
    char *str;
    XEvent *eventPtr;
    int offset;
{
    int *widthPtr;

    widthPtr = (int *) (((char *) eventPtr) + offset);
    if(Tcl_GetInt(interp, str, widthPtr) != TCL_OK) {
	return TCL_ERROR;
    }

    eventPtr->xconfigurerequest.value_mask |= CWWidth;
    return(TCL_OK);
}

/*
 *----------------------------------------------------------------------
 *
 * ParseConfigReqHeight --
 *
 *	ParseConfigReqHeight parses the "height" field for
 *	a ConfigureRequest event.
 *
 * Results:
 *	Returns a standard Tcl result.
 *
 * Side effects:
 *	None.
 *
 *----------------------------------------------------------------------
 */

static int
ParseConfigReqHeight(interp, display, str, eventPtr, offset)
    Tcl_Interp *interp;
    Display *display;
    char *str;
    XEvent *eventPtr;
    int offset;
{
    int *heightPtr;

    heightPtr = (int *) (((char *) eventPtr) + offset);
    if(Tcl_GetInt(interp, str, heightPtr) != TCL_OK) {
	return TCL_ERROR;
    }

    eventPtr->xconfigurerequest.value_mask |= CWHeight;
    return(TCL_OK);
}

/*
 *----------------------------------------------------------------------
 *
 * ParseConfigReqBdWidth --
 *
 *	ParseConfigReqBdWidth parses the "border_width" field for
 *	a ConfigureRequest event.
 *
 * Results:
 *	Returns a standard Tcl result.
 *
 * Side effects:
 *	None.
 *
 *----------------------------------------------------------------------
 */

static int
ParseConfigReqBdWidth(interp, display, str, eventPtr, offset)
    Tcl_Interp *interp;
    Display *display;
    char *str;
    XEvent *eventPtr;
    int offset;
{
    int *bdWidthPtr;

    bdWidthPtr = (int *) (((char *) eventPtr) + offset);
    if(Tcl_GetInt(interp, str, bdWidthPtr) != TCL_OK) {
	return TCL_ERROR;
    }

    eventPtr->xconfigurerequest.value_mask |= CWBorderWidth;
    return(TCL_OK);
}

/*
 *----------------------------------------------------------------------
 *
 * ParseConfigReqDetail --
 *
 *	ParseConfigReqDetail parses the "detail" field for
 *	a ConfigureRequest event.
 *
 * Results:
 *	Returns a standard Tcl result.
 *
 * Side effects:
 *	None.
 *
 *----------------------------------------------------------------------
 */

static struct {
    char *name;
    int detail;
} configReqDetailTab[] = {
    {"none", None},
    {"above", Above},
    {"below", Below},
    {"topif", TopIf},
    {"bottomif", BottomIf},
    {"opposite", Opposite},
    {NULL, 0}
};

static int
ParseConfigReqDetail(interp, display, str, eventPtr, offset)
    Tcl_Interp *interp;
    Display *display;
    char *str;
    XEvent *eventPtr;
    int offset;
{
    int i;
    int *detailPtr;

    if (strlen(str) == 0) {
	Tcl_SetResult(interp,
		"Detail string cannot be null", TCL_STATIC);
	return TCL_ERROR;
    }

    detailPtr = (int *) (((char *) eventPtr) + offset);

    for (i = 0; configReqDetailTab[i].name != NULL; i++) {
	if (strncasecmp(configReqDetailTab[i].name, str, strlen(str)) == 0) {
	    *detailPtr = configReqDetailTab[i].detail;
	    eventPtr->xconfigurerequest.value_mask |= CWStackMode;
	    return TCL_OK;
	}
    }

    Tcl_SetResult(interp,
	"Detail field must be none, above, below, topif, bottomif, or opposite",
							TCL_STATIC);
    return(TCL_ERROR);
}

/*
 *----------------------------------------------------------------------
 *
 * ParseMajorCode --
 *
 *	ParseMajorCode parses the "major_code" field for
 *	a GraphicsExpose and NoExpose events.
 *
 * Results:
 *	Returns a standard Tcl result.
 *
 * Side effects:
 *	None.
 *
 *----------------------------------------------------------------------
 */

static int
ParseMajorCode(interp, display, str, eventPtr, offset)
    Tcl_Interp *interp;
    Display *display;
    char *str;
    XEvent *eventPtr;
    int offset;
{
    int *codePtr;

    codePtr = (int *) (((char *) eventPtr) + offset);

    if (strcasecmp(str, "copyarea") == 0) {
	*codePtr = X_CopyArea;
	return TCL_OK;
    }

    if (strcasecmp(str, "copyplane") == 0) {
	*codePtr = X_CopyPlane;
	return TCL_OK;
    }

    return(Tcl_GetInt(interp, str, codePtr));
}

/*
 *----------------------------------------------------------------------
 *
 * ParseKeyVector --
 *
 *	ParseKeyVector parses the "key_vector" field for
 *	a KeymapNotify event. The vector is written as a
 *	list of key codes, i.e. a list of 8-bit integers.
 *
 * Results:
 *	Returns a standard Tcl result.
 *
 * Side effects:
 *	None.
 *
 *----------------------------------------------------------------------
 */

static int
ParseKeyVector(interp, display, str, eventPtr, offset)
    Tcl_Interp *interp;
    Display *display;
    char *str;
    XEvent *eventPtr;
    int offset;
{
    char *keyVecPtr;
    int argc;
    char **argv;
    int i, keyCode;
    int byteIdx, bitIdx;

    if (Tcl_SplitList(interp, str, &argc, &argv) != TCL_OK) {
	return TCL_ERROR;
    }

    keyVecPtr = (((char *) eventPtr) + offset);
    for(i = 0; i < argc; i++) {
	if(Tcl_GetInt(interp, argv[i], &keyCode) != TCL_OK) {
	    Tcl_Free((char *) argv);
	    return TCL_ERROR;
	}

	if (keyCode < 0 || keyCode > 255) {
	    Tcl_SetResult(interp,
		    "Keycode out of range (must be 0 <= keycode <= 255)",
						TCL_STATIC);
	    Tcl_Free((char *) argv);
	    return TCL_ERROR;
	}

	byteIdx = keyCode/8;
	bitIdx = keyCode%8;
	keyVecPtr[byteIdx] |= (1 << bitIdx);
    }

    Tcl_Free((char *) argv);
    return TCL_OK;
}

/*
 *----------------------------------------------------------------------
 *
 * ParseMappingRequest --
 *
 *	ParseMappingRequest parses the "request" field for
 *	a MappingNotify event.
 *
 * Results:
 *	Returns a standard Tcl result.
 *
 * Side effects:
 *	None.
 *
 *----------------------------------------------------------------------
 */

static int
ParseMappingRequest(interp, display, str, eventPtr, offset)
    Tcl_Interp *interp;
    Display *display;
    char *str;
    XEvent *eventPtr;
    int offset;
{
    int *requestPtr;

    requestPtr = (int *) (((char *) eventPtr) + offset);

    /* Strip off a leading "mapping" prefix, if present. */

    if (strncasecmp(str, "mapping", 7) == 0) {
	str += 7;
    }

    /* Check each possibility... */

    if (strncasecmp(str, "modifier", strlen(str)) == 0) {
	*requestPtr = MappingModifier;
	return TCL_OK;
    }

    if (strncasecmp(str, "keyboard", strlen(str)) == 0) {
	*requestPtr = MappingKeyboard;
	return TCL_OK;
    }

    if (strncasecmp(str, "pointer", strlen(str)) == 0) {
	*requestPtr = MappingPointer;
	return TCL_OK;
    }

    Tcl_SetResult(interp,
		"request must be mapping, keyboard, or pointer", TCL_STATIC);
    return TCL_ERROR;
}

/*
 *----------------------------------------------------------------------
 *
 * ParsePropertyState --
 *
 *	ParsePropertyState parses the "state" field for
 *	a PropertyNotify event.
 *
 * Results:
 *	Returns a standard Tcl result.
 *
 * Side effects:
 *	None.
 *
 *----------------------------------------------------------------------
 */

static int
ParsePropertyState(interp, display, str, eventPtr, offset)
    Tcl_Interp *interp;
    Display *display;
    char *str;
    XEvent *eventPtr;
    int offset;
{
    int *statePtr;

    statePtr = (int *) (((char *) eventPtr) + offset);

    /* Strip off a leading "property" prefix, if present. */

    if (strncasecmp(str, "property", 8) == 0) {
	str += 8;
    }

    /* Check each possibility... */

    if (strncasecmp(str, "newvalue", strlen(str)) == 0) {
	*statePtr = PropertyNewValue;
	return TCL_OK;
    }

    if (strncasecmp(str, "delete", strlen(str)) == 0) {
	*statePtr = PropertyDelete;
	return TCL_OK;
    }

    Tcl_SetResult(interp,
		"state must be newvalue or delete", TCL_STATIC);
    return TCL_ERROR;
}

/*
 *----------------------------------------------------------------------
 *
 * ParseVisibilityState --
 *
 *	ParseVisibilityState parses the "state" field for
 *	a VisibilityNotify event.
 *
 * Results:
 *	Returns a standard Tcl result.
 *
 * Side effects:
 *	None.
 *
 *----------------------------------------------------------------------
 */

static int
ParseVisibilityState(interp, display, str, eventPtr, offset)
    Tcl_Interp *interp;
    Display *display;
    char *str;
    XEvent *eventPtr;
    int offset;
{
    int *statePtr;

    statePtr = (int *) (((char *) eventPtr) + offset);

    /* Strip off a leading "visibility" prefix, if present. */

    if (strncasecmp(str, "visibility", 10) == 0) {
	str += 10;
    }

    /* Check each possibility... */

    if (strncasecmp(str, "fullyobscured", strlen(str)) == 0) {
	*statePtr = VisibilityFullyObscured;
	return TCL_OK;
    }

    if (strncasecmp(str, "partiallyobscured", strlen(str)) == 0) {
	*statePtr = VisibilityPartiallyObscured;
	return TCL_OK;
    }

    if (strncasecmp(str, "unobscured", strlen(str)) == 0) {
	*statePtr = VisibilityUnobscured;
	return TCL_OK;
    }

    Tcl_SetResult(interp,
		"state must be fullyobscured, partiallyobscured or unobscured",
								TCL_STATIC);
    return TCL_ERROR;
}

/*
 * Type EventOptParseTabEntry represents an entry in the event
 * option parsing table. It contains a pointer to the function
 * that will parse the value, and an integer offset into the
 * XEvent struct.
 */

typedef struct {
    EventParseFuncPtr func;	/* Parsing function		*/
    long offset;		/* Offset into the XEvent struct
				 * where we'll put the result;
				 * offset is in bytes.		*/
} EventOptParseTabEntry;

/*
 * eventOptParseTab is the event option parse table. The first
 * index represents the event type, and the second represents
 * the option name. These numbers are determined by indexing
 * into the "eventNames" and "eventOptNames" arrays. NOTE: IF YOU
 * CHANGE THE eventNames AND/OR eventOptNames ARRAYS, YOU MUST
 * MAKE THIS TABLE MATCH THEM!
 */

static EventOptParseTabEntry
eventOptParseTab[N_EVENT_TYPES][N_EVENT_OPTS] = {
    /* ButtonPress */

    {
	{(EventParseFuncPtr) NULL, 0},	/* -above		*/
	{(EventParseFuncPtr) NULL, 0},	/* -atom		*/
	{(EventParseFuncPtr) NULL, 0},	/* -borderwidth		*/
	{ParseButtonName, Tk_Offset(XButtonEvent, button)},
	{(EventParseFuncPtr) NULL, 0},	/* -colormap		*/
	{(EventParseFuncPtr) NULL, 0},	/* -count		*/
	{(EventParseFuncPtr) NULL, 0},	/* -data		*/
	{(EventParseFuncPtr) NULL, 0},	/* -detail		*/
	{(EventParseFuncPtr) NULL, 0},	/* -drawable		*/
	{(EventParseFuncPtr) NULL, 0},	/* -event		*/
	{(EventParseFuncPtr) NULL, 0},	/* -firstkey		*/
	{(EventParseFuncPtr) NULL, 0},	/* -focus		*/
	{(EventParseFuncPtr) NULL, 0},	/* -format		*/
	{(EventParseFuncPtr) NULL, 0},	/* -fromconfigure	*/
	{(EventParseFuncPtr) NULL, 0},	/* -height		*/
	{(EventParseFuncPtr) NULL, 0},	/* -ishint		*/
	{(EventParseFuncPtr) NULL, 0},	/* -keycode		*/
	{(EventParseFuncPtr) NULL, 0},	/* -keyvector		*/
	{(EventParseFuncPtr) NULL, 0},	/* -majorcode		*/
	{(EventParseFuncPtr) NULL, 0},	/* -messagetype		*/
	{(EventParseFuncPtr) NULL, 0},	/* -minorcode		*/
	{(EventParseFuncPtr) NULL, 0},	/* -mode		*/
	{(EventParseFuncPtr) NULL, 0},	/* -new			*/
	{(EventParseFuncPtr) NULL, 0},	/* -overrideredirect	*/
	{(EventParseFuncPtr) NULL, 0},	/* -owner		*/
	{(EventParseFuncPtr) NULL, 0},	/* -parent		*/
	{(EventParseFuncPtr) NULL, 0},	/* -place		*/
	{(EventParseFuncPtr) NULL, 0},	/* -property		*/
	{(EventParseFuncPtr) NULL, 0},	/* -request		*/
	{(EventParseFuncPtr) NULL, 0},	/* -requestor		*/
	{ParseEventWindowId, Tk_Offset(XButtonEvent, root)},
	{ParseEventBool, Tk_Offset(XButtonEvent, same_screen)},
	{(EventParseFuncPtr) NULL, 0},	/* -selection		*/
	{ParseEventBool, Tk_Offset(XButtonEvent, send_event)},
	{ParseEventInt, Tk_Offset(XButtonEvent, serial)},
	{ParseEventModMask, Tk_Offset(XButtonEvent, state)},
	{ParseEventWindowId, Tk_Offset(XButtonEvent, subwindow)},
	{(EventParseFuncPtr) NULL, 0},	/* -target		*/
	{ParseEventInt, Tk_Offset(XButtonEvent, time)},
	{(EventParseFuncPtr) NULL, 0},	/* -width		*/
	{ParseEventWindowId, Tk_Offset(XButtonEvent, window)},
	{ParseEventInt, Tk_Offset(XButtonEvent, x)},
	{ParseEventInt, Tk_Offset(XButtonEvent, x_root)},
	{ParseEventInt, Tk_Offset(XButtonEvent, y)},
	{ParseEventInt, Tk_Offset(XButtonEvent, y_root)}
    },

    /* ButtonRelease */

    {
	{(EventParseFuncPtr) NULL, 0},	/* -above		*/
	{(EventParseFuncPtr) NULL, 0},	/* -atom		*/
	{(EventParseFuncPtr) NULL, 0},	/* -borderwidth		*/
	{ParseButtonName, Tk_Offset(XButtonEvent, button)},
	{(EventParseFuncPtr) NULL, 0},	/* -colormap		*/
	{(EventParseFuncPtr) NULL, 0},	/* -count		*/
	{(EventParseFuncPtr) NULL, 0},	/* -data		*/
	{(EventParseFuncPtr) NULL, 0},	/* -detail		*/
	{(EventParseFuncPtr) NULL, 0},	/* -drawable		*/
	{(EventParseFuncPtr) NULL, 0},	/* -event		*/
	{(EventParseFuncPtr) NULL, 0},	/* -firstkey		*/
	{(EventParseFuncPtr) NULL, 0},	/* -focus		*/
	{(EventParseFuncPtr) NULL, 0},	/* -format		*/
	{(EventParseFuncPtr) NULL, 0},	/* -fromconfigure	*/
	{(EventParseFuncPtr) NULL, 0},	/* -height		*/
	{(EventParseFuncPtr) NULL, 0},	/* -ishint		*/
	{(EventParseFuncPtr) NULL, 0},	/* -keycode		*/
	{(EventParseFuncPtr) NULL, 0},	/* -keyvector		*/
	{(EventParseFuncPtr) NULL, 0},	/* -majorcode		*/
	{(EventParseFuncPtr) NULL, 0},	/* -messagetype		*/
	{(EventParseFuncPtr) NULL, 0},	/* -minorcode		*/
	{(EventParseFuncPtr) NULL, 0},	/* -mode		*/
	{(EventParseFuncPtr) NULL, 0},	/* -new			*/
	{(EventParseFuncPtr) NULL, 0},	/* -overrideredirect	*/
	{(EventParseFuncPtr) NULL, 0},	/* -owner		*/
	{(EventParseFuncPtr) NULL, 0},	/* -parent		*/
	{(EventParseFuncPtr) NULL, 0},	/* -place		*/
	{(EventParseFuncPtr) NULL, 0},	/* -property		*/
	{(EventParseFuncPtr) NULL, 0},	/* -request		*/
	{(EventParseFuncPtr) NULL, 0},	/* -requestor		*/
	{ParseEventWindowId, Tk_Offset(XButtonEvent, root)},
	{ParseEventBool, Tk_Offset(XButtonEvent, same_screen)},
	{(EventParseFuncPtr) NULL, 0},	/* -selection		*/
	{ParseEventBool, Tk_Offset(XButtonEvent, send_event)},
	{ParseEventInt, Tk_Offset(XButtonEvent, serial)},
	{ParseEventModMask, Tk_Offset(XButtonEvent, state)},
	{ParseEventWindowId, Tk_Offset(XButtonEvent, subwindow)},
	{(EventParseFuncPtr) NULL, 0},	/* -target		*/
	{ParseEventInt, Tk_Offset(XButtonEvent, time)},
	{(EventParseFuncPtr) NULL, 0},	/* -width		*/
	{ParseEventWindowId, Tk_Offset(XButtonEvent, window)},
	{ParseEventInt, Tk_Offset(XButtonEvent, x)},
	{ParseEventInt, Tk_Offset(XButtonEvent, x_root)},
	{ParseEventInt, Tk_Offset(XButtonEvent, y)},
	{ParseEventInt, Tk_Offset(XButtonEvent, y_root)}
    },

    /* CirculateNotify */

    {
	{(EventParseFuncPtr) NULL, 0},	/* -above		*/
	{(EventParseFuncPtr) NULL, 0},	/* -atom		*/
	{(EventParseFuncPtr) NULL, 0},	/* -borderwidth		*/
	{(EventParseFuncPtr) NULL, 0},	/* -button		*/
	{(EventParseFuncPtr) NULL, 0},	/* -colormap		*/
	{(EventParseFuncPtr) NULL, 0},	/* -count		*/
	{(EventParseFuncPtr) NULL, 0},	/* -data		*/
	{(EventParseFuncPtr) NULL, 0},	/* -detail		*/
	{(EventParseFuncPtr) NULL, 0},	/* -drawable		*/
	{ParseEventWindowId, Tk_Offset(XCirculateEvent, event)},
	{(EventParseFuncPtr) NULL, 0},	/* -firstkey		*/
	{(EventParseFuncPtr) NULL, 0},	/* -focus		*/
	{(EventParseFuncPtr) NULL, 0},	/* -format		*/
	{(EventParseFuncPtr) NULL, 0},	/* -fromconfigure	*/
	{(EventParseFuncPtr) NULL, 0},	/* -height		*/
	{(EventParseFuncPtr) NULL, 0},	/* -ishint		*/
	{(EventParseFuncPtr) NULL, 0},	/* -keycode		*/
	{(EventParseFuncPtr) NULL, 0},	/* -keyvector		*/
	{(EventParseFuncPtr) NULL, 0},	/* -majorcode		*/
	{(EventParseFuncPtr) NULL, 0},	/* -messagetype		*/
	{(EventParseFuncPtr) NULL, 0},	/* -minorcode		*/
	{(EventParseFuncPtr) NULL, 0},	/* -mode		*/
	{(EventParseFuncPtr) NULL, 0},	/* -new			*/
	{(EventParseFuncPtr) NULL, 0},	/* -overrideredirect	*/
	{(EventParseFuncPtr) NULL, 0},	/* -owner		*/
	{(EventParseFuncPtr) NULL, 0},	/* -parent		*/
	{ParseEventPlace, Tk_Offset(XCirculateEvent, place)},
	{(EventParseFuncPtr) NULL, 0},	/* -property		*/
	{(EventParseFuncPtr) NULL, 0},	/* -request		*/
	{(EventParseFuncPtr) NULL, 0},	/* -requestor		*/
	{(EventParseFuncPtr) NULL, 0},	/* -root		*/
	{(EventParseFuncPtr) NULL, 0},	/* -samescreen		*/
	{(EventParseFuncPtr) NULL, 0},	/* -selection		*/
	{ParseEventBool, Tk_Offset(XCirculateEvent, send_event)},
	{ParseEventInt, Tk_Offset(XCirculateEvent, serial)},
	{(EventParseFuncPtr) NULL, 0},	/* -state		*/
	{(EventParseFuncPtr) NULL, 0},	/* -subwindow		*/
	{(EventParseFuncPtr) NULL, 0},	/* -target		*/
	{(EventParseFuncPtr) NULL, 0},	/* -time		*/
	{(EventParseFuncPtr) NULL, 0},	/* -width		*/
	{ParseEventWindowId, Tk_Offset(XCirculateEvent, window)},
	{(EventParseFuncPtr) NULL, 0},	/* -x			*/
	{(EventParseFuncPtr) NULL, 0},	/* -xroot		*/
	{(EventParseFuncPtr) NULL, 0},	/* -y			*/
	{(EventParseFuncPtr) NULL, 0}	/* -yroot		*/
    },

    /* CirculateRequest */

    {
	{(EventParseFuncPtr) NULL, 0},	/* -above		*/
	{(EventParseFuncPtr) NULL, 0},	/* -atom		*/
	{(EventParseFuncPtr) NULL, 0},	/* -borderwidth		*/
	{(EventParseFuncPtr) NULL, 0},	/* -button		*/
	{(EventParseFuncPtr) NULL, 0},	/* -colormap		*/
	{(EventParseFuncPtr) NULL, 0},	/* -count		*/
	{(EventParseFuncPtr) NULL, 0},	/* -data		*/
	{(EventParseFuncPtr) NULL, 0},	/* -detail		*/
	{(EventParseFuncPtr) NULL, 0},	/* -drawable		*/
	{(EventParseFuncPtr) NULL, 0},	/* -event		*/
	{(EventParseFuncPtr) NULL, 0},	/* -firstkey		*/
	{(EventParseFuncPtr) NULL, 0},	/* -focus		*/
	{(EventParseFuncPtr) NULL, 0},	/* -format		*/
	{(EventParseFuncPtr) NULL, 0},	/* -fromconfigure	*/
	{(EventParseFuncPtr) NULL, 0},	/* -height		*/
	{(EventParseFuncPtr) NULL, 0},	/* -ishint		*/
	{(EventParseFuncPtr) NULL, 0},	/* -keycode		*/
	{(EventParseFuncPtr) NULL, 0},	/* -keyvector		*/
	{(EventParseFuncPtr) NULL, 0},	/* -majorcode		*/
	{(EventParseFuncPtr) NULL, 0},	/* -messagetype		*/
	{(EventParseFuncPtr) NULL, 0},	/* -minorcode		*/
	{(EventParseFuncPtr) NULL, 0},	/* -mode		*/
	{(EventParseFuncPtr) NULL, 0},	/* -new			*/
	{(EventParseFuncPtr) NULL, 0},	/* -overrideredirect	*/
	{(EventParseFuncPtr) NULL, 0},	/* -owner		*/
	{ParseEventWindowId, Tk_Offset(XCirculateRequestEvent, parent)},
	{ParseEventPlace, Tk_Offset(XCirculateRequestEvent, place)},
	{(EventParseFuncPtr) NULL, 0},	/* -property		*/
	{(EventParseFuncPtr) NULL, 0},	/* -request		*/
	{(EventParseFuncPtr) NULL, 0},	/* -requestor		*/
	{(EventParseFuncPtr) NULL, 0},	/* -root		*/
	{(EventParseFuncPtr) NULL, 0},	/* -samescreen		*/
	{(EventParseFuncPtr) NULL, 0},	/* -selection		*/
	{ParseEventBool, Tk_Offset(XCirculateRequestEvent, send_event)},
	{ParseEventInt, Tk_Offset(XCirculateRequestEvent, serial)},
	{(EventParseFuncPtr) NULL, 0},	/* -state		*/
	{(EventParseFuncPtr) NULL, 0},	/* -subwindow		*/
	{(EventParseFuncPtr) NULL, 0},	/* -target		*/
	{(EventParseFuncPtr) NULL, 0},	/* -time		*/
	{(EventParseFuncPtr) NULL, 0},	/* -width		*/
	{ParseEventWindowId, Tk_Offset(XCirculateRequestEvent, window)},
	{(EventParseFuncPtr) NULL, 0},	/* -x			*/
	{(EventParseFuncPtr) NULL, 0},	/* -xroot		*/
	{(EventParseFuncPtr) NULL, 0},	/* -y			*/
	{(EventParseFuncPtr) NULL, 0}	/* -yroot		*/
    },

    /* ClientMessage */

    {
	{(EventParseFuncPtr) NULL, 0},	/* -above		*/
	{(EventParseFuncPtr) NULL, 0},	/* -atom		*/
	{(EventParseFuncPtr) NULL, 0},	/* -borderwidth		*/
	{(EventParseFuncPtr) NULL, 0},	/* -button		*/
	{(EventParseFuncPtr) NULL, 0},	/* -colormap		*/
	{(EventParseFuncPtr) NULL, 0},	/* -count		*/
	{ParseClientMsgData, Tk_Offset(XClientMessageEvent, data)},
	{(EventParseFuncPtr) NULL, 0},	/* -detail		*/
	{(EventParseFuncPtr) NULL, 0},	/* -drawable		*/
	{(EventParseFuncPtr) NULL, 0},	/* -event		*/
	{(EventParseFuncPtr) NULL, 0},	/* -firstkey		*/
	{(EventParseFuncPtr) NULL, 0},	/* -focus		*/
	{ParseClientMsgFmt, Tk_Offset(XClientMessageEvent, format)},
	{(EventParseFuncPtr) NULL, 0},	/* -fromconfigure	*/
	{(EventParseFuncPtr) NULL, 0},	/* -height		*/
	{(EventParseFuncPtr) NULL, 0},	/* -ishint		*/
	{(EventParseFuncPtr) NULL, 0},	/* -keycode		*/
	{(EventParseFuncPtr) NULL, 0},	/* -keyvector		*/
	{(EventParseFuncPtr) NULL, 0},	/* -majorcode		*/
	{ParseEventAtom, Tk_Offset(XClientMessageEvent, message_type)},
	{(EventParseFuncPtr) NULL, 0},	/* -minorcode		*/
	{(EventParseFuncPtr) NULL, 0},	/* -mode		*/
	{(EventParseFuncPtr) NULL, 0},	/* -new			*/
	{(EventParseFuncPtr) NULL, 0},	/* -overrideredirect	*/
	{(EventParseFuncPtr) NULL, 0},	/* -owner		*/
	{(EventParseFuncPtr) NULL, 0},	/* -parent		*/
	{(EventParseFuncPtr) NULL, 0},	/* -place		*/
	{(EventParseFuncPtr) NULL, 0},	/* -property		*/
	{(EventParseFuncPtr) NULL, 0},	/* -request		*/
	{(EventParseFuncPtr) NULL, 0},	/* -requestor		*/
	{(EventParseFuncPtr) NULL, 0},	/* -root		*/
	{(EventParseFuncPtr) NULL, 0},	/* -samescreen		*/
	{(EventParseFuncPtr) NULL, 0},	/* -selection		*/
	{ParseEventBool, Tk_Offset(XClientMessageEvent, send_event)},
	{ParseEventInt, Tk_Offset(XClientMessageEvent, serial)},
	{(EventParseFuncPtr) NULL, 0},	/* -state		*/
	{(EventParseFuncPtr) NULL, 0},	/* -subwindow		*/
	{(EventParseFuncPtr) NULL, 0},	/* -target		*/
	{(EventParseFuncPtr) NULL, 0},	/* -time		*/
	{(EventParseFuncPtr) NULL, 0},	/* -width		*/
	{ParseEventWindowId, Tk_Offset(XClientMessageEvent, window)},
	{(EventParseFuncPtr) NULL, 0},	/* -x			*/
	{(EventParseFuncPtr) NULL, 0},	/* -xroot		*/
	{(EventParseFuncPtr) NULL, 0},	/* -y			*/
	{(EventParseFuncPtr) NULL, 0}	/* -yroot		*/
    },

    /* ColormapNotify */

    {
	{(EventParseFuncPtr) NULL, 0},	/* -above		*/
	{(EventParseFuncPtr) NULL, 0},	/* -atom		*/
	{(EventParseFuncPtr) NULL, 0},	/* -borderwidth		*/
	{(EventParseFuncPtr) NULL, 0},	/* -button		*/
	{ParseColormap, Tk_Offset(XColormapEvent, colormap)},
	{(EventParseFuncPtr) NULL, 0},	/* -count		*/
	{(EventParseFuncPtr) NULL, 0},	/* -data		*/
	{(EventParseFuncPtr) NULL, 0},	/* -detail		*/
	{(EventParseFuncPtr) NULL, 0},	/* -drawable		*/
	{(EventParseFuncPtr) NULL, 0},	/* -event		*/
	{(EventParseFuncPtr) NULL, 0},	/* -firstkey		*/
	{(EventParseFuncPtr) NULL, 0},	/* -focus		*/
	{(EventParseFuncPtr) NULL, 0},	/* -format		*/
	{(EventParseFuncPtr) NULL, 0},	/* -fromconfigure	*/
	{(EventParseFuncPtr) NULL, 0},	/* -height		*/
	{(EventParseFuncPtr) NULL, 0},	/* -ishint		*/
	{(EventParseFuncPtr) NULL, 0},	/* -keycode		*/
	{(EventParseFuncPtr) NULL, 0},	/* -keyvector		*/
	{(EventParseFuncPtr) NULL, 0},	/* -majorcode		*/
	{(EventParseFuncPtr) NULL, 0},	/* -messagetype		*/
	{(EventParseFuncPtr) NULL, 0},	/* -minorcode		*/
	{(EventParseFuncPtr) NULL, 0},	/* -mode		*/
	{ParseEventBool, Tk_Offset(XColormapEvent, new)},
	{(EventParseFuncPtr) NULL, 0},	/* -overrideredirect	*/
	{(EventParseFuncPtr) NULL, 0},	/* -owner		*/
	{(EventParseFuncPtr) NULL, 0},	/* -parent		*/
	{(EventParseFuncPtr) NULL, 0},	/* -place		*/
	{(EventParseFuncPtr) NULL, 0},	/* -property		*/
	{(EventParseFuncPtr) NULL, 0},	/* -request		*/
	{(EventParseFuncPtr) NULL, 0},	/* -requestor		*/
	{(EventParseFuncPtr) NULL, 0},	/* -root		*/
	{(EventParseFuncPtr) NULL, 0},	/* -samescreen		*/
	{(EventParseFuncPtr) NULL, 0},	/* -selection		*/
	{ParseEventBool, Tk_Offset(XColormapEvent, send_event)},
	{ParseEventInt, Tk_Offset(XColormapEvent, serial)},
	{ParseColormapState, Tk_Offset(XColormapEvent, state)},
	{(EventParseFuncPtr) NULL, 0},	/* -subwindow		*/
	{(EventParseFuncPtr) NULL, 0},	/* -target		*/
	{(EventParseFuncPtr) NULL, 0},	/* -time		*/
	{(EventParseFuncPtr) NULL, 0},	/* -width		*/
	{ParseEventWindowId, Tk_Offset(XColormapEvent, window)},
	{(EventParseFuncPtr) NULL, 0},	/* -x			*/
	{(EventParseFuncPtr) NULL, 0},	/* -xroot		*/
	{(EventParseFuncPtr) NULL, 0},	/* -y			*/
	{(EventParseFuncPtr) NULL, 0}	/* -yroot		*/
    },

    /* ConfigureNotify */

    {
	{ParseEventWindowOrNone, Tk_Offset(XConfigureEvent, above)},
	{(EventParseFuncPtr) NULL, 0},	/* -atom		*/
	{ParseEventInt, Tk_Offset(XConfigureEvent, border_width)},
	{(EventParseFuncPtr) NULL, 0},	/* -button		*/
	{(EventParseFuncPtr) NULL, 0},	/* -colormap		*/
	{(EventParseFuncPtr) NULL, 0},	/* -count		*/
	{(EventParseFuncPtr) NULL, 0},	/* -data		*/
	{(EventParseFuncPtr) NULL, 0},	/* -detail		*/
	{(EventParseFuncPtr) NULL, 0},	/* -drawable		*/
	{ParseEventWindowId, Tk_Offset(XConfigureEvent, event)},
	{(EventParseFuncPtr) NULL, 0},	/* -firstkey		*/
	{(EventParseFuncPtr) NULL, 0},	/* -focus		*/
	{(EventParseFuncPtr) NULL, 0},	/* -format		*/
	{(EventParseFuncPtr) NULL, 0},	/* -fromconfigure	*/
	{ParseEventInt, Tk_Offset(XConfigureEvent, height)},
	{(EventParseFuncPtr) NULL, 0},	/* -ishint		*/
	{(EventParseFuncPtr) NULL, 0},	/* -keycode		*/
	{(EventParseFuncPtr) NULL, 0},	/* -keyvector		*/
	{(EventParseFuncPtr) NULL, 0},	/* -majorcode		*/
	{(EventParseFuncPtr) NULL, 0},	/* -messagetype		*/
	{(EventParseFuncPtr) NULL, 0},	/* -minorcode		*/
	{(EventParseFuncPtr) NULL, 0},	/* -mode		*/
	{(EventParseFuncPtr) NULL, 0},	/* -new			*/
	{ParseEventBool, Tk_Offset(XConfigureEvent, override_redirect)},
	{(EventParseFuncPtr) NULL, 0},	/* -owner		*/
	{(EventParseFuncPtr) NULL, 0},	/* -parent		*/
	{(EventParseFuncPtr) NULL, 0},	/* -place		*/
	{(EventParseFuncPtr) NULL, 0},	/* -property		*/
	{(EventParseFuncPtr) NULL, 0},	/* -request		*/
	{(EventParseFuncPtr) NULL, 0},	/* -requestor		*/
	{(EventParseFuncPtr) NULL, 0},	/* -root		*/
	{(EventParseFuncPtr) NULL, 0},	/* -samescreen		*/
	{(EventParseFuncPtr) NULL, 0},	/* -selection		*/
	{ParseEventBool, Tk_Offset(XConfigureEvent, send_event)},
	{ParseEventInt, Tk_Offset(XConfigureEvent, serial)},
	{(EventParseFuncPtr) NULL, 0},	/* -state		*/
	{(EventParseFuncPtr) NULL, 0},	/* -subwindow		*/
	{(EventParseFuncPtr) NULL, 0},	/* -target		*/
	{(EventParseFuncPtr) NULL, 0},	/* -time		*/
	{ParseEventInt, Tk_Offset(XConfigureEvent, width)},
	{ParseEventWindowId, Tk_Offset(XConfigureEvent, window)},
	{ParseEventInt, Tk_Offset(XConfigureEvent, x)},
	{(EventParseFuncPtr) NULL, 0},	/* -xroot		*/
	{ParseEventInt, Tk_Offset(XConfigureEvent, y)},
	{(EventParseFuncPtr) NULL, 0}	/* -yroot		*/
    },

    /* ConfigureRequest */

    {
	{ParseConfigReqAbove, Tk_Offset(XConfigureRequestEvent, above)},
	{(EventParseFuncPtr) NULL, 0},	/* -atom		*/
	{ParseConfigReqBdWidth, Tk_Offset(XConfigureRequestEvent, border_width)},
	{(EventParseFuncPtr) NULL, 0},	/* -button		*/
	{(EventParseFuncPtr) NULL, 0},	/* -colormap		*/
	{(EventParseFuncPtr) NULL, 0},	/* -count		*/
	{(EventParseFuncPtr) NULL, 0},	/* -data		*/
	{ParseConfigReqDetail, Tk_Offset(XConfigureRequestEvent, detail)},
	{(EventParseFuncPtr) NULL, 0},	/* -drawable		*/
	{(EventParseFuncPtr) NULL, 0},	/* -event		*/
	{(EventParseFuncPtr) NULL, 0},	/* -firstkey		*/
	{(EventParseFuncPtr) NULL, 0},	/* -focus		*/
	{(EventParseFuncPtr) NULL, 0},	/* -format		*/
	{(EventParseFuncPtr) NULL, 0},	/* -fromconfigure	*/
	{ParseConfigReqHeight, Tk_Offset(XConfigureRequestEvent, height)},
	{(EventParseFuncPtr) NULL, 0},	/* -ishint		*/
	{(EventParseFuncPtr) NULL, 0},	/* -keycode		*/
	{(EventParseFuncPtr) NULL, 0},	/* -keyvector		*/
	{(EventParseFuncPtr) NULL, 0},	/* -majorcode		*/
	{(EventParseFuncPtr) NULL, 0},	/* -messagetype		*/
	{(EventParseFuncPtr) NULL, 0},	/* -minorcode		*/
	{(EventParseFuncPtr) NULL, 0},	/* -mode		*/
	{(EventParseFuncPtr) NULL, 0},	/* -new			*/
	{(EventParseFuncPtr) NULL, 0},	/* -overrideredirect	*/
	{(EventParseFuncPtr) NULL, 0},	/* -owner		*/
	{ParseEventWindowId, Tk_Offset(XConfigureRequestEvent, parent)},
	{(EventParseFuncPtr) NULL, 0},	/* -place		*/
	{(EventParseFuncPtr) NULL, 0},	/* -property		*/
	{(EventParseFuncPtr) NULL, 0},	/* -request		*/
	{(EventParseFuncPtr) NULL, 0},	/* -requestor		*/
	{(EventParseFuncPtr) NULL, 0},	/* -root		*/
	{(EventParseFuncPtr) NULL, 0},	/* -samescreen		*/
	{(EventParseFuncPtr) NULL, 0},	/* -selection		*/
	{ParseEventBool, Tk_Offset(XConfigureRequestEvent, send_event)},
	{ParseEventInt, Tk_Offset(XConfigureRequestEvent, serial)},
	{(EventParseFuncPtr) NULL, 0},	/* -state		*/
	{(EventParseFuncPtr) NULL, 0},	/* -subwindow		*/
	{(EventParseFuncPtr) NULL, 0},	/* -target		*/
	{(EventParseFuncPtr) NULL, 0},	/* -time		*/
	{ParseConfigReqWidth, Tk_Offset(XConfigureRequestEvent, width)},
	{ParseEventWindowId, Tk_Offset(XConfigureRequestEvent, window)},
	{ParseConfigReqX, Tk_Offset(XConfigureRequestEvent, x)},
	{(EventParseFuncPtr) NULL, 0},	/* -xroot		*/
	{ParseConfigReqY, Tk_Offset(XConfigureRequestEvent, y)},
	{(EventParseFuncPtr) NULL, 0}	/* -yroot		*/
    },

    /* CreateNotify */

    {
	{(EventParseFuncPtr) NULL, 0},	/* -above		*/
	{(EventParseFuncPtr) NULL, 0},	/* -atom		*/
	{ParseEventInt, Tk_Offset(XCreateWindowEvent, border_width)},
	{(EventParseFuncPtr) NULL, 0},	/* -button		*/
	{(EventParseFuncPtr) NULL, 0},	/* -colormap		*/
	{(EventParseFuncPtr) NULL, 0},	/* -count		*/
	{(EventParseFuncPtr) NULL, 0},	/* -data		*/
	{(EventParseFuncPtr) NULL, 0},	/* -detail		*/
	{(EventParseFuncPtr) NULL, 0},	/* -drawable		*/
	{(EventParseFuncPtr) NULL, 0},	/* -event		*/
	{(EventParseFuncPtr) NULL, 0},	/* -firstkey		*/
	{(EventParseFuncPtr) NULL, 0},	/* -focus		*/
	{(EventParseFuncPtr) NULL, 0},	/* -format		*/
	{(EventParseFuncPtr) NULL, 0},	/* -fromconfigure	*/
	{ParseEventInt, Tk_Offset(XCreateWindowEvent, height)},
	{(EventParseFuncPtr) NULL, 0},	/* -ishint		*/
	{(EventParseFuncPtr) NULL, 0},	/* -keycode		*/
	{(EventParseFuncPtr) NULL, 0},	/* -keyvector		*/
	{(EventParseFuncPtr) NULL, 0},	/* -majorcode		*/
	{(EventParseFuncPtr) NULL, 0},	/* -messagetype		*/
	{(EventParseFuncPtr) NULL, 0},	/* -minorcode		*/
	{(EventParseFuncPtr) NULL, 0},	/* -mode		*/
	{(EventParseFuncPtr) NULL, 0},	/* -new			*/
	{ParseEventBool, Tk_Offset(XCreateWindowEvent, override_redirect)},
	{(EventParseFuncPtr) NULL, 0},	/* -owner		*/
	{ParseEventWindowId, Tk_Offset(XCreateWindowEvent, parent)},
	{(EventParseFuncPtr) NULL, 0},	/* -place		*/
	{(EventParseFuncPtr) NULL, 0},	/* -property		*/
	{(EventParseFuncPtr) NULL, 0},	/* -request		*/
	{(EventParseFuncPtr) NULL, 0},	/* -requestor		*/
	{(EventParseFuncPtr) NULL, 0},	/* -root		*/
	{(EventParseFuncPtr) NULL, 0},	/* -samescreen		*/
	{(EventParseFuncPtr) NULL, 0},	/* -selection		*/
	{ParseEventBool, Tk_Offset(XCreateWindowEvent, send_event)},
	{ParseEventInt, Tk_Offset(XCreateWindowEvent, serial)},
	{(EventParseFuncPtr) NULL, 0},	/* -state		*/
	{(EventParseFuncPtr) NULL, 0},	/* -subwindow		*/
	{(EventParseFuncPtr) NULL, 0},	/* -target		*/
	{(EventParseFuncPtr) NULL, 0},	/* -time		*/
	{ParseEventInt, Tk_Offset(XCreateWindowEvent, width)},
	{ParseEventWindowId, Tk_Offset(XCreateWindowEvent, window)},
	{ParseEventInt, Tk_Offset(XCreateWindowEvent, x)},
	{(EventParseFuncPtr) NULL, 0},	/* -xroot		*/
	{ParseEventInt, Tk_Offset(XCreateWindowEvent, y)},
	{(EventParseFuncPtr) NULL, 0}	/* -yroot		*/
    },

    /* DestroyNotify */

    {
	{(EventParseFuncPtr) NULL, 0},	/* -above		*/
	{(EventParseFuncPtr) NULL, 0},	/* -atom		*/
	{(EventParseFuncPtr) NULL, 0},	/* -borderwidth		*/
	{(EventParseFuncPtr) NULL, 0},	/* -button		*/
	{(EventParseFuncPtr) NULL, 0},	/* -colormap		*/
	{(EventParseFuncPtr) NULL, 0},	/* -count		*/
	{(EventParseFuncPtr) NULL, 0},	/* -data		*/
	{(EventParseFuncPtr) NULL, 0},	/* -detail		*/
	{(EventParseFuncPtr) NULL, 0},	/* -drawable		*/
	{ParseEventWindowId, Tk_Offset(XDestroyWindowEvent, event)},
	{(EventParseFuncPtr) NULL, 0},	/* -firstkey		*/
	{(EventParseFuncPtr) NULL, 0},	/* -focus		*/
	{(EventParseFuncPtr) NULL, 0},	/* -format		*/
	{(EventParseFuncPtr) NULL, 0},	/* -fromconfigure	*/
	{(EventParseFuncPtr) NULL, 0},	/* -height		*/
	{(EventParseFuncPtr) NULL, 0},	/* -ishint		*/
	{(EventParseFuncPtr) NULL, 0},	/* -keycode		*/
	{(EventParseFuncPtr) NULL, 0},	/* -keyvector		*/
	{(EventParseFuncPtr) NULL, 0},	/* -majorcode		*/
	{(EventParseFuncPtr) NULL, 0},	/* -messagetype		*/
	{(EventParseFuncPtr) NULL, 0},	/* -minorcode		*/
	{(EventParseFuncPtr) NULL, 0},	/* -mode		*/
	{(EventParseFuncPtr) NULL, 0},	/* -new			*/
	{(EventParseFuncPtr) NULL, 0},	/* -overrideredirect	*/
	{(EventParseFuncPtr) NULL, 0},	/* -owner		*/
	{(EventParseFuncPtr) NULL, 0},	/* -parent		*/
	{(EventParseFuncPtr) NULL, 0},	/* -place		*/
	{(EventParseFuncPtr) NULL, 0},	/* -property		*/
	{(EventParseFuncPtr) NULL, 0},	/* -request		*/
	{(EventParseFuncPtr) NULL, 0},	/* -requestor		*/
	{(EventParseFuncPtr) NULL, 0},	/* -root		*/
	{(EventParseFuncPtr) NULL, 0},	/* -samescreen		*/
	{(EventParseFuncPtr) NULL, 0},	/* -selection		*/
	{ParseEventBool, Tk_Offset(XDestroyWindowEvent, send_event)},
	{ParseEventInt, Tk_Offset(XDestroyWindowEvent, serial)},
	{(EventParseFuncPtr) NULL, 0},	/* -state		*/
	{(EventParseFuncPtr) NULL, 0},	/* -subwindow		*/
	{(EventParseFuncPtr) NULL, 0},	/* -target		*/
	{(EventParseFuncPtr) NULL, 0},	/* -time		*/
	{(EventParseFuncPtr) NULL, 0},	/* -width		*/
	{ParseEventWindowId, Tk_Offset(XDestroyWindowEvent, window)},
	{(EventParseFuncPtr) NULL, 0},	/* -x			*/
	{(EventParseFuncPtr) NULL, 0},	/* -xroot		*/
	{(EventParseFuncPtr) NULL, 0},	/* -y			*/
	{(EventParseFuncPtr) NULL, 0}	/* -yroot		*/
    },

    /* EnterNotify */

    {
	{(EventParseFuncPtr) NULL, 0},	/* -above		*/
	{(EventParseFuncPtr) NULL, 0},	/* -atom		*/
	{(EventParseFuncPtr) NULL, 0},	/* -borderwidth		*/
	{(EventParseFuncPtr) NULL, 0},	/* -button		*/
	{(EventParseFuncPtr) NULL, 0},	/* -colormap		*/
	{(EventParseFuncPtr) NULL, 0},	/* -count		*/
	{(EventParseFuncPtr) NULL, 0},	/* -data		*/
	{ParseELFDetail, Tk_Offset(XCrossingEvent, detail)},
	{(EventParseFuncPtr) NULL, 0},	/* -drawable		*/
	{(EventParseFuncPtr) NULL, 0},	/* -event		*/
	{(EventParseFuncPtr) NULL, 0},	/* -firstkey		*/
	{ParseEventBool, Tk_Offset(XCrossingEvent, focus)},
	{(EventParseFuncPtr) NULL, 0},	/* -format		*/
	{(EventParseFuncPtr) NULL, 0},	/* -fromconfigure	*/
	{(EventParseFuncPtr) NULL, 0},	/* -height		*/
	{(EventParseFuncPtr) NULL, 0},	/* -ishint		*/
	{(EventParseFuncPtr) NULL, 0},	/* -keycode		*/
	{(EventParseFuncPtr) NULL, 0},	/* -keyvector		*/
	{(EventParseFuncPtr) NULL, 0},	/* -majorcode		*/
	{(EventParseFuncPtr) NULL, 0},	/* -messagetype		*/
	{(EventParseFuncPtr) NULL, 0},	/* -minorcode		*/
	{ParseELFMode, Tk_Offset(XCrossingEvent, mode)},
	{(EventParseFuncPtr) NULL, 0},	/* -new			*/
	{(EventParseFuncPtr) NULL, 0},	/* -overrideredirect	*/
	{(EventParseFuncPtr) NULL, 0},	/* -owner		*/
	{(EventParseFuncPtr) NULL, 0},	/* -parent		*/
	{(EventParseFuncPtr) NULL, 0},	/* -place		*/
	{(EventParseFuncPtr) NULL, 0},	/* -property		*/
	{(EventParseFuncPtr) NULL, 0},	/* -request		*/
	{(EventParseFuncPtr) NULL, 0},	/* -requestor		*/
	{ParseEventWindowId, Tk_Offset(XCrossingEvent, root)},
	{ParseEventBool, Tk_Offset(XCrossingEvent, same_screen)},
	{(EventParseFuncPtr) NULL, 0},	/* -selection		*/
	{ParseEventBool, Tk_Offset(XCrossingEvent, send_event)},
	{ParseEventInt, Tk_Offset(XCrossingEvent, serial)},
	{ParseEventModMask, Tk_Offset(XCrossingEvent, state)},
	{ParseEventWindowId, Tk_Offset(XCrossingEvent, subwindow)},
	{(EventParseFuncPtr) NULL, 0},	/* -target		*/
	{ParseEventInt, Tk_Offset(XCrossingEvent, time)},
	{(EventParseFuncPtr) NULL, 0},	/* -width		*/
	{ParseEventWindowId, Tk_Offset(XCrossingEvent, window)},
	{ParseEventInt, Tk_Offset(XCrossingEvent, x)},
	{ParseEventInt, Tk_Offset(XCrossingEvent, x_root)},
	{ParseEventInt, Tk_Offset(XCrossingEvent, y)},
	{ParseEventInt, Tk_Offset(XCrossingEvent, y_root)},
    },

    /* LeaveNotify */

    {
	{(EventParseFuncPtr) NULL, 0},	/* -above		*/
	{(EventParseFuncPtr) NULL, 0},	/* -atom		*/
	{(EventParseFuncPtr) NULL, 0},	/* -borderwidth		*/
	{(EventParseFuncPtr) NULL, 0},	/* -button		*/
	{(EventParseFuncPtr) NULL, 0},	/* -colormap		*/
	{(EventParseFuncPtr) NULL, 0},	/* -count		*/
	{(EventParseFuncPtr) NULL, 0},	/* -data		*/
	{ParseELFDetail, Tk_Offset(XCrossingEvent, detail)},
	{(EventParseFuncPtr) NULL, 0},	/* -drawable		*/
	{(EventParseFuncPtr) NULL, 0},	/* -event		*/
	{(EventParseFuncPtr) NULL, 0},	/* -firstkey		*/
	{ParseEventBool, Tk_Offset(XCrossingEvent, focus)},
	{(EventParseFuncPtr) NULL, 0},	/* -format		*/
	{(EventParseFuncPtr) NULL, 0},	/* -fromconfigure	*/
	{(EventParseFuncPtr) NULL, 0},	/* -height		*/
	{(EventParseFuncPtr) NULL, 0},	/* -ishint		*/
	{(EventParseFuncPtr) NULL, 0},	/* -keycode		*/
	{(EventParseFuncPtr) NULL, 0},	/* -keyvector		*/
	{(EventParseFuncPtr) NULL, 0},	/* -majorcode		*/
	{(EventParseFuncPtr) NULL, 0},	/* -messagetype		*/
	{(EventParseFuncPtr) NULL, 0},	/* -minorcode		*/
	{ParseELFMode, Tk_Offset(XCrossingEvent, mode)},
	{(EventParseFuncPtr) NULL, 0},	/* -new			*/
	{(EventParseFuncPtr) NULL, 0},	/* -overrideredirect	*/
	{(EventParseFuncPtr) NULL, 0},	/* -owner		*/
	{(EventParseFuncPtr) NULL, 0},	/* -parent		*/
	{(EventParseFuncPtr) NULL, 0},	/* -place		*/
	{(EventParseFuncPtr) NULL, 0},	/* -property		*/
	{(EventParseFuncPtr) NULL, 0},	/* -request		*/
	{(EventParseFuncPtr) NULL, 0},	/* -requestor		*/
	{ParseEventWindowId, Tk_Offset(XCrossingEvent, root)},
	{ParseEventBool, Tk_Offset(XCrossingEvent, same_screen)},
	{(EventParseFuncPtr) NULL, 0},	/* -selection		*/
	{ParseEventBool, Tk_Offset(XCrossingEvent, send_event)},
	{ParseEventInt, Tk_Offset(XCrossingEvent, serial)},
	{ParseEventModMask, Tk_Offset(XCrossingEvent, state)},
	{ParseEventWindowId, Tk_Offset(XCrossingEvent, subwindow)},
	{(EventParseFuncPtr) NULL, 0},	/* -target		*/
	{ParseEventInt, Tk_Offset(XCrossingEvent, time)},
	{(EventParseFuncPtr) NULL, 0},	/* -width		*/
	{ParseEventWindowId, Tk_Offset(XCrossingEvent, window)},
	{ParseEventInt, Tk_Offset(XCrossingEvent, x)},
	{ParseEventInt, Tk_Offset(XCrossingEvent, x_root)},
	{ParseEventInt, Tk_Offset(XCrossingEvent, y)},
	{ParseEventInt, Tk_Offset(XCrossingEvent, y_root)},
    },

    /* Expose */

    {
	{(EventParseFuncPtr) NULL, 0},	/* -above		*/
	{(EventParseFuncPtr) NULL, 0},	/* -atom		*/
	{(EventParseFuncPtr) NULL, 0},	/* -borderwidth		*/
	{(EventParseFuncPtr) NULL, 0},	/* -button		*/
	{(EventParseFuncPtr) NULL, 0},	/* -colormap		*/
	{ParseEventInt, Tk_Offset(XExposeEvent, count)},
	{(EventParseFuncPtr) NULL, 0},	/* -data		*/
	{(EventParseFuncPtr) NULL, 0},	/* -detail		*/
	{(EventParseFuncPtr) NULL, 0},	/* -drawable		*/
	{(EventParseFuncPtr) NULL, 0},	/* -event		*/
	{(EventParseFuncPtr) NULL, 0},	/* -firstkey		*/
	{(EventParseFuncPtr) NULL, 0},	/* -focus		*/
	{(EventParseFuncPtr) NULL, 0},	/* -format		*/
	{(EventParseFuncPtr) NULL, 0},	/* -fromconfigure	*/
	{ParseEventInt, Tk_Offset(XExposeEvent, height)},
	{(EventParseFuncPtr) NULL, 0},	/* -ishint		*/
	{(EventParseFuncPtr) NULL, 0},	/* -keycode		*/
	{(EventParseFuncPtr) NULL, 0},	/* -keyvector		*/
	{(EventParseFuncPtr) NULL, 0},	/* -majorcode		*/
	{(EventParseFuncPtr) NULL, 0},	/* -messagetype		*/
	{(EventParseFuncPtr) NULL, 0},	/* -minorcode		*/
	{(EventParseFuncPtr) NULL, 0},	/* -mode		*/
	{(EventParseFuncPtr) NULL, 0},	/* -new			*/
	{(EventParseFuncPtr) NULL, 0},	/* -overrideredirect	*/
	{(EventParseFuncPtr) NULL, 0},	/* -owner		*/
	{(EventParseFuncPtr) NULL, 0},	/* -parent		*/
	{(EventParseFuncPtr) NULL, 0},	/* -place		*/
	{(EventParseFuncPtr) NULL, 0},	/* -property		*/
	{(EventParseFuncPtr) NULL, 0},	/* -request		*/
	{(EventParseFuncPtr) NULL, 0},	/* -requestor		*/
	{(EventParseFuncPtr) NULL, 0},	/* -root		*/
	{(EventParseFuncPtr) NULL, 0},	/* -samescreen		*/
	{(EventParseFuncPtr) NULL, 0},	/* -selection		*/
	{ParseEventBool, Tk_Offset(XExposeEvent, send_event)},
	{ParseEventInt, Tk_Offset(XExposeEvent, serial)},
	{(EventParseFuncPtr) NULL, 0},	/* -state		*/
	{(EventParseFuncPtr) NULL, 0},	/* -subwindow		*/
	{(EventParseFuncPtr) NULL, 0},	/* -target		*/
	{(EventParseFuncPtr) NULL, 0},	/* -time		*/
	{ParseEventInt, Tk_Offset(XExposeEvent, width)},
	{ParseEventWindowId, Tk_Offset(XExposeEvent, window)},
	{ParseEventInt, Tk_Offset(XExposeEvent, x)},
	{(EventParseFuncPtr) NULL, 0},	/* -xroot		*/
	{ParseEventInt, Tk_Offset(XExposeEvent, y)},
	{(EventParseFuncPtr) NULL, 0}	/* -yroot		*/
    },

    /* FocusIn */

    {
	{(EventParseFuncPtr) NULL, 0},	/* -above		*/
	{(EventParseFuncPtr) NULL, 0},	/* -atom		*/
	{(EventParseFuncPtr) NULL, 0},	/* -borderwidth		*/
	{(EventParseFuncPtr) NULL, 0},	/* -button		*/
	{(EventParseFuncPtr) NULL, 0},	/* -colormap		*/
	{(EventParseFuncPtr) NULL, 0},	/* -count		*/
	{(EventParseFuncPtr) NULL, 0},	/* -data		*/
	{ParseELFDetail, Tk_Offset(XFocusChangeEvent, detail)},
	{(EventParseFuncPtr) NULL, 0},	/* -drawable		*/
	{(EventParseFuncPtr) NULL, 0},	/* -event		*/
	{(EventParseFuncPtr) NULL, 0},	/* -firstkey		*/
	{(EventParseFuncPtr) NULL, 0},	/* -focus		*/
	{(EventParseFuncPtr) NULL, 0},	/* -format		*/
	{(EventParseFuncPtr) NULL, 0},	/* -fromconfigure	*/
	{(EventParseFuncPtr) NULL, 0},	/* -height		*/
	{(EventParseFuncPtr) NULL, 0},	/* -ishint		*/
	{(EventParseFuncPtr) NULL, 0},	/* -keycode		*/
	{(EventParseFuncPtr) NULL, 0},	/* -keyvector		*/
	{(EventParseFuncPtr) NULL, 0},	/* -majorcode		*/
	{(EventParseFuncPtr) NULL, 0},	/* -messagetype		*/
	{(EventParseFuncPtr) NULL, 0},	/* -minorcode		*/
	{ParseELFMode, Tk_Offset(XFocusChangeEvent, mode)},
	{(EventParseFuncPtr) NULL, 0},	/* -new			*/
	{(EventParseFuncPtr) NULL, 0},	/* -overrideredirect	*/
	{(EventParseFuncPtr) NULL, 0},	/* -owner		*/
	{(EventParseFuncPtr) NULL, 0},	/* -parent		*/
	{(EventParseFuncPtr) NULL, 0},	/* -place		*/
	{(EventParseFuncPtr) NULL, 0},	/* -property		*/
	{(EventParseFuncPtr) NULL, 0},	/* -request		*/
	{(EventParseFuncPtr) NULL, 0},	/* -requestor		*/
	{(EventParseFuncPtr) NULL, 0},	/* -root		*/
	{(EventParseFuncPtr) NULL, 0},	/* -samescreen		*/
	{(EventParseFuncPtr) NULL, 0},	/* -selection		*/
	{ParseEventBool, Tk_Offset(XFocusChangeEvent, send_event)},
	{ParseEventInt, Tk_Offset(XFocusChangeEvent, serial)},
	{(EventParseFuncPtr) NULL, 0},	/* -state		*/
	{(EventParseFuncPtr) NULL, 0},	/* -subwindow		*/
	{(EventParseFuncPtr) NULL, 0},	/* -target		*/
	{(EventParseFuncPtr) NULL, 0},	/* -time		*/
	{(EventParseFuncPtr) NULL, 0},	/* -width		*/
	{ParseEventWindowId, Tk_Offset(XFocusChangeEvent, window)},
	{(EventParseFuncPtr) NULL, 0},	/* -x			*/
	{(EventParseFuncPtr) NULL, 0},	/* -xroot		*/
	{(EventParseFuncPtr) NULL, 0},	/* -y			*/
	{(EventParseFuncPtr) NULL, 0}	/* -yroot		*/
    },

    /* FocusOut */

    {
	{(EventParseFuncPtr) NULL, 0},	/* -above		*/
	{(EventParseFuncPtr) NULL, 0},	/* -atom		*/
	{(EventParseFuncPtr) NULL, 0},	/* -borderwidth		*/
	{(EventParseFuncPtr) NULL, 0},	/* -button		*/
	{(EventParseFuncPtr) NULL, 0},	/* -colormap		*/
	{(EventParseFuncPtr) NULL, 0},	/* -count		*/
	{(EventParseFuncPtr) NULL, 0},	/* -data		*/
	{ParseELFDetail, Tk_Offset(XFocusChangeEvent, detail)},
	{(EventParseFuncPtr) NULL, 0},	/* -drawable		*/
	{(EventParseFuncPtr) NULL, 0},	/* -event		*/
	{(EventParseFuncPtr) NULL, 0},	/* -firstkey		*/
	{(EventParseFuncPtr) NULL, 0},	/* -focus		*/
	{(EventParseFuncPtr) NULL, 0},	/* -format		*/
	{(EventParseFuncPtr) NULL, 0},	/* -fromconfigure	*/
	{(EventParseFuncPtr) NULL, 0},	/* -height		*/
	{(EventParseFuncPtr) NULL, 0},	/* -ishint		*/
	{(EventParseFuncPtr) NULL, 0},	/* -keycode		*/
	{(EventParseFuncPtr) NULL, 0},	/* -keyvector		*/
	{(EventParseFuncPtr) NULL, 0},	/* -majorcode		*/
	{(EventParseFuncPtr) NULL, 0},	/* -messagetype		*/
	{(EventParseFuncPtr) NULL, 0},	/* -minorcode		*/
	{ParseELFMode, Tk_Offset(XFocusChangeEvent, mode)},
	{(EventParseFuncPtr) NULL, 0},	/* -new			*/
	{(EventParseFuncPtr) NULL, 0},	/* -overrideredirect	*/
	{(EventParseFuncPtr) NULL, 0},	/* -owner		*/
	{(EventParseFuncPtr) NULL, 0},	/* -parent		*/
	{(EventParseFuncPtr) NULL, 0},	/* -place		*/
	{(EventParseFuncPtr) NULL, 0},	/* -property		*/
	{(EventParseFuncPtr) NULL, 0},	/* -request		*/
	{(EventParseFuncPtr) NULL, 0},	/* -requestor		*/
	{(EventParseFuncPtr) NULL, 0},	/* -root		*/
	{(EventParseFuncPtr) NULL, 0},	/* -samescreen		*/
	{(EventParseFuncPtr) NULL, 0},	/* -selection		*/
	{ParseEventBool, Tk_Offset(XFocusChangeEvent, send_event)},
	{ParseEventInt, Tk_Offset(XFocusChangeEvent, serial)},
	{(EventParseFuncPtr) NULL, 0},	/* -state		*/
	{(EventParseFuncPtr) NULL, 0},	/* -subwindow		*/
	{(EventParseFuncPtr) NULL, 0},	/* -target		*/
	{(EventParseFuncPtr) NULL, 0},	/* -time		*/
	{(EventParseFuncPtr) NULL, 0},	/* -width		*/
	{ParseEventWindowId, Tk_Offset(XFocusChangeEvent, window)},
	{(EventParseFuncPtr) NULL, 0},	/* -x			*/
	{(EventParseFuncPtr) NULL, 0},	/* -xroot		*/
	{(EventParseFuncPtr) NULL, 0},	/* -y			*/
	{(EventParseFuncPtr) NULL, 0}	/* -yroot		*/
    },

    /* GraphicsExpose */

    {
	{(EventParseFuncPtr) NULL, 0},	/* -above		*/
	{(EventParseFuncPtr) NULL, 0},	/* -atom		*/
	{(EventParseFuncPtr) NULL, 0},	/* -borderwidth		*/
	{(EventParseFuncPtr) NULL, 0},	/* -button		*/
	{(EventParseFuncPtr) NULL, 0},	/* -colormap		*/
	{ParseEventInt, Tk_Offset(XGraphicsExposeEvent, count)},
	{(EventParseFuncPtr) NULL, 0},	/* -data		*/
	{(EventParseFuncPtr) NULL, 0},	/* -detail		*/
	{ParseEventDrawableId, Tk_Offset(XGraphicsExposeEvent, drawable)},
	{(EventParseFuncPtr) NULL, 0},	/* -event		*/
	{(EventParseFuncPtr) NULL, 0},	/* -firstkey		*/
	{(EventParseFuncPtr) NULL, 0},	/* -focus		*/
	{(EventParseFuncPtr) NULL, 0},	/* -format		*/
	{(EventParseFuncPtr) NULL, 0},	/* -fromconfigure	*/
	{ParseEventInt, Tk_Offset(XGraphicsExposeEvent, height)},
	{(EventParseFuncPtr) NULL, 0},	/* -ishint		*/
	{(EventParseFuncPtr) NULL, 0},	/* -keycode		*/
	{(EventParseFuncPtr) NULL, 0},	/* -keyvector		*/
	{ParseMajorCode, Tk_Offset(XGraphicsExposeEvent, major_code)},
	{(EventParseFuncPtr) NULL, 0},	/* -messagetype		*/
	{ParseEventInt, Tk_Offset(XGraphicsExposeEvent, minor_code)},
	{(EventParseFuncPtr) NULL, 0},	/* -mode		*/
	{(EventParseFuncPtr) NULL, 0},	/* -new			*/
	{(EventParseFuncPtr) NULL, 0},	/* -overrideredirect	*/
	{(EventParseFuncPtr) NULL, 0},	/* -owner		*/
	{(EventParseFuncPtr) NULL, 0},	/* -parent		*/
	{(EventParseFuncPtr) NULL, 0},	/* -place		*/
	{(EventParseFuncPtr) NULL, 0},	/* -property		*/
	{(EventParseFuncPtr) NULL, 0},	/* -request		*/
	{(EventParseFuncPtr) NULL, 0},	/* -requestor		*/
	{(EventParseFuncPtr) NULL, 0},	/* -root		*/
	{(EventParseFuncPtr) NULL, 0},	/* -samescreen		*/
	{(EventParseFuncPtr) NULL, 0},	/* -selection		*/
	{ParseEventBool, Tk_Offset(XGraphicsExposeEvent, send_event)},
	{ParseEventInt, Tk_Offset(XGraphicsExposeEvent, serial)},
	{(EventParseFuncPtr) NULL, 0},	/* -state		*/
	{(EventParseFuncPtr) NULL, 0},	/* -subwindow		*/
	{(EventParseFuncPtr) NULL, 0},	/* -target		*/
	{(EventParseFuncPtr) NULL, 0},	/* -time		*/
	{ParseEventInt, Tk_Offset(XGraphicsExposeEvent, width)},
	{(EventParseFuncPtr) NULL, 0},	/* -window		*/
	{ParseEventInt, Tk_Offset(XGraphicsExposeEvent, x)},
	{(EventParseFuncPtr) NULL, 0},	/* -xroot		*/
	{ParseEventInt, Tk_Offset(XGraphicsExposeEvent, y)},
	{(EventParseFuncPtr) NULL, 0}	/* -yroot		*/
    },

    /* NoExpose */

    {
	{(EventParseFuncPtr) NULL, 0},	/* -above		*/
	{(EventParseFuncPtr) NULL, 0},	/* -atom		*/
	{(EventParseFuncPtr) NULL, 0},	/* -borderwidth		*/
	{(EventParseFuncPtr) NULL, 0},	/* -button		*/
	{(EventParseFuncPtr) NULL, 0},	/* -colormap		*/
	{(EventParseFuncPtr) NULL, 0},	/* -count		*/
	{(EventParseFuncPtr) NULL, 0},	/* -data		*/
	{(EventParseFuncPtr) NULL, 0},	/* -detail		*/
	{ParseEventDrawableId, Tk_Offset(XNoExposeEvent, drawable)},
	{(EventParseFuncPtr) NULL, 0},	/* -event		*/
	{(EventParseFuncPtr) NULL, 0},	/* -firstkey		*/
	{(EventParseFuncPtr) NULL, 0},	/* -focus		*/
	{(EventParseFuncPtr) NULL, 0},	/* -format		*/
	{(EventParseFuncPtr) NULL, 0},	/* -fromconfigure	*/
	{(EventParseFuncPtr) NULL, 0},	/* -height		*/
	{(EventParseFuncPtr) NULL, 0},	/* -ishint		*/
	{(EventParseFuncPtr) NULL, 0},	/* -keycode		*/
	{(EventParseFuncPtr) NULL, 0},	/* -keyvector		*/
	{ParseMajorCode, Tk_Offset(XNoExposeEvent, major_code)},
	{(EventParseFuncPtr) NULL, 0},	/* -messagetype		*/
	{ParseEventInt, Tk_Offset(XNoExposeEvent, minor_code)},
	{(EventParseFuncPtr) NULL, 0},	/* -mode		*/
	{(EventParseFuncPtr) NULL, 0},	/* -new			*/
	{(EventParseFuncPtr) NULL, 0},	/* -overrideredirect	*/
	{(EventParseFuncPtr) NULL, 0},	/* -owner		*/
	{(EventParseFuncPtr) NULL, 0},	/* -parent		*/
	{(EventParseFuncPtr) NULL, 0},	/* -place		*/
	{(EventParseFuncPtr) NULL, 0},	/* -property		*/
	{(EventParseFuncPtr) NULL, 0},	/* -request		*/
	{(EventParseFuncPtr) NULL, 0},	/* -requestor		*/
	{(EventParseFuncPtr) NULL, 0},	/* -root		*/
	{(EventParseFuncPtr) NULL, 0},	/* -samescreen		*/
	{(EventParseFuncPtr) NULL, 0},	/* -selection		*/
	{ParseEventBool, Tk_Offset(XNoExposeEvent, send_event)},
	{ParseEventInt, Tk_Offset(XNoExposeEvent, serial)},
	{(EventParseFuncPtr) NULL, 0},	/* -state		*/
	{(EventParseFuncPtr) NULL, 0},	/* -subwindow		*/
	{(EventParseFuncPtr) NULL, 0},	/* -target		*/
	{(EventParseFuncPtr) NULL, 0},	/* -time		*/
	{(EventParseFuncPtr) NULL, 0},	/* -width		*/
	{(EventParseFuncPtr) NULL, 0},	/* -window		*/
	{(EventParseFuncPtr) NULL, 0},	/* -x			*/
	{(EventParseFuncPtr) NULL, 0},	/* -xroot		*/
	{(EventParseFuncPtr) NULL, 0},	/* -y			*/
	{(EventParseFuncPtr) NULL, 0}	/* -yroot		*/
    },

    /* GravityNotify */

    {
	{(EventParseFuncPtr) NULL, 0},	/* -above		*/
	{(EventParseFuncPtr) NULL, 0},	/* -atom		*/
	{(EventParseFuncPtr) NULL, 0},	/* -borderwidth		*/
	{(EventParseFuncPtr) NULL, 0},	/* -button		*/
	{(EventParseFuncPtr) NULL, 0},	/* -colormap		*/
	{(EventParseFuncPtr) NULL, 0},	/* -count		*/
	{(EventParseFuncPtr) NULL, 0},	/* -data		*/
	{(EventParseFuncPtr) NULL, 0},	/* -detail		*/
	{(EventParseFuncPtr) NULL, 0},	/* -drawable		*/
	{ParseEventWindowId, Tk_Offset(XGravityEvent, event)},
	{(EventParseFuncPtr) NULL, 0},	/* -firstkey		*/
	{(EventParseFuncPtr) NULL, 0},	/* -focus		*/
	{(EventParseFuncPtr) NULL, 0},	/* -format		*/
	{(EventParseFuncPtr) NULL, 0},	/* -fromconfigure	*/
	{(EventParseFuncPtr) NULL, 0},	/* -height		*/
	{(EventParseFuncPtr) NULL, 0},	/* -ishint		*/
	{(EventParseFuncPtr) NULL, 0},	/* -keycode		*/
	{(EventParseFuncPtr) NULL, 0},	/* -keyvector		*/
	{(EventParseFuncPtr) NULL, 0},	/* -majorcode		*/
	{(EventParseFuncPtr) NULL, 0},	/* -messagetype		*/
	{(EventParseFuncPtr) NULL, 0},	/* -minorcode		*/
	{(EventParseFuncPtr) NULL, 0},	/* -mode		*/
	{(EventParseFuncPtr) NULL, 0},	/* -new			*/
	{(EventParseFuncPtr) NULL, 0},	/* -overrideredirect	*/
	{(EventParseFuncPtr) NULL, 0},	/* -owner		*/
	{(EventParseFuncPtr) NULL, 0},	/* -parent		*/
	{(EventParseFuncPtr) NULL, 0},	/* -place		*/
	{(EventParseFuncPtr) NULL, 0},	/* -property		*/
	{(EventParseFuncPtr) NULL, 0},	/* -request		*/
	{(EventParseFuncPtr) NULL, 0},	/* -requestor		*/
	{(EventParseFuncPtr) NULL, 0},	/* -root		*/
	{(EventParseFuncPtr) NULL, 0},	/* -samescreen		*/
	{(EventParseFuncPtr) NULL, 0},	/* -selection		*/
	{ParseEventBool, Tk_Offset(XGravityEvent, send_event)},
	{ParseEventInt, Tk_Offset(XGravityEvent, serial)},
	{(EventParseFuncPtr) NULL, 0},	/* -state		*/
	{(EventParseFuncPtr) NULL, 0},	/* -subwindow		*/
	{(EventParseFuncPtr) NULL, 0},	/* -target		*/
	{(EventParseFuncPtr) NULL, 0},	/* -time		*/
	{(EventParseFuncPtr) NULL, 0},	/* -width		*/
	{ParseEventWindowId, Tk_Offset(XGravityEvent, window)},
	{ParseEventInt, Tk_Offset(XGravityEvent, x)},
	{(EventParseFuncPtr) NULL, 0},	/* -xroot		*/
	{ParseEventInt, Tk_Offset(XGravityEvent, y)},
	{(EventParseFuncPtr) NULL, 0}	/* -yroot		*/
    },

    /* KeyPress */

    {
	{(EventParseFuncPtr) NULL, 0},	/* -above		*/
	{(EventParseFuncPtr) NULL, 0},	/* -atom		*/
	{(EventParseFuncPtr) NULL, 0},	/* -borderwidth		*/
	{(EventParseFuncPtr) NULL, 0},	/* -button		*/
	{(EventParseFuncPtr) NULL, 0},	/* -colormap		*/
	{(EventParseFuncPtr) NULL, 0},	/* -count		*/
	{(EventParseFuncPtr) NULL, 0},	/* -data		*/
	{(EventParseFuncPtr) NULL, 0},	/* -detail		*/
	{(EventParseFuncPtr) NULL, 0},	/* -drawable		*/
	{(EventParseFuncPtr) NULL, 0},	/* -event		*/
	{(EventParseFuncPtr) NULL, 0},	/* -firstkey		*/
	{(EventParseFuncPtr) NULL, 0},	/* -focus		*/
	{(EventParseFuncPtr) NULL, 0},	/* -format		*/
	{(EventParseFuncPtr) NULL, 0},	/* -fromconfigure	*/
	{(EventParseFuncPtr) NULL, 0},	/* -height		*/
	{(EventParseFuncPtr) NULL, 0},	/* -ishint		*/
	{ParseEventInt, Tk_Offset(XKeyEvent, keycode)},
	{(EventParseFuncPtr) NULL, 0},	/* -keyvector		*/
	{(EventParseFuncPtr) NULL, 0},	/* -majorcode		*/
	{(EventParseFuncPtr) NULL, 0},	/* -messagetype		*/
	{(EventParseFuncPtr) NULL, 0},	/* -minorcode		*/
	{(EventParseFuncPtr) NULL, 0},	/* -mode		*/
	{(EventParseFuncPtr) NULL, 0},	/* -new			*/
	{(EventParseFuncPtr) NULL, 0},	/* -overrideredirect	*/
	{(EventParseFuncPtr) NULL, 0},	/* -owner		*/
	{(EventParseFuncPtr) NULL, 0},	/* -parent		*/
	{(EventParseFuncPtr) NULL, 0},	/* -place		*/
	{(EventParseFuncPtr) NULL, 0},	/* -property		*/
	{(EventParseFuncPtr) NULL, 0},	/* -request		*/
	{(EventParseFuncPtr) NULL, 0},	/* -requestor		*/
	{ParseEventWindowId, Tk_Offset(XKeyEvent, root)},
	{ParseEventBool, Tk_Offset(XKeyEvent, same_screen)},
	{(EventParseFuncPtr) NULL, 0},	/* -selection		*/
	{ParseEventBool, Tk_Offset(XKeyEvent, send_event)},
	{ParseEventInt, Tk_Offset(XKeyEvent, serial)},
	{ParseEventModMask, Tk_Offset(XKeyEvent, state)},
	{ParseEventWindowId, Tk_Offset(XKeyEvent, subwindow)},
	{(EventParseFuncPtr) NULL, 0},	/* -target		*/
	{ParseEventInt, Tk_Offset(XKeyEvent, time)},
	{(EventParseFuncPtr) NULL, 0},	/* -width		*/
	{ParseEventWindowId, Tk_Offset(XKeyEvent, window)},
	{ParseEventInt, Tk_Offset(XKeyEvent, x)},
	{ParseEventInt, Tk_Offset(XKeyEvent, x_root)},
	{ParseEventInt, Tk_Offset(XKeyEvent, y)},
	{ParseEventInt, Tk_Offset(XKeyEvent, y_root)},
    },

    /* KeyRelease */

    {
	{(EventParseFuncPtr) NULL, 0},	/* -above		*/
	{(EventParseFuncPtr) NULL, 0},	/* -atom		*/
	{(EventParseFuncPtr) NULL, 0},	/* -borderwidth		*/
	{(EventParseFuncPtr) NULL, 0},	/* -button		*/
	{(EventParseFuncPtr) NULL, 0},	/* -colormap		*/
	{(EventParseFuncPtr) NULL, 0},	/* -count		*/
	{(EventParseFuncPtr) NULL, 0},	/* -data		*/
	{(EventParseFuncPtr) NULL, 0},	/* -detail		*/
	{(EventParseFuncPtr) NULL, 0},	/* -drawable		*/
	{(EventParseFuncPtr) NULL, 0},	/* -event		*/
	{(EventParseFuncPtr) NULL, 0},	/* -firstkey		*/
	{(EventParseFuncPtr) NULL, 0},	/* -focus		*/
	{(EventParseFuncPtr) NULL, 0},	/* -format		*/
	{(EventParseFuncPtr) NULL, 0},	/* -fromconfigure	*/
	{(EventParseFuncPtr) NULL, 0},	/* -height		*/
	{(EventParseFuncPtr) NULL, 0},	/* -ishint		*/
	{ParseEventInt, Tk_Offset(XKeyEvent, keycode)},
	{(EventParseFuncPtr) NULL, 0},	/* -keyvector		*/
	{(EventParseFuncPtr) NULL, 0},	/* -majorcode		*/
	{(EventParseFuncPtr) NULL, 0},	/* -messagetype		*/
	{(EventParseFuncPtr) NULL, 0},	/* -minorcode		*/
	{(EventParseFuncPtr) NULL, 0},	/* -mode		*/
	{(EventParseFuncPtr) NULL, 0},	/* -new			*/
	{(EventParseFuncPtr) NULL, 0},	/* -overrideredirect	*/
	{(EventParseFuncPtr) NULL, 0},	/* -owner		*/
	{(EventParseFuncPtr) NULL, 0},	/* -parent		*/
	{(EventParseFuncPtr) NULL, 0},	/* -place		*/
	{(EventParseFuncPtr) NULL, 0},	/* -property		*/
	{(EventParseFuncPtr) NULL, 0},	/* -request		*/
	{(EventParseFuncPtr) NULL, 0},	/* -requestor		*/
	{ParseEventWindowId, Tk_Offset(XKeyEvent, root)},
	{ParseEventBool, Tk_Offset(XKeyEvent, same_screen)},
	{(EventParseFuncPtr) NULL, 0},	/* -selection		*/
	{ParseEventBool, Tk_Offset(XKeyEvent, send_event)},
	{ParseEventInt, Tk_Offset(XKeyEvent, serial)},
	{ParseEventModMask, Tk_Offset(XKeyEvent, state)},
	{ParseEventWindowId, Tk_Offset(XKeyEvent, subwindow)},
	{(EventParseFuncPtr) NULL, 0},	/* -target		*/
	{ParseEventInt, Tk_Offset(XKeyEvent, time)},
	{(EventParseFuncPtr) NULL, 0},	/* -width		*/
	{ParseEventWindowId, Tk_Offset(XKeyEvent, window)},
	{ParseEventInt, Tk_Offset(XKeyEvent, x)},
	{ParseEventInt, Tk_Offset(XKeyEvent, x_root)},
	{ParseEventInt, Tk_Offset(XKeyEvent, y)},
	{ParseEventInt, Tk_Offset(XKeyEvent, y_root)},
    },

    /* KeymapNotify */

    {
	{(EventParseFuncPtr) NULL, 0},	/* -above		*/
	{(EventParseFuncPtr) NULL, 0},	/* -atom		*/
	{(EventParseFuncPtr) NULL, 0},	/* -borderwidth		*/
	{(EventParseFuncPtr) NULL, 0},	/* -button		*/
	{(EventParseFuncPtr) NULL, 0},	/* -colormap		*/
	{(EventParseFuncPtr) NULL, 0},	/* -count		*/
	{(EventParseFuncPtr) NULL, 0},	/* -data		*/
	{(EventParseFuncPtr) NULL, 0},	/* -detail		*/
	{(EventParseFuncPtr) NULL, 0},	/* -drawable		*/
	{(EventParseFuncPtr) NULL, 0},	/* -event		*/
	{(EventParseFuncPtr) NULL, 0},	/* -firstkey		*/
	{(EventParseFuncPtr) NULL, 0},	/* -focus		*/
	{(EventParseFuncPtr) NULL, 0},	/* -format		*/
	{(EventParseFuncPtr) NULL, 0},	/* -fromconfigure	*/
	{(EventParseFuncPtr) NULL, 0},	/* -height		*/
	{(EventParseFuncPtr) NULL, 0},	/* -ishint		*/
	{(EventParseFuncPtr) NULL, 0},	/* -keycode		*/
	{ParseKeyVector, Tk_Offset(XKeymapEvent, key_vector)},
	{(EventParseFuncPtr) NULL, 0},	/* -majorcode		*/
	{(EventParseFuncPtr) NULL, 0},	/* -messagetype		*/
	{(EventParseFuncPtr) NULL, 0},	/* -minorcode		*/
	{(EventParseFuncPtr) NULL, 0},	/* -mode		*/
	{(EventParseFuncPtr) NULL, 0},	/* -new			*/
	{(EventParseFuncPtr) NULL, 0},	/* -overrideredirect	*/
	{(EventParseFuncPtr) NULL, 0},	/* -owner		*/
	{(EventParseFuncPtr) NULL, 0},	/* -parent		*/
	{(EventParseFuncPtr) NULL, 0},	/* -place		*/
	{(EventParseFuncPtr) NULL, 0},	/* -property		*/
	{(EventParseFuncPtr) NULL, 0},	/* -request		*/
	{(EventParseFuncPtr) NULL, 0},	/* -requestor		*/
	{(EventParseFuncPtr) NULL, 0},	/* -root		*/
	{(EventParseFuncPtr) NULL, 0},	/* -samescreen		*/
	{(EventParseFuncPtr) NULL, 0},	/* -selection		*/
	{ParseEventBool, Tk_Offset(XKeymapEvent, send_event)},
	{ParseEventInt, Tk_Offset(XKeymapEvent, serial)},
	{(EventParseFuncPtr) NULL, 0},	/* -state		*/
	{(EventParseFuncPtr) NULL, 0},	/* -subwindow		*/
	{(EventParseFuncPtr) NULL, 0},	/* -target		*/
	{(EventParseFuncPtr) NULL, 0},	/* -time		*/
	{(EventParseFuncPtr) NULL, 0},	/* -width		*/
	{ParseEventWindowId, Tk_Offset(XKeymapEvent, window)},
	{(EventParseFuncPtr) NULL, 0},	/* -x			*/
	{(EventParseFuncPtr) NULL, 0},	/* -xroot		*/
	{(EventParseFuncPtr) NULL, 0},	/* -y			*/
	{(EventParseFuncPtr) NULL, 0}	/* -yroot		*/
    },

    /* MapNotify */

    {
	{(EventParseFuncPtr) NULL, 0},	/* -above		*/
	{(EventParseFuncPtr) NULL, 0},	/* -atom		*/
	{(EventParseFuncPtr) NULL, 0},	/* -borderwidth		*/
	{(EventParseFuncPtr) NULL, 0},	/* -button		*/
	{(EventParseFuncPtr) NULL, 0},	/* -colormap		*/
	{(EventParseFuncPtr) NULL, 0},	/* -count		*/
	{(EventParseFuncPtr) NULL, 0},	/* -data		*/
	{(EventParseFuncPtr) NULL, 0},	/* -detail		*/
	{(EventParseFuncPtr) NULL, 0},	/* -drawable		*/
	{ParseEventWindowId, Tk_Offset(XMapEvent, event)},
	{(EventParseFuncPtr) NULL, 0},	/* -firstkey		*/
	{(EventParseFuncPtr) NULL, 0},	/* -focus		*/
	{(EventParseFuncPtr) NULL, 0},	/* -format		*/
	{(EventParseFuncPtr) NULL, 0},	/* -fromconfigure	*/
	{(EventParseFuncPtr) NULL, 0},	/* -height		*/
	{(EventParseFuncPtr) NULL, 0},	/* -ishint		*/
	{(EventParseFuncPtr) NULL, 0},	/* -keycode		*/
	{(EventParseFuncPtr) NULL, 0},	/* -keyvector		*/
	{(EventParseFuncPtr) NULL, 0},	/* -majorcode		*/
	{(EventParseFuncPtr) NULL, 0},	/* -messagetype		*/
	{(EventParseFuncPtr) NULL, 0},	/* -minorcode		*/
	{(EventParseFuncPtr) NULL, 0},	/* -mode		*/
	{(EventParseFuncPtr) NULL, 0},	/* -new			*/
	{ParseEventBool, Tk_Offset(XMapEvent, override_redirect)},
	{(EventParseFuncPtr) NULL, 0},	/* -owner		*/
	{(EventParseFuncPtr) NULL, 0},	/* -parent		*/
	{(EventParseFuncPtr) NULL, 0},	/* -place		*/
	{(EventParseFuncPtr) NULL, 0},	/* -property		*/
	{(EventParseFuncPtr) NULL, 0},	/* -request		*/
	{(EventParseFuncPtr) NULL, 0},	/* -requestor		*/
	{(EventParseFuncPtr) NULL, 0},	/* -root		*/
	{(EventParseFuncPtr) NULL, 0},	/* -samescreen		*/
	{(EventParseFuncPtr) NULL, 0},	/* -selection		*/
	{ParseEventBool, Tk_Offset(XMapEvent, send_event)},
	{ParseEventInt, Tk_Offset(XMapEvent, serial)},
	{(EventParseFuncPtr) NULL, 0},	/* -state		*/
	{(EventParseFuncPtr) NULL, 0},	/* -subwindow		*/
	{(EventParseFuncPtr) NULL, 0},	/* -target		*/
	{(EventParseFuncPtr) NULL, 0},	/* -time		*/
	{(EventParseFuncPtr) NULL, 0},	/* -width		*/
	{ParseEventWindowId, Tk_Offset(XMapEvent, window)},
	{(EventParseFuncPtr) NULL, 0},	/* -x			*/
	{(EventParseFuncPtr) NULL, 0},	/* -xroot		*/
	{(EventParseFuncPtr) NULL, 0},	/* -y			*/
	{(EventParseFuncPtr) NULL, 0}	/* -yroot		*/
    },

    /* UnmapNotify */

    {
	{(EventParseFuncPtr) NULL, 0},	/* -above		*/
	{(EventParseFuncPtr) NULL, 0},	/* -atom		*/
	{(EventParseFuncPtr) NULL, 0},	/* -borderwidth		*/
	{(EventParseFuncPtr) NULL, 0},	/* -button		*/
	{(EventParseFuncPtr) NULL, 0},	/* -colormap		*/
	{(EventParseFuncPtr) NULL, 0},	/* -count		*/
	{(EventParseFuncPtr) NULL, 0},	/* -data		*/
	{(EventParseFuncPtr) NULL, 0},	/* -detail		*/
	{(EventParseFuncPtr) NULL, 0},	/* -drawable		*/
	{ParseEventWindowId, Tk_Offset(XUnmapEvent, event)},
	{(EventParseFuncPtr) NULL, 0},	/* -firstkey		*/
	{(EventParseFuncPtr) NULL, 0},	/* -focus		*/
	{(EventParseFuncPtr) NULL, 0},	/* -format		*/
	{ParseEventBool, Tk_Offset(XUnmapEvent, from_configure)},
	{(EventParseFuncPtr) NULL, 0},	/* -height		*/
	{(EventParseFuncPtr) NULL, 0},	/* -ishint		*/
	{(EventParseFuncPtr) NULL, 0},	/* -keycode		*/
	{(EventParseFuncPtr) NULL, 0},	/* -keyvector		*/
	{(EventParseFuncPtr) NULL, 0},	/* -majorcode		*/
	{(EventParseFuncPtr) NULL, 0},	/* -messagetype		*/
	{(EventParseFuncPtr) NULL, 0},	/* -minorcode		*/
	{(EventParseFuncPtr) NULL, 0},	/* -mode		*/
	{(EventParseFuncPtr) NULL, 0},	/* -new			*/
	{(EventParseFuncPtr) NULL, 0},	/* -overrideredirect	*/
	{(EventParseFuncPtr) NULL, 0},	/* -owner		*/
	{(EventParseFuncPtr) NULL, 0},	/* -parent		*/
	{(EventParseFuncPtr) NULL, 0},	/* -place		*/
	{(EventParseFuncPtr) NULL, 0},	/* -property		*/
	{(EventParseFuncPtr) NULL, 0},	/* -request		*/
	{(EventParseFuncPtr) NULL, 0},	/* -requestor		*/
	{(EventParseFuncPtr) NULL, 0},	/* -root		*/
	{(EventParseFuncPtr) NULL, 0},	/* -samescreen		*/
	{(EventParseFuncPtr) NULL, 0},	/* -selection		*/
	{ParseEventBool, Tk_Offset(XUnmapEvent, send_event)},
	{ParseEventInt, Tk_Offset(XUnmapEvent, serial)},
	{(EventParseFuncPtr) NULL, 0},	/* -state		*/
	{(EventParseFuncPtr) NULL, 0},	/* -subwindow		*/
	{(EventParseFuncPtr) NULL, 0},	/* -target		*/
	{(EventParseFuncPtr) NULL, 0},	/* -time		*/
	{(EventParseFuncPtr) NULL, 0},	/* -width		*/
	{ParseEventWindowId, Tk_Offset(XUnmapEvent, window)},
	{(EventParseFuncPtr) NULL, 0},	/* -x			*/
	{(EventParseFuncPtr) NULL, 0},	/* -xroot		*/
	{(EventParseFuncPtr) NULL, 0},	/* -y			*/
	{(EventParseFuncPtr) NULL, 0}	/* -yroot		*/
    },

    /* MapRequest */

    {
	{(EventParseFuncPtr) NULL, 0},	/* -above		*/
	{(EventParseFuncPtr) NULL, 0},	/* -atom		*/
	{(EventParseFuncPtr) NULL, 0},	/* -borderwidth		*/
	{(EventParseFuncPtr) NULL, 0},	/* -button		*/
	{(EventParseFuncPtr) NULL, 0},	/* -colormap		*/
	{(EventParseFuncPtr) NULL, 0},	/* -count		*/
	{(EventParseFuncPtr) NULL, 0},	/* -data		*/
	{(EventParseFuncPtr) NULL, 0},	/* -detail		*/
	{(EventParseFuncPtr) NULL, 0},	/* -drawable		*/
	{(EventParseFuncPtr) NULL, 0},	/* -event		*/
	{(EventParseFuncPtr) NULL, 0},	/* -firstkey		*/
	{(EventParseFuncPtr) NULL, 0},	/* -focus		*/
	{(EventParseFuncPtr) NULL, 0},	/* -format		*/
	{(EventParseFuncPtr) NULL, 0},	/* -fromconfigure	*/
	{(EventParseFuncPtr) NULL, 0},	/* -height		*/
	{(EventParseFuncPtr) NULL, 0},	/* -ishint		*/
	{(EventParseFuncPtr) NULL, 0},	/* -keycode		*/
	{(EventParseFuncPtr) NULL, 0},	/* -keyvector		*/
	{(EventParseFuncPtr) NULL, 0},	/* -majorcode		*/
	{(EventParseFuncPtr) NULL, 0},	/* -messagetype		*/
	{(EventParseFuncPtr) NULL, 0},	/* -minorcode		*/
	{(EventParseFuncPtr) NULL, 0},	/* -mode		*/
	{(EventParseFuncPtr) NULL, 0},	/* -new			*/
	{(EventParseFuncPtr) NULL, 0},	/* -overrideredirect	*/
	{(EventParseFuncPtr) NULL, 0},	/* -owner		*/
	{ParseEventWindowId, Tk_Offset(XMapRequestEvent, parent)},
	{(EventParseFuncPtr) NULL, 0},	/* -place		*/
	{(EventParseFuncPtr) NULL, 0},	/* -property		*/
	{(EventParseFuncPtr) NULL, 0},	/* -request		*/
	{(EventParseFuncPtr) NULL, 0},	/* -requestor		*/
	{(EventParseFuncPtr) NULL, 0},	/* -root		*/
	{(EventParseFuncPtr) NULL, 0},	/* -samescreen		*/
	{(EventParseFuncPtr) NULL, 0},	/* -selection		*/
	{ParseEventBool, Tk_Offset(XMapRequestEvent, send_event)},
	{ParseEventInt, Tk_Offset(XMapRequestEvent, serial)},
	{(EventParseFuncPtr) NULL, 0},	/* -state		*/
	{(EventParseFuncPtr) NULL, 0},	/* -subwindow		*/
	{(EventParseFuncPtr) NULL, 0},	/* -target		*/
	{(EventParseFuncPtr) NULL, 0},	/* -time		*/
	{(EventParseFuncPtr) NULL, 0},	/* -width		*/
	{ParseEventWindowId, Tk_Offset(XMapRequestEvent, window)},
	{(EventParseFuncPtr) NULL, 0},	/* -x			*/
	{(EventParseFuncPtr) NULL, 0},	/* -xroot		*/
	{(EventParseFuncPtr) NULL, 0},	/* -y			*/
	{(EventParseFuncPtr) NULL, 0}	/* -yroot		*/
    },

    /* MappingNotify */

    {
	{(EventParseFuncPtr) NULL, 0},	/* -above		*/
	{(EventParseFuncPtr) NULL, 0},	/* -atom		*/
	{(EventParseFuncPtr) NULL, 0},	/* -borderwidth		*/
	{(EventParseFuncPtr) NULL, 0},	/* -button		*/
	{(EventParseFuncPtr) NULL, 0},	/* -colormap		*/
	{ParseEventInt, Tk_Offset(XMappingEvent, count)},
	{(EventParseFuncPtr) NULL, 0},	/* -data		*/
	{(EventParseFuncPtr) NULL, 0},	/* -detail		*/
	{(EventParseFuncPtr) NULL, 0},	/* -drawable		*/
	{(EventParseFuncPtr) NULL, 0},	/* -event		*/
	{ParseEventInt, Tk_Offset(XMappingEvent, first_keycode)},
	{(EventParseFuncPtr) NULL, 0},	/* -focus		*/
	{(EventParseFuncPtr) NULL, 0},	/* -format		*/
	{(EventParseFuncPtr) NULL, 0},	/* -fromconfigure	*/
	{(EventParseFuncPtr) NULL, 0},	/* -height		*/
	{(EventParseFuncPtr) NULL, 0},	/* -ishint		*/
	{(EventParseFuncPtr) NULL, 0},	/* -keycode		*/
	{(EventParseFuncPtr) NULL, 0},	/* -keyvector		*/
	{(EventParseFuncPtr) NULL, 0},	/* -majorcode		*/
	{(EventParseFuncPtr) NULL, 0},	/* -messagetype		*/
	{(EventParseFuncPtr) NULL, 0},	/* -minorcode		*/
	{(EventParseFuncPtr) NULL, 0},	/* -mode		*/
	{(EventParseFuncPtr) NULL, 0},	/* -new			*/
	{(EventParseFuncPtr) NULL, 0},	/* -overrideredirect	*/
	{(EventParseFuncPtr) NULL, 0},	/* -owner		*/
	{(EventParseFuncPtr) NULL, 0},	/* -parent		*/
	{(EventParseFuncPtr) NULL, 0},	/* -place		*/
	{(EventParseFuncPtr) NULL, 0},	/* -property		*/
	{ParseMappingRequest, Tk_Offset(XMappingEvent, request)},
	{(EventParseFuncPtr) NULL, 0},	/* -requestor		*/
	{(EventParseFuncPtr) NULL, 0},	/* -root		*/
	{(EventParseFuncPtr) NULL, 0},	/* -samescreen		*/
	{(EventParseFuncPtr) NULL, 0},	/* -selection		*/
	{ParseEventBool, Tk_Offset(XMappingEvent, send_event)},
	{ParseEventInt, Tk_Offset(XMappingEvent, serial)},
	{(EventParseFuncPtr) NULL, 0},	/* -state		*/
	{(EventParseFuncPtr) NULL, 0},	/* -subwindow		*/
	{(EventParseFuncPtr) NULL, 0},	/* -target		*/
	{(EventParseFuncPtr) NULL, 0},	/* -time		*/
	{(EventParseFuncPtr) NULL, 0},	/* -width		*/
	{ParseEventWindowId, Tk_Offset(XMappingEvent, window)},
	{(EventParseFuncPtr) NULL, 0},	/* -x			*/
	{(EventParseFuncPtr) NULL, 0},	/* -xroot		*/
	{(EventParseFuncPtr) NULL, 0},	/* -y			*/
	{(EventParseFuncPtr) NULL, 0}	/* -yroot		*/
    },

    /* MotionNotify */

    {
	{(EventParseFuncPtr) NULL, 0},	/* -above		*/
	{(EventParseFuncPtr) NULL, 0},	/* -atom		*/
	{(EventParseFuncPtr) NULL, 0},	/* -borderwidth		*/
	{(EventParseFuncPtr) NULL, 0},	/* -button		*/
	{(EventParseFuncPtr) NULL, 0},	/* -colormap		*/
	{(EventParseFuncPtr) NULL, 0},	/* -count		*/
	{(EventParseFuncPtr) NULL, 0},	/* -data		*/
	{(EventParseFuncPtr) NULL, 0},	/* -detail		*/
	{(EventParseFuncPtr) NULL, 0},	/* -drawable		*/
	{(EventParseFuncPtr) NULL, 0},	/* -event		*/
	{(EventParseFuncPtr) NULL, 0},	/* -firstkey		*/
	{(EventParseFuncPtr) NULL, 0},	/* -focus		*/
	{(EventParseFuncPtr) NULL, 0},	/* -format		*/
	{(EventParseFuncPtr) NULL, 0},	/* -fromconfigure	*/
	{(EventParseFuncPtr) NULL, 0},	/* -height		*/
	{ParseEventBool, Tk_Offset(XMotionEvent, is_hint)},
	{(EventParseFuncPtr) NULL, 0},	/* -keycode		*/
	{(EventParseFuncPtr) NULL, 0},	/* -keyvector		*/
	{(EventParseFuncPtr) NULL, 0},	/* -majorcode		*/
	{(EventParseFuncPtr) NULL, 0},	/* -messagetype		*/
	{(EventParseFuncPtr) NULL, 0},	/* -minorcode		*/
	{(EventParseFuncPtr) NULL, 0},	/* -mode		*/
	{(EventParseFuncPtr) NULL, 0},	/* -new			*/
	{(EventParseFuncPtr) NULL, 0},	/* -overrideredirect	*/
	{(EventParseFuncPtr) NULL, 0},	/* -owner		*/
	{(EventParseFuncPtr) NULL, 0},	/* -parent		*/
	{(EventParseFuncPtr) NULL, 0},	/* -place		*/
	{(EventParseFuncPtr) NULL, 0},	/* -property		*/
	{(EventParseFuncPtr) NULL, 0},	/* -request		*/
	{(EventParseFuncPtr) NULL, 0},	/* -requestor		*/
	{ParseEventWindowId, Tk_Offset(XMotionEvent, root)},
	{ParseEventBool, Tk_Offset(XMotionEvent, same_screen)},
	{(EventParseFuncPtr) NULL, 0},	/* -selection		*/
	{ParseEventBool, Tk_Offset(XMotionEvent, send_event)},
	{ParseEventInt, Tk_Offset(XMotionEvent, serial)},
	{ParseEventModMask, Tk_Offset(XMotionEvent, state)},
	{ParseEventWindowId, Tk_Offset(XMotionEvent, subwindow)},
	{(EventParseFuncPtr) NULL, 0},	/* -target		*/
	{ParseEventInt, Tk_Offset(XMotionEvent, time)},
	{(EventParseFuncPtr) NULL, 0},	/* -width		*/
	{ParseEventWindowId, Tk_Offset(XMotionEvent, window)},
	{ParseEventInt, Tk_Offset(XMotionEvent, x)},
	{ParseEventInt, Tk_Offset(XMotionEvent, x_root)},
	{ParseEventInt, Tk_Offset(XMotionEvent, y)},
	{ParseEventInt, Tk_Offset(XMotionEvent, y_root)},
    },

    /* PropertyNotify */

    {
	{(EventParseFuncPtr) NULL, 0},	/* -above		*/
	{ParseEventAtom, Tk_Offset(XPropertyEvent, atom)},
	{(EventParseFuncPtr) NULL, 0},	/* -borderwidth		*/
	{(EventParseFuncPtr) NULL, 0},	/* -button		*/
	{(EventParseFuncPtr) NULL, 0},	/* -colormap		*/
	{(EventParseFuncPtr) NULL, 0},	/* -count		*/
	{(EventParseFuncPtr) NULL, 0},	/* -data		*/
	{(EventParseFuncPtr) NULL, 0},	/* -detail		*/
	{(EventParseFuncPtr) NULL, 0},	/* -drawable		*/
	{(EventParseFuncPtr) NULL, 0},	/* -event		*/
	{(EventParseFuncPtr) NULL, 0},	/* -firstkey		*/
	{(EventParseFuncPtr) NULL, 0},	/* -focus		*/
	{(EventParseFuncPtr) NULL, 0},	/* -format		*/
	{(EventParseFuncPtr) NULL, 0},	/* -fromconfigure	*/
	{(EventParseFuncPtr) NULL, 0},	/* -height		*/
	{(EventParseFuncPtr) NULL, 0},	/* -ishint		*/
	{(EventParseFuncPtr) NULL, 0},	/* -keycode		*/
	{(EventParseFuncPtr) NULL, 0},	/* -keyvector		*/
	{(EventParseFuncPtr) NULL, 0},	/* -majorcode		*/
	{(EventParseFuncPtr) NULL, 0},	/* -messagetype		*/
	{(EventParseFuncPtr) NULL, 0},	/* -minorcode		*/
	{(EventParseFuncPtr) NULL, 0},	/* -mode		*/
	{(EventParseFuncPtr) NULL, 0},	/* -new			*/
	{(EventParseFuncPtr) NULL, 0},	/* -overrideredirect	*/
	{(EventParseFuncPtr) NULL, 0},	/* -owner		*/
	{(EventParseFuncPtr) NULL, 0},	/* -parent		*/
	{(EventParseFuncPtr) NULL, 0},	/* -place		*/
	{(EventParseFuncPtr) NULL, 0},	/* -property		*/
	{(EventParseFuncPtr) NULL, 0},	/* -request		*/
	{(EventParseFuncPtr) NULL, 0},	/* -requestor		*/
	{(EventParseFuncPtr) NULL, 0},	/* -root		*/
	{(EventParseFuncPtr) NULL, 0},	/* -samescreen		*/
	{(EventParseFuncPtr) NULL, 0},	/* -selection		*/
	{ParseEventBool, Tk_Offset(XPropertyEvent, send_event)},
	{ParseEventInt, Tk_Offset(XPropertyEvent, serial)},
	{ParsePropertyState, Tk_Offset(XPropertyEvent, state)},
	{(EventParseFuncPtr) NULL, 0},	/* -subwindow		*/
	{(EventParseFuncPtr) NULL, 0},	/* -target		*/
	{ParseEventInt, Tk_Offset(XPropertyEvent, time)},
	{(EventParseFuncPtr) NULL, 0},	/* -width		*/
	{ParseEventWindowId, Tk_Offset(XPropertyEvent, window)},
	{(EventParseFuncPtr) NULL, 0},	/* -x			*/
	{(EventParseFuncPtr) NULL, 0},	/* -xroot		*/
	{(EventParseFuncPtr) NULL, 0},	/* -y			*/
	{(EventParseFuncPtr) NULL, 0}	/* -yroot		*/
    },

    /* ReparentNotify */

    {
	{(EventParseFuncPtr) NULL, 0},	/* -above		*/
	{(EventParseFuncPtr) NULL, 0},	/* -atom		*/
	{(EventParseFuncPtr) NULL, 0},	/* -borderwidth		*/
	{(EventParseFuncPtr) NULL, 0},	/* -button		*/
	{(EventParseFuncPtr) NULL, 0},	/* -colormap		*/
	{(EventParseFuncPtr) NULL, 0},	/* -count		*/
	{(EventParseFuncPtr) NULL, 0},	/* -data		*/
	{(EventParseFuncPtr) NULL, 0},	/* -detail		*/
	{(EventParseFuncPtr) NULL, 0},	/* -drawable		*/
	{ParseEventWindowId, Tk_Offset(XReparentEvent, event)},
	{(EventParseFuncPtr) NULL, 0},	/* -firstkey		*/
	{(EventParseFuncPtr) NULL, 0},	/* -focus		*/
	{(EventParseFuncPtr) NULL, 0},	/* -format		*/
	{(EventParseFuncPtr) NULL, 0},	/* -fromconfigure	*/
	{(EventParseFuncPtr) NULL, 0},	/* -height		*/
	{(EventParseFuncPtr) NULL, 0},	/* -ishint		*/
	{(EventParseFuncPtr) NULL, 0},	/* -keycode		*/
	{(EventParseFuncPtr) NULL, 0},	/* -keyvector		*/
	{(EventParseFuncPtr) NULL, 0},	/* -majorcode		*/
	{(EventParseFuncPtr) NULL, 0},	/* -messagetype		*/
	{(EventParseFuncPtr) NULL, 0},	/* -minorcode		*/
	{(EventParseFuncPtr) NULL, 0},	/* -mode		*/
	{(EventParseFuncPtr) NULL, 0},	/* -new			*/
	{ParseEventBool, Tk_Offset(XReparentEvent, override_redirect)},
	{(EventParseFuncPtr) NULL, 0},	/* -owner		*/
	{ParseEventWindowId, Tk_Offset(XReparentEvent, parent)},
	{(EventParseFuncPtr) NULL, 0},	/* -place		*/
	{(EventParseFuncPtr) NULL, 0},	/* -property		*/
	{(EventParseFuncPtr) NULL, 0},	/* -request		*/
	{(EventParseFuncPtr) NULL, 0},	/* -requestor		*/
	{(EventParseFuncPtr) NULL, 0},	/* -root		*/
	{(EventParseFuncPtr) NULL, 0},	/* -samescreen		*/
	{(EventParseFuncPtr) NULL, 0},	/* -selection		*/
	{ParseEventBool, Tk_Offset(XReparentEvent, send_event)},
	{ParseEventInt, Tk_Offset(XReparentEvent, serial)},
	{(EventParseFuncPtr) NULL, 0},	/* -state		*/
	{(EventParseFuncPtr) NULL, 0},	/* -subwindow		*/
	{(EventParseFuncPtr) NULL, 0},	/* -target		*/
	{(EventParseFuncPtr) NULL, 0},	/* -time		*/
	{(EventParseFuncPtr) NULL, 0},	/* -width		*/
	{ParseEventWindowId, Tk_Offset(XReparentEvent, window)},
	{ParseEventInt, Tk_Offset(XReparentEvent, x)},
	{(EventParseFuncPtr) NULL, 0},	/* -xroot		*/
	{ParseEventInt, Tk_Offset(XReparentEvent, y)},
	{(EventParseFuncPtr) NULL, 0}	/* -yroot		*/
    },

    /* ResizeRequest */

    {
	{(EventParseFuncPtr) NULL, 0},	/* -above		*/
	{(EventParseFuncPtr) NULL, 0},	/* -atom		*/
	{(EventParseFuncPtr) NULL, 0},	/* -borderwidth		*/
	{(EventParseFuncPtr) NULL, 0},	/* -button		*/
	{(EventParseFuncPtr) NULL, 0},	/* -colormap		*/
	{(EventParseFuncPtr) NULL, 0},	/* -count		*/
	{(EventParseFuncPtr) NULL, 0},	/* -data		*/
	{(EventParseFuncPtr) NULL, 0},	/* -detail		*/
	{(EventParseFuncPtr) NULL, 0},	/* -drawable		*/
	{(EventParseFuncPtr) NULL, 0},	/* -event		*/
	{(EventParseFuncPtr) NULL, 0},	/* -firstkey		*/
	{(EventParseFuncPtr) NULL, 0},	/* -focus		*/
	{(EventParseFuncPtr) NULL, 0},	/* -format		*/
	{(EventParseFuncPtr) NULL, 0},	/* -fromconfigure	*/
	{ParseEventInt, Tk_Offset(XResizeRequestEvent, height)},
	{(EventParseFuncPtr) NULL, 0},	/* -ishint		*/
	{(EventParseFuncPtr) NULL, 0},	/* -keycode		*/
	{(EventParseFuncPtr) NULL, 0},	/* -keyvector		*/
	{(EventParseFuncPtr) NULL, 0},	/* -majorcode		*/
	{(EventParseFuncPtr) NULL, 0},	/* -messagetype		*/
	{(EventParseFuncPtr) NULL, 0},	/* -minorcode		*/
	{(EventParseFuncPtr) NULL, 0},	/* -mode		*/
	{(EventParseFuncPtr) NULL, 0},	/* -new			*/
	{(EventParseFuncPtr) NULL, 0},	/* -overrideredirect	*/
	{(EventParseFuncPtr) NULL, 0},	/* -owner		*/
	{(EventParseFuncPtr) NULL, 0},	/* -parent		*/
	{(EventParseFuncPtr) NULL, 0},	/* -place		*/
	{(EventParseFuncPtr) NULL, 0},	/* -property		*/
	{(EventParseFuncPtr) NULL, 0},	/* -request		*/
	{(EventParseFuncPtr) NULL, 0},	/* -requestor		*/
	{(EventParseFuncPtr) NULL, 0},	/* -root		*/
	{(EventParseFuncPtr) NULL, 0},	/* -samescreen		*/
	{(EventParseFuncPtr) NULL, 0},	/* -selection		*/
	{ParseEventBool, Tk_Offset(XResizeRequestEvent, send_event)},
	{ParseEventInt, Tk_Offset(XResizeRequestEvent, serial)},
	{(EventParseFuncPtr) NULL, 0},	/* -state		*/
	{(EventParseFuncPtr) NULL, 0},	/* -subwindow		*/
	{(EventParseFuncPtr) NULL, 0},	/* -target		*/
	{(EventParseFuncPtr) NULL, 0},	/* -time		*/
	{ParseEventInt, Tk_Offset(XResizeRequestEvent, width)},
	{ParseEventWindowId, Tk_Offset(XResizeRequestEvent, window)},
	{(EventParseFuncPtr) NULL, 0},	/* -x			*/
	{(EventParseFuncPtr) NULL, 0},	/* -xroot		*/
	{(EventParseFuncPtr) NULL, 0},	/* -y			*/
	{(EventParseFuncPtr) NULL, 0}	/* -yroot		*/
    },

    /* SelectionClear */

    {
	{(EventParseFuncPtr) NULL, 0},	/* -above		*/
	{(EventParseFuncPtr) NULL, 0},	/* -atom		*/
	{(EventParseFuncPtr) NULL, 0},	/* -borderwidth		*/
	{(EventParseFuncPtr) NULL, 0},	/* -button		*/
	{(EventParseFuncPtr) NULL, 0},	/* -colormap		*/
	{(EventParseFuncPtr) NULL, 0},	/* -count		*/
	{(EventParseFuncPtr) NULL, 0},	/* -data		*/
	{(EventParseFuncPtr) NULL, 0},	/* -detail		*/
	{(EventParseFuncPtr) NULL, 0},	/* -drawable		*/
	{(EventParseFuncPtr) NULL, 0},	/* -event		*/
	{(EventParseFuncPtr) NULL, 0},	/* -firstkey		*/
	{(EventParseFuncPtr) NULL, 0},	/* -focus		*/
	{(EventParseFuncPtr) NULL, 0},	/* -format		*/
	{(EventParseFuncPtr) NULL, 0},	/* -fromconfigure	*/
	{(EventParseFuncPtr) NULL, 0},	/* -height		*/
	{(EventParseFuncPtr) NULL, 0},	/* -ishint		*/
	{(EventParseFuncPtr) NULL, 0},	/* -keycode		*/
	{(EventParseFuncPtr) NULL, 0},	/* -keyvector		*/
	{(EventParseFuncPtr) NULL, 0},	/* -majorcode		*/
	{(EventParseFuncPtr) NULL, 0},	/* -messagetype		*/
	{(EventParseFuncPtr) NULL, 0},	/* -minorcode		*/
	{(EventParseFuncPtr) NULL, 0},	/* -mode		*/
	{(EventParseFuncPtr) NULL, 0},	/* -new			*/
	{(EventParseFuncPtr) NULL, 0},	/* -overrideredirect	*/
	{(EventParseFuncPtr) NULL, 0},	/* -owner		*/
	{(EventParseFuncPtr) NULL, 0},	/* -parent		*/
	{(EventParseFuncPtr) NULL, 0},	/* -place		*/
	{(EventParseFuncPtr) NULL, 0},	/* -property		*/
	{(EventParseFuncPtr) NULL, 0},	/* -request		*/
	{(EventParseFuncPtr) NULL, 0},	/* -requestor		*/
	{(EventParseFuncPtr) NULL, 0},	/* -root		*/
	{(EventParseFuncPtr) NULL, 0},	/* -samescreen		*/
	{ParseEventAtom, Tk_Offset(XSelectionClearEvent, selection)},
	{ParseEventBool, Tk_Offset(XSelectionClearEvent, send_event)},
	{ParseEventInt, Tk_Offset(XSelectionClearEvent, serial)},
	{(EventParseFuncPtr) NULL, 0},	/* -state		*/
	{(EventParseFuncPtr) NULL, 0},	/* -subwindow		*/
	{(EventParseFuncPtr) NULL, 0},	/* -target		*/
	{ParseEventInt, Tk_Offset(XSelectionClearEvent, time)},
	{(EventParseFuncPtr) NULL, 0},	/* -width		*/
	{ParseEventWindowId, Tk_Offset(XSelectionClearEvent, window)},
	{(EventParseFuncPtr) NULL, 0},	/* -x			*/
	{(EventParseFuncPtr) NULL, 0},	/* -xroot		*/
	{(EventParseFuncPtr) NULL, 0},	/* -y			*/
	{(EventParseFuncPtr) NULL, 0}	/* -yroot		*/
    },

    /* SelectionNotify */

    {
	{(EventParseFuncPtr) NULL, 0},	/* -above		*/
	{(EventParseFuncPtr) NULL, 0},	/* -atom		*/
	{(EventParseFuncPtr) NULL, 0},	/* -borderwidth		*/
	{(EventParseFuncPtr) NULL, 0},	/* -button		*/
	{(EventParseFuncPtr) NULL, 0},	/* -colormap		*/
	{(EventParseFuncPtr) NULL, 0},	/* -count		*/
	{(EventParseFuncPtr) NULL, 0},	/* -data		*/
	{(EventParseFuncPtr) NULL, 0},	/* -detail		*/
	{(EventParseFuncPtr) NULL, 0},	/* -drawable		*/
	{(EventParseFuncPtr) NULL, 0},	/* -event		*/
	{(EventParseFuncPtr) NULL, 0},	/* -firstkey		*/
	{(EventParseFuncPtr) NULL, 0},	/* -focus		*/
	{(EventParseFuncPtr) NULL, 0},	/* -format		*/
	{(EventParseFuncPtr) NULL, 0},	/* -fromconfigure	*/
	{(EventParseFuncPtr) NULL, 0},	/* -height		*/
	{(EventParseFuncPtr) NULL, 0},	/* -ishint		*/
	{(EventParseFuncPtr) NULL, 0},	/* -keycode		*/
	{(EventParseFuncPtr) NULL, 0},	/* -keyvector		*/
	{(EventParseFuncPtr) NULL, 0},	/* -majorcode		*/
	{(EventParseFuncPtr) NULL, 0},	/* -messagetype		*/
	{(EventParseFuncPtr) NULL, 0},	/* -minorcode		*/
	{(EventParseFuncPtr) NULL, 0},	/* -mode		*/
	{(EventParseFuncPtr) NULL, 0},	/* -new			*/
	{(EventParseFuncPtr) NULL, 0},	/* -overrideredirect	*/
	{(EventParseFuncPtr) NULL, 0},	/* -owner		*/
	{(EventParseFuncPtr) NULL, 0},	/* -parent		*/
	{(EventParseFuncPtr) NULL, 0},	/* -place		*/
	{ParseEventAtomOrNone, Tk_Offset(XSelectionEvent, property)},
	{(EventParseFuncPtr) NULL, 0},	/* -request		*/
	{ParseEventWindowId, Tk_Offset(XSelectionEvent, requestor)},
	{(EventParseFuncPtr) NULL, 0},	/* -root		*/
	{(EventParseFuncPtr) NULL, 0},	/* -samescreen		*/
	{ParseEventAtom, Tk_Offset(XSelectionEvent, selection)},
	{ParseEventBool, Tk_Offset(XSelectionEvent, send_event)},
	{ParseEventInt, Tk_Offset(XSelectionEvent, serial)},
	{(EventParseFuncPtr) NULL, 0},	/* -state		*/
	{(EventParseFuncPtr) NULL, 0},	/* -subwindow		*/
	{ParseEventAtom, Tk_Offset(XSelectionEvent, target)},
	{ParseEventInt, Tk_Offset(XSelectionEvent, time)},
	{(EventParseFuncPtr) NULL, 0},	/* -width		*/
	{(EventParseFuncPtr) NULL, 0},	/* -window		*/
	{(EventParseFuncPtr) NULL, 0},	/* -x			*/
	{(EventParseFuncPtr) NULL, 0},	/* -xroot		*/
	{(EventParseFuncPtr) NULL, 0},	/* -y			*/
	{(EventParseFuncPtr) NULL, 0}	/* -yroot		*/
    },

    /* SelectionRequest */

    {
	{(EventParseFuncPtr) NULL, 0},	/* -above		*/
	{(EventParseFuncPtr) NULL, 0},	/* -atom		*/
	{(EventParseFuncPtr) NULL, 0},	/* -borderwidth		*/
	{(EventParseFuncPtr) NULL, 0},	/* -button		*/
	{(EventParseFuncPtr) NULL, 0},	/* -colormap		*/
	{(EventParseFuncPtr) NULL, 0},	/* -count		*/
	{(EventParseFuncPtr) NULL, 0},	/* -data		*/
	{(EventParseFuncPtr) NULL, 0},	/* -detail		*/
	{(EventParseFuncPtr) NULL, 0},	/* -drawable		*/
	{(EventParseFuncPtr) NULL, 0},	/* -event		*/
	{(EventParseFuncPtr) NULL, 0},	/* -firstkey		*/
	{(EventParseFuncPtr) NULL, 0},	/* -focus		*/
	{(EventParseFuncPtr) NULL, 0},	/* -format		*/
	{(EventParseFuncPtr) NULL, 0},	/* -fromconfigure	*/
	{(EventParseFuncPtr) NULL, 0},	/* -height		*/
	{(EventParseFuncPtr) NULL, 0},	/* -ishint		*/
	{(EventParseFuncPtr) NULL, 0},	/* -keycode		*/
	{(EventParseFuncPtr) NULL, 0},	/* -keyvector		*/
	{(EventParseFuncPtr) NULL, 0},	/* -majorcode		*/
	{(EventParseFuncPtr) NULL, 0},	/* -messagetype		*/
	{(EventParseFuncPtr) NULL, 0},	/* -minorcode		*/
	{(EventParseFuncPtr) NULL, 0},	/* -mode		*/
	{(EventParseFuncPtr) NULL, 0},	/* -new			*/
	{(EventParseFuncPtr) NULL, 0},	/* -overrideredirect	*/
	{ParseEventWindowId, Tk_Offset(XSelectionRequestEvent, owner)},
	{(EventParseFuncPtr) NULL, 0},	/* -parent		*/
	{(EventParseFuncPtr) NULL, 0},	/* -place		*/
	{ParseEventAtomOrNone, Tk_Offset(XSelectionRequestEvent, property)},
	{(EventParseFuncPtr) NULL, 0},	/* -request		*/
	{ParseEventWindowId, Tk_Offset(XSelectionRequestEvent, requestor)},
	{(EventParseFuncPtr) NULL, 0},	/* -root		*/
	{(EventParseFuncPtr) NULL, 0},	/* -samescreen		*/
	{ParseEventAtom, Tk_Offset(XSelectionRequestEvent, selection)},
	{ParseEventBool, Tk_Offset(XSelectionRequestEvent, send_event)},
	{ParseEventInt, Tk_Offset(XSelectionRequestEvent, serial)},
	{(EventParseFuncPtr) NULL, 0},	/* -state		*/
	{(EventParseFuncPtr) NULL, 0},	/* -subwindow		*/
	{ParseEventAtom, Tk_Offset(XSelectionRequestEvent, target)},
	{ParseEventInt, Tk_Offset(XSelectionRequestEvent, time)},
	{(EventParseFuncPtr) NULL, 0},	/* -width		*/
	{(EventParseFuncPtr) NULL, 0},	/* -window		*/
	{(EventParseFuncPtr) NULL, 0},	/* -x			*/
	{(EventParseFuncPtr) NULL, 0},	/* -xroot		*/
	{(EventParseFuncPtr) NULL, 0},	/* -y			*/
	{(EventParseFuncPtr) NULL, 0}	/* -yroot		*/
    },

    /* VisibilityNotify */

    {
	{(EventParseFuncPtr) NULL, 0},	/* -above		*/
	{(EventParseFuncPtr) NULL, 0},	/* -atom		*/
	{(EventParseFuncPtr) NULL, 0},	/* -borderwidth		*/
	{(EventParseFuncPtr) NULL, 0},	/* -button		*/
	{(EventParseFuncPtr) NULL, 0},	/* -colormap		*/
	{(EventParseFuncPtr) NULL, 0},	/* -count		*/
	{(EventParseFuncPtr) NULL, 0},	/* -data		*/
	{(EventParseFuncPtr) NULL, 0},	/* -detail		*/
	{(EventParseFuncPtr) NULL, 0},	/* -drawable		*/
	{(EventParseFuncPtr) NULL, 0},	/* -event		*/
	{(EventParseFuncPtr) NULL, 0},	/* -firstkey		*/
	{(EventParseFuncPtr) NULL, 0},	/* -focus		*/
	{(EventParseFuncPtr) NULL, 0},	/* -format		*/
	{(EventParseFuncPtr) NULL, 0},	/* -fromconfigure	*/
	{(EventParseFuncPtr) NULL, 0},	/* -height		*/
	{(EventParseFuncPtr) NULL, 0},	/* -ishint		*/
	{(EventParseFuncPtr) NULL, 0},	/* -keycode		*/
	{(EventParseFuncPtr) NULL, 0},	/* -keyvector		*/
	{(EventParseFuncPtr) NULL, 0},	/* -majorcode		*/
	{(EventParseFuncPtr) NULL, 0},	/* -messagetype		*/
	{(EventParseFuncPtr) NULL, 0},	/* -minorcode		*/
	{(EventParseFuncPtr) NULL, 0},	/* -mode		*/
	{(EventParseFuncPtr) NULL, 0},	/* -new			*/
	{(EventParseFuncPtr) NULL, 0},	/* -overrideredirect	*/
	{(EventParseFuncPtr) NULL, 0},	/* -owner		*/
	{(EventParseFuncPtr) NULL, 0},	/* -parent		*/
	{(EventParseFuncPtr) NULL, 0},	/* -place		*/
	{(EventParseFuncPtr) NULL, 0},	/* -property		*/
	{(EventParseFuncPtr) NULL, 0},	/* -request		*/
	{(EventParseFuncPtr) NULL, 0},	/* -requestor		*/
	{(EventParseFuncPtr) NULL, 0},	/* -root		*/
	{(EventParseFuncPtr) NULL, 0},	/* -samescreen		*/
	{(EventParseFuncPtr) NULL, 0},	/* -selection		*/
	{ParseEventBool, Tk_Offset(XVisibilityEvent, send_event)},
	{ParseEventInt, Tk_Offset(XVisibilityEvent, serial)},
	{ParseVisibilityState, Tk_Offset(XVisibilityEvent, state)},
	{(EventParseFuncPtr) NULL, 0},	/* -subwindow		*/
	{(EventParseFuncPtr) NULL, 0},	/* -target		*/
	{(EventParseFuncPtr) NULL, 0},	/* -time		*/
	{(EventParseFuncPtr) NULL, 0},	/* -width		*/
	{ParseEventWindowId, Tk_Offset(XVisibilityEvent, window)},
	{(EventParseFuncPtr) NULL, 0},	/* -x			*/
	{(EventParseFuncPtr) NULL, 0},	/* -xroot		*/
	{(EventParseFuncPtr) NULL, 0},	/* -y			*/
	{(EventParseFuncPtr) NULL, 0}	/* -yroot		*/
    }
};

/*
 *--------------------------------------------------------------
 *
 * XopParseEvent --
 *
 *	Parses an argument list that describes an X event
 *	structure, and returns the filled-in event structure
 *	and the recipient window.
 *
 * Results:
 *	Returns a standard Tcl result.
 *
 * Side effects:
 *	None.
 *
 *--------------------------------------------------------------
 */

int
XopParseEvent(interp, display, recip, eventPtr, maskPtr, argc, argv)
    Tcl_Interp *interp;	/* Error messages go here	*/
    Display *display;	/* Display for recipient window	*/
    Window recip;	/* Recipient window		*/
    XEvent *eventPtr;	/* Event struct returned here	*/
    long *maskPtr;	/* Event mask returned here	*/
    int argc;		/* Length of arg list		*/
    char **argv;	/* Argument list to parse	*/
{
    int i;
    int argIdx, eventTypeIdx, eventOptIdx;
    int elemOffset;
    ClientData parseAddr;
    EventParseFuncPtr parseFunc;

    /*
     * The event type and recipient window args are mandatory;
     * check the argument count to make sure that at least
     * something is present for these two args.
     */

    if (argc < 1) {
	Tcl_SetResult(interp,
		"Event argument list must contain event type", TCL_STATIC);
	return TCL_ERROR;
    }

    /*
     * First argument is event type. Find its index in the
     * eventNames table, if it's there.
     */

    eventTypeIdx = -1;
    for(i = 0; eventNames[i] != NULL; i++) {
	if (strcmp(argv[0], eventNames[i]) == 0) {
	    eventTypeIdx = i;
	    break;
	}
    }

    if (eventTypeIdx < 0) {
	Tcl_ResetResult(interp);
	Tcl_AppendResult(interp, "Unknown event type \"",
			argv[0], "\"", (char *) 0);
	return TCL_ERROR;
    }

    /* Set the event mask. */

    *maskPtr = eventMaskList[eventTypeIdx];

    /* Set default values. */

    if (defEventProcList[eventTypeIdx] != NULL) {
	(*(defEventProcList[eventTypeIdx]))(display, recip, eventPtr);
    }

    /* Parse the remaining args to fill in the struct. */

    for(argIdx = 1; argIdx < argc; argIdx++) {
	/* Look up the option name. */

	eventOptIdx = -1;
	for(i = 0; eventOptNames[i] != NULL; i++) {
	    if (strcmp(argv[argIdx], eventOptNames[i]) == 0) {
		eventOptIdx = i;
		break;
	    }
	}
	if (eventOptIdx < 0) {
	    Tcl_AppendResult(interp, "Unknown option \"",
			argv[argIdx], "\"", (char *) 0);
	    return TCL_ERROR;
	}

	/* Make sure there's a value here to parse. */

	argIdx++;
	if (argIdx >= argc) {
	    Tcl_AppendResult(interp, "Option \"", argv[argIdx - 1],
			"\" not followed by value", (char *) 0);
	    return TCL_ERROR;
	}

	/* Make sure that this is a valid option for the event type. */

	parseFunc = eventOptParseTab[eventTypeIdx][eventOptIdx].func;
	if (parseFunc == NULL) {
	    Tcl_AppendResult(interp, "Option \"", argv[argIdx - 1],
			"\" s not valid for event type \"",
			argv[0], "\"", (char *) 0);
	    return TCL_ERROR;
	}

	/* Call the function for parsing the value. */

	elemOffset = eventOptParseTab[eventTypeIdx][eventOptIdx].offset;
	if ((*parseFunc)(interp, display,
			argv[argIdx], eventPtr, elemOffset) != TCL_OK) {
	    return TCL_ERROR;
	}

    }

    /* Done! */

    return TCL_OK;
}

