/* 
 * pointer.c --
 *
 *	This file implements xop's "pointer" subcommand.
 */

/*
 * Copyright (C) 1998-2001, Neil McKay
 */

#include "tk.h"

/* Include all the X11 stuff we need. */

#include <X11/Xutil.h>

#include "xopInt.h"


/*
 *--------------------------------------------------------------
 *
 * DoubleToRational --
 *
 *	Converts a (non-negative) double to a numerator and
 *	denominator which, when divided, will give back the
 *	original floating-point number to within a given
 *	accuracy.
 *
 * Results:
 *	None.
 *
 * Side effects:
 *	None.
 *
 *--------------------------------------------------------------
 */

static void
DoubleToRational(dbl, eps, ratNumPtr, ratDenomPtr)
    double dbl;		/* Number to convert to rational	*/
    double eps;		/* Accuracy of conversion		*/
    int *ratNumPtr;	/* Numerator returned here		*/
    int *ratDenomPtr;	/* Denominator returned here		*/
{
    if (eps >= 1) {
	*ratDenomPtr = 1;
	*ratNumPtr = (dbl + 0.5);
	return;
    }

    *ratDenomPtr = 1 + 1.0/eps;
    *ratNumPtr = 1 + (dbl*(*ratDenomPtr)); 

    /*
     *@@@ Should put in some code to try to simplify
     *@@@ this fraction. We'll do this when we have time...
     */
}


/*
 *--------------------------------------------------------------
 *
 * XopPointerCommand --
 *
 *	Implements the "pointer" Xop subcommand. See the Xop
 *	man page for a description.
 *
 * Results:
 *	A standard Tcl result.
 *
 * Side effects:
 *	See the user documentation.
 *
 *--------------------------------------------------------------
 */

static struct {
    char *name;
    unsigned int mask;
} EventConvList[] = {
    {"ButtonPress",	ButtonPressMask},
    {"ButtonRelease",	ButtonReleaseMask},
    {"EnterWindow",	EnterWindowMask},
    {"LeaveWindow",	LeaveWindowMask},
    {"PointerMotion",	PointerMotionMask},
    {"PointerMotionHint",	PointerMotionHintMask},
    {"Button1Motion",	Button1MotionMask},
    {"Button2Motion",	Button2MotionMask},
    {"Button3Motion",	Button3MotionMask},
    {"Button4Motion",	Button4MotionMask},
    {"Button5Motion",	Button5MotionMask},
    {"ButtonMotion",	ButtonMotionMask},
    {"KeymapState",	KeymapStateMask},
    {NULL,	0}
};

static struct {
    char *name;
    unsigned int mask;
} ModConvList[] = {
    {"Shift",	ShiftMask},
    {"Lock",	LockMask},
    {"Control",	ControlMask},
    {"Mod1",	Mod1Mask},
    {"Mod2",	Mod2Mask},
    {"Mod3",	Mod3Mask},
    {"Mod4",	Mod4Mask},
    {"Mod5",	Mod5Mask},
    {"Any",	AnyModifier},
    {NULL,	0}
};

static struct {
    char *name;
    unsigned int mask;
} pointerModList[] = {
    {"Shift",	ShiftMask},
    {"Lock",	LockMask},
    {"Control",	ControlMask},
    {"Mod1",	Mod1Mask},
    {"Mod2",	Mod2Mask},
    {"Mod3",	Mod3Mask},
    {"Mod4",	Mod4Mask},
    {"Mod5",	Mod5Mask},
    {"Button1",	Button1Mask},
    {"Button2",	Button2Mask},
    {"Button3",	Button3Mask},
    {"Button4",	Button4Mask},
    {"Button5",	Button5Mask},
    {NULL,	0}
};
static unsigned int buttonMap[] = {
    0,
    Button1,
    Button2,
    Button3,
    Button4,
    Button5
};

int
XopPointerCommand(interp, argc, argv, opts)
    Tcl_Interp *interp;	/* Result goes here	*/
    int argc;		/* Argument count	*/
    char **argv;	/* Argument vector	*/
    XopOptions *opts;	/* Options		*/
{
    double accel;
    int accelNum, accelDenom, accelThresh;
    char junkStr[32];
    int i,j, result;
    int nModifiers;
    char **modList;
    unsigned int button, modifiers;
    Window grabWindow, confineTo;
    Window dummyWin1, dummyWin2;
    int ownerEvents;
    int nEvents;
    char **eventList;
    unsigned int eventMask;
    int num, denom;
    int nButtons, buttonNo;
    unsigned char junkMap[5];
    unsigned char *pointerMap;
    int nMapSettings;
    char **pointerMapList;
    int rootX, rootY, winX, winY;
    unsigned int modMask;
    int nCoords;
    char **coordList;

    if (argc < 3) {
	Tcl_AppendResult(interp, "Usage: ", argv[0], " ", argv[1],
		" pointerCommand, where pointerCommand may be:\n",
		"	acceleration, accelthresh, grab, grabbutton, mapping,",
		"	position, state, ungrab, or ungrabbutton",
							(char *) NULL);
	return TCL_ERROR;
    }

    if (strcmp(argv[2], "acceleration") == 0) {
	if (argc > 4) {
	    Tcl_AppendResult(interp, "Usage: ", argv[0], " ",
			argv[1], " ", argv[2], " ?accelValue?", (char *) 0);
	    return TCL_ERROR;
	}

	if (argc == 3) {
	    XGetPointerControl(opts->display,
				&accelNum, &accelDenom, &accelThresh);
	    sprintf(junkStr, "%17.9g", ((double) accelNum)/accelDenom);
	    Tcl_SetResult(interp, junkStr, TCL_VOLATILE);
	    return TCL_OK;
	}

	/* Get the acceleration value. */

	if (Tcl_GetDouble(interp, argv[3], &accel) != TCL_OK) {
	    return TCL_ERROR;
	}

	/* Convert it to a rational number. */

	DoubleToRational(accel, 0.001, &num, &denom);

	/* Set the acceleration numbers for the pointer. */

	XChangePointerControl(opts->display, 1, 0, num, denom, 1);
	return TCL_OK;
    }

    if (strcmp(argv[2], "accelthresh") == 0) {
	if (argc > 4) {
	    Tcl_AppendResult(interp, "Usage: ", argv[0], " ",
			argv[1], " ", argv[2], " ?accelThresh?", (char *) 0);
	    return TCL_ERROR;
	}

	if (argc == 3) {
	    XGetPointerControl(opts->display,
				&accelNum, &accelDenom, &accelThresh);
	    sprintf(junkStr, "%1d", accelThresh);
	    Tcl_SetResult(interp, junkStr, TCL_VOLATILE);
	    return TCL_OK;
	}

	/* Get the acceleration threshold. */

	if (Tcl_GetInt(interp, argv[3], &accelThresh) != TCL_OK) {
	    return TCL_ERROR;
	}

	/* Set the acceleration threshold for the pointer. */

	XChangePointerControl(opts->display, 0, 1, 1, 1, accelThresh);
	return TCL_OK;
    }

    if (strcmp(argv[2], "grab") == 0) {
	if (argc < 5) {
	    Tcl_AppendResult(interp, "Usage: ", argv[0], " ", argv[1], " ",
			argv[2], " eventList grabWindow ?options?",
			(char *) 0);
	    return TCL_ERROR;
	}

	/* Parse the event list. */

	if (Tcl_SplitList(interp, argv[3], &nEvents, &eventList) != TCL_OK) {
	    return TCL_ERROR;
	}

	eventMask = 0;
	for(i = 0; i < nEvents; i++) {
	    for (j = 0; EventConvList[j].name != NULL; j++) {
		if (strcmp(eventList[i],  EventConvList[j].name) == 0) {
		    eventMask |=  EventConvList[j].mask;
		    break;
		}
	    }
	    if (EventConvList[j].name == NULL) {
		Tcl_AppendResult(interp, "Unknown event name \"",
			eventList[i], "\"", (char *) 0);
		ckfree((VOID *) eventList);
		return TCL_ERROR;
	    }
	}
	ckfree((VOID *) eventList);

	/* Get the ID of the grab window. */

	result = XopGetAndCheckWindowId(interp,
			opts->display, argv[4], &grabWindow);
	if(result != TCL_OK) {
	    return TCL_ERROR;
	}

	/* Parse options, if any. */

	ownerEvents = 0;
	confineTo = None;
	i = 5;
	while(i < argc) {
	    if(strcmp(argv[i], "-confineto") == 0) {
		i += 1;
		if (i >= argc) {
		    Tcl_SetResult(interp,
			    "-confineto must be followed by a window ID",
			    TCL_STATIC);
		    return TCL_ERROR;
		}

		result = XopGetAndCheckWindowId(interp,
				opts->display, argv[i], &confineTo);
		if(result != TCL_OK) {
		    return TCL_ERROR;
		}
	    } else if (strcmp(argv[i], "-ownerevents") == 0) {
		i += 1;
		if (i >= argc) {
		    Tcl_SetResult(interp,
			    "-ownerevents must be followed by a Boolean value",
			    TCL_STATIC);
		    return TCL_ERROR;
		}

		if(Tcl_GetBoolean(interp, argv[i], &ownerEvents) != TCL_OK) {
		    return TCL_ERROR;
		}
	    } else {
		Tcl_AppendResult(interp, "Unknown option \"",
			argv[i], "\"; must be -confineto or -ownerevents");
	    }
	    i += 1;
	}

	/* Grab... */

	XGrabPointer(opts->display, grabWindow, ownerEvents, eventMask,
			GrabModeAsync, GrabModeAsync, confineTo,
			None, CurrentTime);
	return TCL_OK;
    }

    if (strcmp(argv[2], "grabbutton") == 0) {
	if (argc < 6) {
	    Tcl_AppendResult(interp, "Usage: ", argv[0], " ", argv[1], " ",
			argv[2], "button eventList grabWindow ?options?",
			(char *) 0);
	    return TCL_ERROR;
	}

	/* Get the button number. */

	if (strcmp(argv[3], "Any") == 0) {
	    button = AnyButton;
	} else {
	    if (strlen(argv[3]) != 1 || argv[3][0] < '1' || argv[3][0] > '5') {
		Tcl_SetResult(interp,
			"button name must be 1, 2, 3, 4, 5, or \"Any\"",
			TCL_STATIC);
		return TCL_ERROR;
	    }
	    button = buttonMap[argv[3][0] - '1' + 1];
	}

	/* Parse the event list. */

	if (Tcl_SplitList(interp, argv[4], &nEvents, &eventList) != TCL_OK) {
	    return TCL_ERROR;
	}

	eventMask = 0;
	for(i = 0; i < nEvents; i++) {
	    for (j = 0; EventConvList[j].name != NULL; j++) {
		if (strcmp(eventList[i],  EventConvList[j].name) == 0) {
		    eventMask |=  EventConvList[j].mask;
		    break;
		}
	    }
	    if (EventConvList[j].name == NULL) {
		Tcl_AppendResult(interp, "Unknown event name \"",
			eventList[i], "\"", (char *) 0);
		ckfree((VOID *) eventList);
		return TCL_ERROR;
	    }
	}
	ckfree((VOID *) eventList);

	/* Get the ID of the grab window. */

	result = XopGetAndCheckWindowId(interp,
			opts->display, argv[5], &grabWindow);
	if(result != TCL_OK) {
	    return TCL_ERROR;
	}

	/* Parse options, if any. */

	ownerEvents = 0;
	confineTo = None;
	modifiers = AnyModifier;
	i = 6;
	while(i < argc) {
	    if(strcmp(argv[i], "-confineto") == 0) {
		i += 1;
		if (i >= argc) {
		    Tcl_SetResult(interp,
			    "-confineto must be followed by a window ID",
			    TCL_STATIC);
		    return TCL_ERROR;
		}

		result = XopGetAndCheckWindowId(interp,
				opts->display, argv[i], &confineTo);
		if(result != TCL_OK) {
		    return TCL_ERROR;
		}
	    } else if (strcmp(argv[i], "-ownerevents") == 0) {
		i += 1;
		if (i >= argc) {
		    Tcl_SetResult(interp,
			    "-ownerevents must be followed by a Boolean value",
			    TCL_STATIC);
		    return TCL_ERROR;
		}

		if(Tcl_GetBoolean(interp, argv[i], &ownerEvents) != TCL_OK) {
		    return TCL_ERROR;
		}
	    } else if (strcmp(argv[i], "-modifiers") == 0) {
		i += 1;
		if (Tcl_SplitList(interp, argv[i], &nModifiers, &modList) != TCL_OK) {
		    return TCL_ERROR;
		}


		modifiers = 0;
		for(i = 0; i < nEvents; i++) {
		    for (j = 0; ModConvList[j].name != NULL; j++) {
			if (strcmp(modList[i],  ModConvList[j].name) == 0) {
			    modifiers |=  ModConvList[j].mask;
			    break;
			}
		    }
		    if (ModConvList[j].name == NULL) {
			Tcl_AppendResult(interp, "Unknown modifier name \"",
				modList[i], "\"", (char *) 0);
			ckfree((VOID *) modList);
			return TCL_ERROR;
		    }
		}
		ckfree((VOID *) modList);
	    } else {
		Tcl_AppendResult(interp, "Unknown option \"",
			argv[i], "\"; must be -confineto, -ownerevents, or -modifiers");
	    }
	    i += 1;
	}

	/* Grab... */

	XGrabButton(opts->display, button, modifiers, grabWindow,
			ownerEvents, eventMask,
			GrabModeAsync, GrabModeAsync, confineTo,
			None);
	return TCL_OK;
    }

    if (strcmp(argv[2], "mapping") == 0) {
	if (argc > 4) {
	    Tcl_AppendResult(interp, "Usage: ", argv[0], " ",
			argv[1], " ", argv[2], " ?buttonMap?", (char *) 0);
	    return TCL_ERROR;
	}

	if (argc == 3) {
	    /* Call XGetPointerMapping once to get number of buttons */

	    nButtons = XGetPointerMapping(opts->display, junkMap, 5);

	    /* Call it again to get pointer mapping. */

	    pointerMap = (unsigned char *) ckalloc(nButtons*sizeof(unsigned char));
	    if (pointerMap == (unsigned char *) NULL) {
		Tcl_SetResult(interp, "out of memory", TCL_STATIC);
		return TCL_ERROR;
	    }

	    XGetPointerMapping(opts->display, pointerMap, nButtons);
	    for (i = 0; i < nButtons; i++) {
		sprintf(junkStr, "%1d", pointerMap[i]);
		Tcl_AppendElement(interp, junkStr);
	    }

	    ckfree((VOID *) pointerMap);
	    return TCL_OK;
	}

	/* Must be setting the pointer map; get it. */

	if (Tcl_SplitList(interp, argv[3], &nMapSettings, &pointerMapList) != TCL_OK) {
	    return TCL_ERROR;
	}

	pointerMap = (unsigned char *) ckalloc(nMapSettings*sizeof(unsigned char));
	if (pointerMap == (unsigned char *) NULL) {
	    ckfree((VOID *) pointerMapList);
	    Tcl_SetResult(interp, "out of memory", TCL_STATIC);
	    return TCL_ERROR;
	}

	for (i = 0; i < nMapSettings; i++) {
	    if(Tcl_GetInt(interp, pointerMapList[i], &buttonNo) != TCL_OK) {
		ckfree((VOID *) pointerMapList);
		ckfree((VOID *) pointerMap);
		return TCL_ERROR;
	    }

	    if (buttonNo < 0 || buttonNo > 255) {
		ckfree((VOID *) pointerMapList);
		ckfree((VOID *) pointerMap);
		Tcl_SetResult(interp, "Button number must be >= 0 and <= 255",
				TCL_STATIC);
		return TCL_ERROR;
	    }

	    pointerMap[i] = buttonNo;
	}
	ckfree((VOID *) pointerMapList);

	/* Make sure we have no repeats in the list. */

	for(i = 0; i < nMapSettings; i++) {
	    for(j = i + 1; j < nMapSettings; j++) {
		if (pointerMap[i] == pointerMap[j]) {
		    ckfree((VOID *) pointerMap);
		    Tcl_SetResult(interp,
			"pointer button map contains duplicate entries",
								TCL_STATIC);
		    return TCL_ERROR;
		}
	    }
	}

	/* Change the pointer map. */

	XSetPointerMapping(opts->display, pointerMap, nMapSettings);

	/* Clean up and return. */

	ckfree((VOID *) pointerMap);
	return TCL_OK;
    }

    if (strcmp(argv[2], "position") == 0) {
	if (argc > 4) {
	    Tcl_AppendResult(interp, "Usage: ", argv[0], " ",
			argv[1], " ", argv[2], " ?pos?", (char *) 0);
	    return TCL_ERROR;
	}

	if (argc == 3) {
	    XQueryPointer(opts->display,
			XRootWindow(opts->display, opts->screenNumber),
			&dummyWin1, &dummyWin2, &rootX, &rootY,
			&winX, &winY, &modMask);

	    sprintf(junkStr, "%1d", rootX);
	    Tcl_AppendElement(interp, junkStr);
	    sprintf(junkStr, "%1d", rootY);
	    Tcl_AppendElement(interp, junkStr);

	    return TCL_OK;
	}

	/* Extract the desired position from the argument list. */

	if (Tcl_SplitList(interp, argv[3], &nCoords, &coordList) != TCL_OK) {
	    return TCL_ERROR;
	}

	if (nCoords != 2) {
	    Tcl_SetResult(interp,
		    "Must have 2 coordinates when setting pointer position",
								TCL_STATIC);
	    ckfree((VOID *) coordList);
	    return TCL_ERROR;
	}

	if(Tcl_GetInt(interp, coordList[0], &rootX) != TCL_OK) {
	    ckfree((VOID *) coordList);
	    return TCL_ERROR;
	}

	if(Tcl_GetInt(interp, coordList[1], &rootY) != TCL_OK) {
	    ckfree((VOID *) coordList);
	    return TCL_ERROR;
	}

	ckfree((VOID *) coordList);
	XWarpPointer(opts->display, None,
			XRootWindow(opts->display, opts->screenNumber),
			0, 0, 0, 0, rootX, rootY);
	return TCL_OK;
    }

    if (strcmp(argv[2], "state") == 0) {
	if (argc != 3) {
	    Tcl_AppendResult(interp, "Usage: ", argv[0], " ",
			argv[1], " ", argv[2], (char *) 0);
	    return TCL_ERROR;
	}

	XQueryPointer(opts->display,
			XRootWindow(opts->display, opts->screenNumber),
			&dummyWin1, &dummyWin2, &rootX, &rootY,
			&winX, &winY, &modMask);

	for (i = 0; pointerModList[i].name != NULL; i++) {
	    if (modMask & pointerModList[i].mask) {
		Tcl_AppendElement(interp, pointerModList[i].name);
	    }
	}

	return TCL_OK;
    }

    if (strcmp(argv[2], "ungrab") == 0) {
	if (argc > 3) {
	    Tcl_AppendResult(interp, "Usage: ", argv[0], " ",
			argv[1], " ", argv[2], (char *) 0);
	    return TCL_ERROR;
	}

	XUngrabPointer(opts->display, CurrentTime);
	return TCL_OK;
    }

    if (strcmp(argv[2], "ungrabbutton") == 0) {
	if (argc < 5) {
	    Tcl_AppendResult(interp, "Usage: ", argv[0], " ",
			argv[1], " ", argv[2],
			" button grabWindow ?-modifiers modList?",
								(char *) 0);
	    return TCL_ERROR;
	}

	/* Get the button number. */

	if (strcmp(argv[3], "Any") == 0) {
	    button = AnyButton;
	} else {
	    if (strlen(argv[3]) != 1 || argv[3][0] < '1' || argv[3][0] > '5') {
		Tcl_SetResult(interp,
			"button name must be 1, 2, 3, 4, 5, or \"Any\"",
			TCL_STATIC);
		return TCL_ERROR;
	    }
	    button = buttonMap[argv[3][0] - '1' + 1];
	}

	/* Get the ID of the grab window. */

	result = XopGetAndCheckWindowId(interp,
			opts->display, argv[4], &grabWindow);
	if(result != TCL_OK) {
	    return TCL_ERROR;
	}

	/* Parse options, if any. */

	ownerEvents = 0;
	confineTo = None;
	modifiers = AnyModifier;
	i = 5;
	while(i < argc) {
	    if (strcmp(argv[i], "-modifiers") == 0) {
		i += 1;
		if (Tcl_SplitList(interp, argv[i], &nModifiers, &modList) != TCL_OK) {
		    return TCL_ERROR;
		}


		modifiers = 0;
		for(i = 0; i < nEvents; i++) {
		    for (j = 0; ModConvList[j].name != NULL; j++) {
			if (strcmp(modList[i],  ModConvList[j].name) == 0) {
			    modifiers |=  ModConvList[j].mask;
			    break;
			}
		    }
		    if (ModConvList[j].name == NULL) {
			Tcl_AppendResult(interp, "Unknown modifier name \"",
				modList[i], "\"", (char *) 0);
			ckfree((VOID *) modList);
			return TCL_ERROR;
		    }
		}
		ckfree((VOID *) modList);
	    } else {
		Tcl_AppendResult(interp, "Unknown option \"",
			argv[i], "\"; must be -modifiers");
	    }
	    i += 1;
	}


	XUngrabButton(opts->display, button, modifiers, grabWindow); 
	return TCL_OK;
    }

    Tcl_AppendResult(interp, "unknown pointer command \"", argv[2],
		"\"; allowed commands are:\n",
		"	acceleration, accelthresh, grab, mapping,",
		"	position, state, and ungrab", (char *) NULL);
    return TCL_ERROR;
}

