/* 
 * utils.c --
 *
 *	This file provides some basic utilities used by Xop:
 *	converting Drawable Id's to/from text form, getting
 *	window/drawable attributes, etc.
 */

/*
 * Copyright (C) 1998-2001, Neil D. McKay
 */

#include "tk.h"
#include <X11/Xproto.h>

#include "xopInt.h"

/*
 *----------------------------------------------------------------------
 *
 * XopGetDrawableId --
 *
 *	XopGetDrawableId converts a string to an X drawable ID.
 *
 * Results:
 *	Returns a standard Tcl result.
 *
 * Side effects:
 *	None.
 *
 *----------------------------------------------------------------------
 */

int
XopGetDrawableId(interp, str, drawablePtr)
    Tcl_Interp *interp; /* Error messages go here */
    char *str;          /* String to convert to drawable ID */
    Drawable *drawablePtr;  /* Drawable ID goes in here */
{
    int i, result;

    i = (int) None;
    result = Tcl_GetInt(interp,str,&i);
    *drawablePtr = (Drawable) i;
    return result;
}

/*
 *----------------------------------------------------------------------
 *
 * XopAppendDrawableId --
 *
 *	XopAppendDrawableId appends a string representation of
 *	an X drawable ID onto the interpreter's result string.
 *
 * Results:
 *	None.
 *
 * Side effects:
 *	None.
 *
 *----------------------------------------------------------------------
 */

void
XopAppendDrawableId(interp,drawable)
    Tcl_Interp *interp;
    Drawable drawable;
{
    char junkStr[64];

    sprintf(junkStr,"0x%08x", (unsigned int) drawable);
    Tcl_AppendElement(interp,junkStr);
}


/*
 *--------------------------------------------------------------
 *
 * XopGetWindowAttributes --
 *	Get window attributes, if window exists. This is a
 *	wrapper for XGetWindowAttributes. (We need to
 *	install our own dummy error handler.)
 *
 * Returns:
 *	Zero on failure (e.g. window doesn't exist),
 *	non-zero on success
 *
 * Side effects:
 *	None.
 *
 *--------------------------------------------------------------
 */

int
XopGetWindowAttributes(display,win,attrPtr)
    Display *display;
    Window win;
    XWindowAttributes *attrPtr;
{
    Tk_ErrorHandler handler;
    Status status;
    int gotXError;

    gotXError = 0;
    handler = Tk_CreateErrorHandler(display,
		-1, -1, -1, XopXErrorHandler,
		(ClientData) (&gotXError));
    status = XGetWindowAttributes(display,win,attrPtr);
    Tk_DeleteErrorHandler(handler);
    XSync(display, False);
    return status;
}

/*
 *--------------------------------------------------------------
 *
 * XopGetDrawableGeometry --
 *	Get drawable gemoetry, if drawable exists. This is a
 *	wrapper for XGetGeometry. (We need to install our own
 *	dummy error handler.)
 *
 * Returns:
 *	Zero on failure (e.g. drawable doesn't exist),
 *	non-zero on success
 *
 * Side effects:
 *	None.
 *
 *--------------------------------------------------------------
 */

int
XopGetDrawableGeometry(display,drawable,geomPtr)
    Display *display;
    Drawable drawable;
    XopDrawableGeometry *geomPtr;
{
    Tk_ErrorHandler handler;
    int gotXError;
    Status status;

    gotXError = 0;
    handler = Tk_CreateErrorHandler(display, -1, -1, -1,
		XopXErrorHandler, (ClientData) (&gotXError));
    status = XGetGeometry(display, drawable, &(geomPtr->rootwin),
		&(geomPtr->x),&(geomPtr->y),
		&(geomPtr->width),&(geomPtr->height),
		&(geomPtr->border_width),&(geomPtr->depth));
    Tk_DeleteErrorHandler(handler);
    XSync(display, False);
    return status;
}

/*
 *--------------------------------------------------------------
 *
 * XopQueryColor --
 *	Get a color out of a colormap. This is just a
 *	wrapper for XQueryColor. (We need to install our own
 *	dummy error handler.)
 *
 * Returns:
 *	Zero on success, non-zero on failure (e.g. colormap
 *	doesn't exist)
 *
 * Side effects:
 *	None.
 *
 *--------------------------------------------------------------
 */

int
XopQueryColor(display, colormap, colorPtr)
    Display *display;	/* Display on which colormap resides	*/
    Colormap colormap;	/* Colormap to look up color in		*/
    XColor *colorPtr;	/* Pixel to look up color for, and
			 * location where we return RGB values	*/
{
    Tk_ErrorHandler handler;
    int gotXError;

    gotXError = 0;
    handler = Tk_CreateErrorHandler(display, -1, -1, -1,
		XopXErrorHandler, (ClientData) (&gotXError));
    XQueryColor(display, colormap, colorPtr);
    Tk_DeleteErrorHandler(handler);
    XSync(display, False);
    return gotXError;
}

/*
 *--------------------------------------------------------------
 *
 * XopQueryColors --
 *	Get several colors out of a colormap. This is just a
 *	wrapper for XQueryColors. (We need to install our own
 *	dummy error handler.)
 *
 * Returns:
 *	Zero on success, non-zero on failure (e.g. colormap
 *	doesn't exist)
 *
 * Side effects:
 *	None.
 *
 *--------------------------------------------------------------
 */

int
XopQueryColors(display, colormap, colorPtr, nColors)
    Display *display;	/* Display on which colormap resides	*/
    Colormap colormap;	/* Colormap to look up color in		*/
    XColor *colorPtr;	/* Pixels to look up color for, and
			 * locations where we return RGB values	*/
    int nColors;	/* Number of colors to look up		*/
{
    Tk_ErrorHandler handler;
    int gotXError;

    gotXError = 0;
    handler = Tk_CreateErrorHandler(display, -1, -1, -1,
		XopXErrorHandler, (ClientData) (&gotXError));
    XQueryColors(display, colormap, colorPtr, nColors);
    Tk_DeleteErrorHandler(handler);
    XSync(display, False);
    return gotXError;
}

/*
 *--------------------------------------------------------------
 *
 * XopIsValidWindow --
 *
 *	Determines whether a given window ID is valid on a
 *	particular display, i.e. whether it is an XID for
 *	a window.
 *
 * Results:
 *	Returns an int.
 *
 * Side effects:
 *	None.
 *
 *--------------------------------------------------------------
 */

int
XopIsValidWindow(display, window)
    Display *display;
    Window window;
{
    XWindowAttributes attr;

    return(XopGetWindowAttributes(display, window, &attr));
}

/*
 *--------------------------------------------------------------
 *
 * XopIsValidDrawable --
 *
 *	Determines whether a given drawable ID is valid on a
 *	particular display, i.e. whether it is an XID for
 *	a drawable (window or pixmap).
 *
 * Results:
 *	Returns an int.
 *
 * Side effects:
 *	None.
 *
 *--------------------------------------------------------------
 */

int
XopIsValidDrawable(display, drawable)
    Display *display;
    Drawable drawable;
{
    XopDrawableGeometry geom;

    return(XopGetDrawableGeometry(display, drawable, &geom));
}


/*
 *--------------------------------------------------------------
 *
 * XopIsValidColormap --
 *
 *	Determines whether a given colormap ID is valid on a
 *	particular display
 *
 * Results:
 *	Returns an int.
 *
 * Side effects:
 *	None.
 *
 *--------------------------------------------------------------
 */

int
XopIsValidColormap(display, colormap)
    Display *display;
    Colormap colormap;
{
    XColor color;

    color.pixel = 0;
    color.flags = (DoRed|DoGreen|DoBlue);
    return(!XopQueryColor(display, colormap, &color));
}

/*
 *--------------------------------------------------------------
 *
 * XopGetAndCheckWindowId --
 *
 *	Gets a window ID from a string, and makes sure
 *	that the window exists on the given display.
 *
 * Results:
 *	Returns a standard Tcl result.
 *
 * Side effects:
 *	None.
 *
 *--------------------------------------------------------------
 */

int
XopGetAndCheckWindowId(interp, display, str, windowPtr)
    Tcl_Interp *interp;
    Display *display;
    char *str;
    Window *windowPtr;
{
    int result;

    /* Get the window ID, and make sure that it exists. */

    result = XopGetDrawableId(interp,str,(Drawable *) windowPtr);
    if (result != TCL_OK) {
	return TCL_ERROR;
    }

    if (XopIsValidWindow(display, *windowPtr)) {
	return TCL_OK;
    }

    Tcl_AppendResult(interp, "No window ", str, " on display", (char *) 0);
    return TCL_ERROR;
}


/*
 *--------------------------------------------------------------
 *
 * XopGetAndCheckDrawableId --
 *
 *	Gets a drawable ID from a string, and makes sure
 *	that it's a valid drawable ID for the given display.
 *
 * Results:
 *	Returns a standard Tcl result.
 *
 * Side effects:
 *	None.
 *
 *--------------------------------------------------------------
 */

int
XopGetAndCheckDrawableId(interp, display, str, drawablePtr)
    Tcl_Interp *interp;
    Display *display;
    char *str;
    Drawable *drawablePtr;
{
    int result;

    /* Get the drawable ID, and make sure that it exists. */

    result = XopGetDrawableId(interp,str,drawablePtr);
    if (result != TCL_OK) {
	return TCL_ERROR;
    }

    if (XopIsValidDrawable(display, *drawablePtr)) {
	return TCL_OK;
    }
    Tcl_AppendResult(interp, "No drawable", str, " on display", (char *) 0);
    return TCL_ERROR;
}


/*
 *--------------------------------------------------------------
 *
 * XopGetAndCheckColormapId --
 *
 *	Gets a colormap ID from a string, and makes sure that
 *	the colormap exists on the given display.
 *
 * Results:
 *	Returns a standard Tcl result.
 *
 * Side effects:
 *	None.
 *
 *--------------------------------------------------------------
 */

int
XopGetAndCheckColormapId(interp, display, str, colormapPtr)
    Tcl_Interp *interp;
    Display *display;
    char *str;
    Colormap *colormapPtr;
{
    int result;

    /* Get the colormap ID, and make sure that it exists. */

    result = Tcl_GetInt(interp, str, (int *) colormapPtr);
    if (result != TCL_OK) {
	return TCL_ERROR;
    }

    if (XopIsValidColormap(display, *colormapPtr)) {
	return TCL_OK;
    }
    Tcl_AppendResult(interp, "No colormap ", str, " on display", (char *) 0);
    return TCL_ERROR;
}


/*
 *--------------------------------------------------------------
 *
 * XopParentXId --
 *
 *	Returns the parent of the given window, or "None"
 *	if the window doesn't exist.
 *
 * Results:
 *	Returns an X window ID.
 *
 * Side effects:
 *	None.
 *
 *--------------------------------------------------------------
 */

Window
XopParentXId(display, w)
    Display *display;
    Window w;
{
    Tk_ErrorHandler handler;
    int gotXError;
    Status status;
    Window parent;
    Window root;
    Window *childList;
    unsigned int nChildren;

    /* Handle errors ourselves. */

    gotXError = 0;
    handler = Tk_CreateErrorHandler(display, -1, -1, -1,
		XopXErrorHandler, (ClientData) (&gotXError));

    /* Get the parent window. */

    status = XQueryTree(display, w, &root, &parent, &childList, &nChildren);

    /* Do some cleanup; gotta return "None" if we got an error. */

    Tk_DeleteErrorHandler(handler);
    XSync(display, False);
    XFree(childList);
    if (status == 0) {
	parent = None;
    }

    return parent;
}

/*
 *--------------------------------------------------------------
 *
 * XopXErrorHandler --
 *
 *	XopXErrorHandler is an error handler, to be installed
 *	via Tk_CreateErrorHandler, that will set a flag if an
 *	X error occurred.
 *
 * Results:
 *	Always returns 0, indicating that the X error was
 *	handled.
 *
 * Side effects:
 *	None.
 *
 *--------------------------------------------------------------
 */

int
XopXErrorHandler (clientData, errEventPtr)
    ClientData clientData;	/* Pointer to flag we set	*/
    XErrorEvent *errEventPtr;	/* X error info			*/
{
    int *error;

    error = (int *) clientData;
    *error = 1;
    return 0;
}

